/* Copyright (C) 2000 KOM/Darmstadt University of Technology
 *
 * You are allowed to use all other parts of the code under the following terms:
 *
 * For non-commercial use, code may be used in unmodified form provided
 * that this copyright notice and this permission notice appear in
 * supporting documentation.
 *
 * This software is provided "as is" and without any express or implied
 * warranties, including, without limitation, the implied warranty of
 * fitness for a particular purpose.
 *
 * The code may be subjected to the GNU General Public License, Version 2,
 * and re-distributed under the terms of this license.
 *
 * Commercial use other than under the terms of the GNU General Public
 * License is allowed only after express negotiation of conditions
 * with the authors.
 */





#include 	"Conf.h"
#include 	"Log.h"

#include <sys/socket.h> /* inet_aton */
#include <netinet/in.h> /* inet_aton */
#include <arpa/inet.h> /* inet_aton */
#include <string.h>	/* strncmp */


Conf::Conf(String confFile_t) : confFile(confFile_t), size(0) {
}

Conf::~Conf() {
	
}

int Conf::setKeyPrm(String key,String prm){
	MUTEX_AUTO_LOCK(confLock);
	for(int i=0;i<size;++i){
                if (configValues[i][0]==key) {
			configValues[i][1] =  prm;	
			return(i);
		}
	}
	if ((size<MAXCONFENTRIES)) {
		configValues[size][0] =  key;	
		configValues[size][1] =  prm;	
		++size;
		return(size);
	}
	return(-1);
}

int Conf::getValAsInt(String key){
	MUTEX_AUTO_LOCK(confLock);
	for(int i=0;i<size;++i){
                if (configValues[i][0]==key) return(configValues[i][1].c_int());
        }
	return(-1);
}

int Conf::getValAsIP(String key){
	MUTEX_AUTO_LOCK(confLock);
	for(int i=0;i<size;++i){
                if (configValues[i][0]==key) {
			struct in_addr inp;
			if ((inet_aton(configValues[i][1].c_str(), &inp))>0) {
				return(inp.s_addr); 
			}
			else return(0);
		}
        }
	return(0);
}

String Conf::getValAsString(String key){
	MUTEX_AUTO_LOCK(confLock);
	for(int i=0;i<size;++i){
                if (configValues[i][0]==key) return(configValues[i][1]);
        }
	return("");
}

int Conf::log(){
	MUTEX_AUTO_LOCK(confLock);
	for(int i=0;i<size;++i){
                INF((__FILE__,__FUNCTION__,"%s = %s\n",configValues[i][0].c_str(),configValues[i][1].c_str()))
        }
	return(0);
}

int Conf::read(){
	MUTEX_AUTO_LOCK(confLock);
	String line[2];
        int ret=1;
	
	size=0;
        while (size<MAXCONFENTRIES) {
		configValues[size][0] ="";	
		configValues[size][1] ="";	
		++size;
	}
	size=0;

        if(Conf::openFile()<0){
                WRN((__FILE__,__FUNCTION__,"could not open file\n"))
                return(-1);
        }

        LOG((__FILE__,__FUNCTION__,10,"reading configuration file\n"))

        while ((ret!=0) && (size<MAXCONFENTRIES)) {
                ret=getKeyPrm(&line[0],&line[1]);
                if (ret==1) {
			configValues[size][0] =  line[0];	
			configValues[size][1] =  line[1];	
			++size;
                }
        }

        if(Conf::closeFile()<0){
                WRN((__FILE__,__FUNCTION__,"could not close file\n"))
                return(-1);
        }

        return(1);
}

int Conf::openFile(){
	if ((file=fopen(confFile.c_str(),"r"))==NULL) {
                WRN((__FILE__,__FUNCTION__,"could not open config file %s\n",confFile.c_str()))
                return(-1);
        }
	return(0);
}

int Conf::closeFile(){
	if(fclose(file)<0){
                WRN((__FILE__,__FUNCTION__,"could not close config file %s\n",confFile.c_str()))
                return(-1);
	}
	return(0);
}

int Conf::getKeyPrm(String *key_str,String *prm_str) {    
	/* leerzeichen werden ueberlesen */
	/* ; ist abschluss */
	/* zeilen mit # am anfang werden ueberlesen */
	/* KEY_WORD=PRM_WORD; */

        int key_rd=1;
        int prm_rd=0;
        int trm_ok=0;
        int ass_ok=0;
        char line[maxP_StringLength];
        char key[maxP_StringLength];
        char prm[maxP_StringLength];
        const char *comment="#";
        const char *lineend="\0";
        int  c=0;
        char *cs1;
        char *cs2;
        char *cs3;
        int  n=maxP_StringLength;

        cs1=line;
        cs2=key;
        cs3=prm;
        while(--n > 0 && (c = getc(file)) != EOF){
                if (c=='\n') break;
                if (c=='='){
                        if(ass_ok==0) {
                                ass_ok=1;
                                if (prm_rd==0) prm_rd=1;
                                if (key_rd==1) key_rd=0;
                        }
                        else {
                                ass_ok=-1;
                                prm_rd=0;
                                key_rd=0;
                        }
                }
                if (c==';') {
                        if(trm_ok==0) {
                                trm_ok=1;
                        }
                        else {
                                trm_ok=-1;
                                prm_rd=0;
                                key_rd=0;
                        }
                }
                if (c=='#') {
                        prm_rd=0;
                        key_rd=0;
                }
                if ((c!=' ')&&(c!=';')&&(c!='=')) {
                        *cs1++=c;
                        if(key_rd==1) *cs2++=c;
                        if(prm_rd==1) *cs3++=c;
                }
        }
        *cs1='\0';
        *cs2='\0';
        *cs3='\0';
        if (c==EOF) {
                return(0);
        }
        if (strncmp(line,comment,1)==0){
		LOG((__FILE__,__FUNCTION__,15,"comment line, skipping %s\n",line))
                return (-1);
        }
        if (strncmp(line,lineend,1)==0){
		LOG((__FILE__,__FUNCTION__,15,"empty line, skipping %s\n",line))
                return (-1);
        }
        if (trm_ok!=1){
		WRN((__FILE__,__FUNCTION__,"syntax error in config file: \";\" identifier %s\n",line))
                return (-1);
        }
	if (ass_ok!=1){
		WRN((__FILE__,__FUNCTION__,"syntax error in config file: \"=\" identifier %s\n",line))
                return (-1);
        }

	LOG((__FILE__,__FUNCTION__,15,"LINE: %s=%s;\n",key,prm))

	*key_str=String(key);
	*prm_str=String(prm);

	return(1);
}

