/* Copyright (C) 2000 KOM/Darmstadt University of Technology
 *
 * You are allowed to use all other parts of the code under the following terms:
 *
 * For non-commercial use, code may be used in unmodified form provided
 * that this copyright notice and this permission notice appear in
 * supporting documentation.
 *
 * This software is provided "as is" and without any express or implied
 * warranties, including, without limitation, the implied warranty of
 * fitness for a particular purpose.
 *
 * The code may be subjected to the GNU General Public License, Version 2,
 * and re-distributed under the terms of this license.
 *
 * Commercial use other than under the terms of the GNU General Public
 * License is allowed only after express negotiation of conditions
 * with the authors.
 */



#include "Connection.h"
#include "Log.h"
#include "Conf.h"
#include "Proxy.h"

#ifdef _USE_FIREWALL_CONTROL_
#include "Fwfil.h"
#include "FirewallControl.h"
#endif

#include <fcntl.h> /* fcntl */
#include <netdb.h> /* gethostbyname */
#include <errno.h> /* errno */
#include <string.h> /* sterror */
#include <sys/types.h> /* sendto,socket,bind,accept,setsockopt,recvfrom */
#include <sys/socket.h> /* sendto,socket,bind,listen,connect,setsockopt,shutdown,recvfrom  */
#include <unistd.h> /* read, write, close */
#include <netinet/in.h> /* struct sockaddr_in on FreeBSD */


Connection::Connection():regFD(false){
	systemFD=NULL;
	LOCAL_IP = (Global::getGlobal()->getConf()->getValAsIP("LOCALIP"));
	connectionInitTCPUDP();
}
Connection::Connection(Connection *c):regFD(false){
	systemFD=NULL;
	LOCAL_IP = (Global::getGlobal()->getConf()->getValAsIP("LOCALIP"));
	connectionCloneTCPUDP(c);
}
Connection::Connection(Proxy* systemFD_t):regFD(false){
	systemFD=systemFD_t;
	LOCAL_IP = (Global::getGlobal()->getConf()->getValAsIP("LOCALIP"));
	connectionInitTCPUDP();
}
Connection::Connection(Connection *c,Proxy* systemFD_t):regFD(false){
	systemFD=systemFD_t;
	LOCAL_IP = (Global::getGlobal()->getConf()->getValAsIP("LOCALIP"));
	connectionCloneTCPUDP(c);
}

Connection::~Connection() {
	connectionClose();
	if (regFD) FAT((__FILE__,__FUNCTION__,"FD is registered %s\n",getDescript().c_str()))
}

int Connection::connectionInitTCPUDP(){
	fd		= -1;
        state		= CLOSE;	
        queue_len	= 0;

	l_ip		= 0; 
	l_mask 		= 0;
        l_port		= -1;

	p_ip		= 0; 
	p_mask 		= 0;
        p_port		= -1;

	d_name		= String("---");
        d_number	= 0;

        ipproto		= 6;
        direction	= 0;
	zeroBufferTCPUDP();

	hardUDP		= true;
	tryPort		= 0;
	RTCP		=false;
	initiator	=false;

	return(0);
}

int Connection::connectionCloneTCPUDP(Connection *c){

        fd 		= c->get_fd();
        state		= c->state;

        packet_len	= c->packet_len;
	memcpy(buffer,c->buffer,BUFFERSIZE);
        queue_len	= c->queue_len;
	memcpy(queue,c->queue,BUFFERSIZE);

	l_ip		= c->l_ip;
	l_mask		= c->l_mask;
	l_port		= c->l_port;

	p_ip		= c->p_ip;
	p_mask		= c->p_mask;
	p_port		= c->p_port;

        ipproto		= c->ipproto;
        direction	= c->direction;

	d_name		= c->d_name;
        d_number	= c->d_number;
	hardUDP		= c->hardUDP;
	tryPort		= 0;
	RTCP		= c->RTCP;
	initiator	= c->initiator;

	return(0);
}


String Connection::getDescript(){
	/* [desc:192.168.100.1:200 <-> 192.168.200.2:222 (P:17|S:0|D:1|N:50|L:500)] */ 	
	String d;

	d = d +	"[" 	+ d_name;
	d = d + ":" 	+ String::IPtoString(l_ip); 
	d = d + ":" 	+ l_port;
	d = d + " <-> "	+ String::IPtoString(p_ip);
	d = d + ":" 	+ p_port;
	d = d + " (FD:"	+ fd;
	d = d + "|P:"	+ ipproto;
	d = d + "|S:"	+ state;
	d = d + "|D:"	+ direction;
	d = d + "|N:"	+ d_number;
	d = d + "|L:"	+ packet_len; 
	d = d + ")]";

	return (d);
}

bool Connection::toState(states s){
	switch(state) {
                case CLOSE:
			switch(s) {
                		case CLOSE:
					FAT((__FILE__,__FUNCTION__,"STATE TRANSITION: CLOSE->CLOSE transition not allowed %s\n",getDescript().c_str()))
					break;
                		case LISTEN:
					state = s;
					if (systemFD!=NULL){
						if ((fd>=0) && (!regFD)) {systemFD->addFD(fd);regFD=true;}
						else FAT((__FILE__,__FUNCTION__,"STATE TRANSITION: CLOSE->LISTEN add FD %s\n",getDescript().c_str()))
					}
					return true;
					break;
                		case CONNECT:
					state = s;
					if (systemFD!=NULL){
						if ((fd>=0) && (!regFD)) {systemFD->addFD(fd);regFD=true;}
						else FAT((__FILE__,__FUNCTION__,"STATE TRANSITION: CLOSE->CONNECT add FD %s\n",getDescript().c_str()))
					}
					return true;
					break;
			}
			break;
                case LISTEN:
			switch(s) {
                		case CLOSE:
					state = s;
					if (systemFD!=NULL){
						if ((fd>=0) && (regFD)) {systemFD->delFD(fd);regFD=false;}
						else FAT((__FILE__,__FUNCTION__,"STATE TRANSITION: LISTEN->CLOSE del FD %s\n",getDescript().c_str()))
					}
					return true;
					break;
                		case LISTEN:
					FAT((__FILE__,__FUNCTION__,"STATE TRANSITION: LISTEN->LISTEN transition not allowed %s\n",getDescript().c_str()))
					break;
                		case CONNECT:
					state = s;
					if (systemFD!=NULL){
						if ((fd>=0) && (!regFD)) {systemFD->addFD(fd);regFD=true;}
						/* the UDP accept case */
						else if ((fd>=0) && (regFD) && (ipproto==17)) {systemFD->addFD(fd);regFD=true;}
						else FAT((__FILE__,__FUNCTION__,"STATE TRANSITION: LISTEN->CONNECT add FD %s\n",getDescript().c_str()))
					}
					return true;
					break;
			}
			break;
                case CONNECT:
			switch(s) {
                		case CLOSE:
					state = s;
					if (systemFD!=NULL){
						if ((fd>=0) && (regFD)) {systemFD->delFD(fd);regFD=false;}
						else FAT((__FILE__,__FUNCTION__,"STATE TRANSITION: CONNECT->CLOSE del FD %s\n",getDescript().c_str()))
					}
					return true;
					break;
                		case LISTEN:
					FAT((__FILE__,__FUNCTION__,"STATE TRANSITION: CONNECT->LISTEN transition not allowed %s\n",getDescript().c_str()))
					break;
                		case CONNECT:
					FAT((__FILE__,__FUNCTION__,"STATE TRANSITION: CONNECT->CONNECT transition not allowed %s\n",getDescript().c_str()))
					break;
			}
			break;
	}
	return false;
}

void Connection::setSystemFD(Proxy* systemFD_t){
	systemFD=systemFD_t;
	if ((fd>=0) && (systemFD!=NULL)){
		if (!regFD) {systemFD->addFD(fd);regFD=true;}
		else FAT((__FILE__,__FUNCTION__,"add FD %s\n",getDescript().c_str()))
	}
}


int Connection::connectionReadTCP(int len){
	int r_len=0;
	if (state!=CONNECT){
		WRN((__FILE__,__FUNCTION__,"TCP: read socket not connected %s\n",getDescript().c_str()))
		return(-1);
	}
	if (len<0){
		WRN((__FILE__,__FUNCTION__,"TCP: read length %d specified %s\n",len,getDescript().c_str()))
		return(-1);
	}
        if (((packet_len)+len) >= (BUFFERSIZE)) {
		WRN((__FILE__,__FUNCTION__,"TCP: request larger than buffer len:%d %s\n",len,getDescript().c_str()))
		return(-1);
	}
        r_len=read(fd,(buffer)+(packet_len),len);
	if (r_len==-1) {
		if (errno==ECONNRESET) {
			LOG((__FILE__,__FUNCTION__,10,"TCP: read socket errno: %d %s %s\n",errno,strerror(errno),getDescript().c_str()))
			return(0);
		}
		FAT((__FILE__,__FUNCTION__,"TCP: read socket errno: %d %s %s\n",errno,strerror(errno),getDescript().c_str()))
	}
	if (r_len==0) {
		LOG((__FILE__,__FUNCTION__,10,"TCP: read was 0 %s\n",getDescript().c_str()))
		return(0);
	}
        (packet_len)=(packet_len)+r_len;
        return(r_len);
}

int Connection::connectionReadUDP(int len){
	int r_len=0;
	char b[BUFFERSIZE];

	if (state!=CONNECT){
		WRN((__FILE__,__FUNCTION__,"UDP: read socket not connected %s\n",getDescript().c_str()))
		return(-1);
	}
	if (len<0){
		WRN((__FILE__,__FUNCTION__,"UDP: read length %d specified %s\n",len,getDescript().c_str()))
		return(-1);
	}

        if (((packet_len)+len) >= (BUFFERSIZE)) {
		WRN((__FILE__,__FUNCTION__,"UDP: request larger than buffer len:%d %s\n",len,getDescript().c_str()))
		return(-1);
	}
	/* read packet and add to queue if queue empty or not enough bytes in queue*/
	if (((queue_len)==0 ) || (len>(queue_len))) {
		/* we need a check if the packet is lager than the available space in queue */
		r_len=read(fd,(queue)+(queue_len),BUFFERSIZE-(queue_len));
		if (r_len==-1) {
			if (errno==ECONNREFUSED) {
				LOG((__FILE__,__FUNCTION__,10,"UDP: read socket errno: %d %s %s\n",errno,strerror(errno),getDescript().c_str()))
				return(0);
			}
			FAT((__FILE__,__FUNCTION__,"UDP: read socket errno: %d %s %s\n",errno,strerror(errno),getDescript().c_str()))
		}
		if (r_len==0) {
			LOG((__FILE__,__FUNCTION__,10,"UDP: read was 0 %s\n",getDescript().c_str()))
			return(0);
		}
        	(queue_len)=(queue_len)+r_len;
	}
	/* put the bytes requested in the buffer */
	if (len>queue_len) len=queue_len;
	memcpy((buffer)+(packet_len),(queue),len);
	memcpy(b,(queue)+len,(queue_len)-len);
	memcpy((queue),b,(queue_len)-len);
	(queue_len)=(queue_len)-len;
        (packet_len)=(packet_len)+len;
        return(len);
}

int Connection::connectionRead(int len){
	if (ipproto==6 ) return connectionReadTCP(len);
	if (ipproto==17) return connectionReadUDP(len);
	return (-1);
}

int Connection::connectionWriteTCP(Connection *cin){
	int w_len=0;
	if (state!=CONNECT){
		WRN((__FILE__,__FUNCTION__,"TCP: write socket not connected %s\n",getDescript().c_str()))
		return(-1);
	}
        if ((cin->packet_len) >= (BUFFERSIZE)) {
		WRN((__FILE__,__FUNCTION__,"TCP: message size larger than buffer %s\n",getDescript().c_str()))
		return(-1);
	}
	while(w_len!=(cin->packet_len)){
                w_len=w_len+write(fd,(cin->buffer)+w_len,(cin->packet_len)-w_len);
		if (w_len==-1) {
			FAT((__FILE__,__FUNCTION__,"TCP: write socket errno: %d %s %s\n",errno,strerror(errno),getDescript().c_str()))
		}
        }
	return (w_len);
}

int Connection::connectionWriteUDP(Connection *cin){
	int w_len=0;
	if (state!=CONNECT){
		WRN((__FILE__,__FUNCTION__,"UDP: write socket not connected %s\n",getDescript().c_str()))
		return(-1);
	}
	if ((p_ip==0)||(p_port==0)) {
		WRN((__FILE__,__FUNCTION__,"UDP: destination not specified, could not send data %s\n",getDescript().c_str()))
		return(-1);
	}
        if ((cin->packet_len) >= (BUFFERSIZE)) {
		WRN((__FILE__,__FUNCTION__,"UDP: message size larger than buffer %s\n",getDescript().c_str()))
		return(-1);
	}
	if(hardUDP){
		w_len=write(fd,(cin->buffer),(cin->packet_len));
		if (w_len<0){
			if (errno==ECONNREFUSED) {
				LOG((__FILE__,__FUNCTION__,10,"UDP: write socket errno: %d %s %s\n",errno,strerror(errno),getDescript().c_str()))
				return(0);
			}
			else {
				FAT((__FILE__,__FUNCTION__,"UDP: write socket errno: %d %s %s\n",errno,strerror(errno),getDescript().c_str()))
			}
		}
	}
	else {
		LOG((__FILE__,__FUNCTION__,20,"UDP: socket is not connected using sendto %s\n",getDescript().c_str()))
		struct sockaddr_in p_addr;	
		ACCEPT_SIZE_T size_p_addr = sizeof(p_addr);
		memset( &p_addr.sin_zero, 0, sizeof(p_addr.sin_zero) );
		p_addr.sin_family       = AF_INET;
		p_addr.sin_addr.s_addr  = p_ip;
		p_addr.sin_port         = htons(p_port);
#if defined(HAVE_SIN_LEN)
		p_addr.sin_len          = sizeof( p_addr );
#endif
		if (p_ip!=0)    p_mask  = 0xffffffff;
		else            p_mask  = 0;
		w_len=sendto(fd,(cin->buffer), (cin->packet_len), 0, (struct sockaddr*)&p_addr,size_p_addr);
		if (w_len<0){
			if (errno==ECONNREFUSED) {
				LOG((__FILE__,__FUNCTION__,10,"UDP: write socket errno: %d %s %s\n",errno,strerror(errno),getDescript().c_str()))
				return(0);
			}
			else {
				FAT((__FILE__,__FUNCTION__,"UDP: write socket errno: %d %s %s\n",errno,strerror(errno),getDescript().c_str()))
			}
		}
	}
	return (w_len);
}

int Connection::connectionWrite(Connection *cin){
	if (ipproto==6 ) return connectionWriteTCP(cin);
	if (ipproto==17) return connectionWriteUDP(cin);
	return (-1);
}

int Connection::connectionListenTCP(){
        int s;                         
        struct sockaddr_in l_addr;  

	if (state!=CLOSE){
		FAT((__FILE__,__FUNCTION__,"TCP: listen %s\n",getDescript().c_str()))
	}

        if ((s = socket(AF_INET, SOCK_STREAM,0))<0) {
		FAT((__FILE__,__FUNCTION__,"TCP: allocate socket errno: %d %s %s\n",errno,strerror(errno),getDescript().c_str()))
	}
	memset( &l_addr.sin_zero, 0, sizeof(l_addr.sin_zero) );
#if defined(HAVE_SIN_LEN)
	l_addr.sin_len 			= sizeof( l_addr );
#endif
        l_addr.sin_family 		= AF_INET;
	if (l_ip==0)
       	  	l_addr.sin_addr.s_addr 	= INADDR_ANY; 
	else 
		l_addr.sin_addr.s_addr 	= l_ip;
	if (l_port==-1){
		l_addr.sin_port 	= htons(Connection::getPort());
        	while (bind(s,(struct sockaddr*)&l_addr,sizeof(l_addr))<0){
			if ((errno==EADDRINUSE) && (l_port==-1) && (tryPort<MAXTRY)) {
				LOG((__FILE__,__FUNCTION__,10,"TCP: bind errno: %d %s %s\n",errno,strerror(errno),getDescript().c_str()))
                		l_addr.sin_port = htons(Connection::getPort());
				++tryPort;
			}
	                else if (errno==EADDRINUSE) {                
                        	LOG((__FILE__,__FUNCTION__,10,"TCP: bind errno: %d %s %s\n",errno,strerror(errno),getDescript().c_str()))
                        	return (-1);
                	}
			else FAT((__FILE__,__FUNCTION__,"TCP: bind errno: %d %s %s\n",errno,strerror(errno),getDescript().c_str()))
        	}
	}
	else {
		l_addr.sin_port 	= htons(l_port);
        	if (bind(s,(struct sockaddr*)&l_addr,sizeof(l_addr))<0){
			FAT((__FILE__,__FUNCTION__,"TCP: bind errno: %d %s %s\n",errno,strerror(errno),getDescript().c_str()))
		}
	}
	tryPort=0;
	l_ip	= l_addr.sin_addr.s_addr;
	l_port	= ntohs(l_addr.sin_port);
	if (l_ip!=0) l_mask=0xffffffff;
	else  l_mask=0;

#ifdef _USE_FIREWALL_CONTROL_
        if(connectionGetChannelTCPUDP()<0){
                ERR((__FILE__,__FUNCTION__,"could not configure firewall for incomming connection %s\n",getDescript().c_str()))
        }
#endif

        if (listen(s,25)<0){
		FAT((__FILE__,__FUNCTION__,"TCP: listen errno: %d %s %s\n",errno,strerror(errno),getDescript().c_str()))
	}
	fd=s;
	toState(LISTEN);
	LOG((__FILE__,__FUNCTION__,10,"TCP: socket is connected %s\n",getDescript().c_str()))
        return(fd);
}

int Connection::connectionListenUDP(){
        int s;                         
        struct sockaddr_in l_addr;  
	const int on=1;
	int port;

	if (state!=CLOSE){
		FAT((__FILE__,__FUNCTION__,"UDP: listen %s\n",getDescript().c_str()))
	}

        if ((s = socket(AF_INET, SOCK_DGRAM,0))<0) {
		FAT((__FILE__,__FUNCTION__,"UDP: allocate socket errno: %d %s %s\n",errno,strerror(errno),getDescript().c_str()))
	}

	memset( &l_addr.sin_zero, 0, sizeof(l_addr.sin_zero) );
#if defined(HAVE_SIN_LEN)
       	l_addr.sin_len 			= sizeof( l_addr );
#endif
        l_addr.sin_family 		= AF_INET;
	if (l_ip==0)
        	l_addr.sin_addr.s_addr 	= INADDR_ANY; 
	else    
		l_addr.sin_addr.s_addr 	= l_ip;
	if (l_port==-1){
		port=Connection::getPort();
                if (RTCP) {
                	port=Connection::getPort();
                	if ((port%2)==0) port=Connection::getPort();
                }
		l_addr.sin_port 	= htons(port);
        	while (bind(s,(struct sockaddr*)&l_addr,sizeof(l_addr))<0){
			if ((errno==EADDRINUSE) && (l_port==-1) && (tryPort<MAXTRY)) {
				LOG((__FILE__,__FUNCTION__,10,"UDP: bind errno: %d %s %s\n",errno,strerror(errno),getDescript().c_str()))
				port=Connection::getPort();
				if (RTCP) {	
					port=Connection::getPort();
					if ((port%2)==0) port=Connection::getPort();
				}
                		l_addr.sin_port = htons(port);
				++tryPort;
			}
                        else if (errno==EADDRINUSE) {                
                                LOG((__FILE__,__FUNCTION__,10,"UDP: bind errno: %d %s %s\n",errno,strerror(errno),getDescript().c_str()))
                                return (-1);
                        }
			else FAT((__FILE__,__FUNCTION__,"UDP: bind errno: %d %s %s\n",errno,strerror(errno),getDescript().c_str()))
        	}
	}
	else {
		l_addr.sin_port 	= htons(l_port);
        	if (bind(s,(struct sockaddr*)&l_addr,sizeof(l_addr))<0){
			FAT((__FILE__,__FUNCTION__,"UDP: bind errno: %d %s %s\n",errno,strerror(errno),getDescript().c_str()))
		}
	}
	tryPort=0;
	l_ip	= l_addr.sin_addr.s_addr;
	l_port	= ntohs(l_addr.sin_port);
	if (l_ip!=0) 	l_mask	= 0xffffffff;
	else  		l_mask	= 0;

#ifdef _USE_FIREWALL_CONTROL_
        if(connectionGetChannelTCPUDP()<0){
                ERR((__FILE__,__FUNCTION__,0,"could not configure firewall for incomming connection %s\n",getDescript().c_str()))
        }
#endif

	fd=s;
	toState(LISTEN);
	LOG((__FILE__,__FUNCTION__,10,"UDP: socket is connected %s\n",getDescript().c_str()))
       	return(fd);
}

int Connection::connectionListen(){
	if ((ipproto)==6 ) return connectionListenTCP();
	if ((ipproto)==17) return connectionListenUDP();
	return (-1);
}

Connection* Connection::connectionAcceptTCP(){
        int s=-1;
        struct sockaddr_in p_addr;
	ACCEPT_SIZE_T size_p_addr = sizeof(p_addr);

	Connection *c = new Connection(this,systemFD);
	c->set_fd(-1);

	if (state!=LISTEN){
		FAT((__FILE__,__FUNCTION__,"TCP: accept %s\n",getDescript().c_str()))
	}

	memset( &p_addr.sin_zero, 0, sizeof(p_addr.sin_zero) );
#if defined(HAVE_SIN_LEN)
       	p_addr.sin_len 	= sizeof( p_addr );
#endif
        size_p_addr=sizeof(p_addr);
        if((s = accept(fd, (struct sockaddr* )&p_addr, &size_p_addr))<0){
		if (errno==EINTR) WRN((__FILE__,__FUNCTION__,"TCP: accept, interrupted system call %s\n",getDescript().c_str()))
		else 		  FAT((__FILE__,__FUNCTION__,"TCP: accept errno: %d %s %s\n",errno,strerror(errno),getDescript().c_str()))
		delete c;	
		toState(CLOSE);
       		return (NULL);
	}

	LOG((__FILE__,__FUNCTION__,10,"TCP: socket is connected %s\n",getDescript().c_str()))
	c->set_fd(s);
        c->p_ip   = p_addr.sin_addr.s_addr;
	c->p_mask = 0xffffffff;
	c->p_port = ntohs(p_addr.sin_port);

#ifdef _USE_FIREWALL_CONTROL_
       	if(c->connectionGetChannelTCPUDP()<0){
               	ERR((__FILE__,__FUNCTION__,0,"could not configure firewall %s\n",getDescript().c_str()))
       	}
#endif
	c->toState(CONNECT);
	return(c);
}

Connection* Connection::connectionAcceptUDP(){
       	struct sockaddr_in p_addr;  
        struct sockaddr_in l_addr;  
	ACCEPT_SIZE_T size_p_addr = sizeof(p_addr);
	int len=1;
	char b[1];
        int s;
	const int on=1;

	if (state!=LISTEN){
		FAT((__FILE__,__FUNCTION__,"UDP: accept %s\n",getDescript().c_str()))
	}

	memset( &p_addr.sin_zero, 0, sizeof(p_addr.sin_zero) );
#if defined(HAVE_SIN_LEN)
       	p_addr.sin_len = sizeof( p_addr );
#endif
	if ((p_ip==0) || (p_port==-1)) {
		LOG((__FILE__,__FUNCTION__,10,"UDP: destination not specified, setting src by read %s\n",getDescript().c_str()))
		if (recvfrom(fd, b, len, MSG_PEEK, (struct sockaddr*)&p_addr,&size_p_addr)<0){
			if (errno==EINTR) WRN((__FILE__,__FUNCTION__,"UDP: accept, interrupted system call %s\n",getDescript().c_str()))
			else 		  FAT((__FILE__,__FUNCTION__,"UDP: accept errno: %d %s %s\n",errno,strerror(errno),getDescript().c_str()))
       			return (NULL);
		}
        	p_ip		= p_addr.sin_addr.s_addr;
		p_port	= ntohs(p_addr.sin_port);
		p_mask	= 0xffffffff;
        	if (connect(fd, (struct sockaddr*)&p_addr, sizeof(p_addr) )<0 ){
                	FAT((__FILE__,__FUNCTION__,"UDP: connect %s\n",getDescript().c_str()))
        	}
	}
	toState(CONNECT);
	LOG((__FILE__,__FUNCTION__,10,"UDP: socket is connected %s\n",getDescript().c_str()))
       	return(this);
}

Connection* Connection::connectionAccept(){
	if ((ipproto)==6 ) return connectionAcceptTCP();
	if ((ipproto)==17) return connectionAcceptUDP();
	return(NULL);
}

int Connection::connectionConnectTCP(){        
        struct sockaddr_in l_addr; 
        struct sockaddr_in p_addr;  
        int    s;                       

	if (state!=CLOSE){
		FAT((__FILE__,__FUNCTION__,"TCP: connect %s\n",getDescript().c_str()))
	}

	memset( &p_addr.sin_zero, 0, sizeof(p_addr.sin_zero) );
#if defined(HAVE_SIN_LEN)
        p_addr.sin_len 		= sizeof( p_addr );
#endif
        p_addr.sin_family 	= AF_INET;
	p_addr.sin_addr.s_addr	= p_ip;
        p_addr.sin_port		= htons(p_port);
	if (p_ip!=0) 	p_mask	= 0xffffffff;
	else  		p_mask	= 0;

        if ((s = socket(AF_INET, SOCK_STREAM,0))<0) {
		FAT((__FILE__,__FUNCTION__,"TCP: allocate socket errno: %d %s %s\n",errno,strerror(errno),getDescript().c_str()))
	}

	memset( &l_addr.sin_zero, 0, sizeof(l_addr.sin_zero) );
#if defined(HAVE_SIN_LEN)
        l_addr.sin_len 		= sizeof( l_addr );
#endif
        l_addr.sin_family 	= AF_INET;
	if (l_port==-1){
		l_addr.sin_port 	= htons(Connection::getPort());
	}
	else 	l_addr.sin_port         = htons(l_port);
	l_addr.sin_port 	= htons(Connection::getPort());
        l_addr.sin_addr.s_addr 	= LOCAL_IP; 
        while (bind(s,(struct sockaddr*)&l_addr,sizeof(l_addr))<0){
		if ((errno==EADDRINUSE) && (l_port==-1) && (tryPort<MAXTRY)) {
			LOG((__FILE__,__FUNCTION__,10,"TCP: bind errno: %d %s %s\n",errno,strerror(errno),getDescript().c_str()))
               		l_addr.sin_port = htons(Connection::getPort());
			++tryPort;
		}
                else if (errno==EADDRINUSE) {                
                        LOG((__FILE__,__FUNCTION__,10,"TCP: bind errno: %d %s %s\n",errno,strerror(errno),getDescript().c_str()))
			close(s);
                        return (-1);
                }
		else FAT((__FILE__,__FUNCTION__,"TCP: bind errno: %d %s %s\n",errno,strerror(errno),getDescript().c_str()))
	}
	tryPort=0;
	l_ip    = l_addr.sin_addr.s_addr;
        l_port  = ntohs(l_addr.sin_port);
        if (l_ip!=0) l_mask = 0xffffffff;
        else  	     l_mask = 0;

#ifdef _USE_FIREWALL_CONTROL_
        if(connectionGetChannelTCPUDP()<0){
                ERR((__FILE__,__FUNCTION__,"could not configure firewall for incomming connection %s\n",getDescript().c_str()))
        }
#endif

	if (connect(s, (struct sockaddr *)&p_addr, sizeof(p_addr)) < 0) {
        	ERR((__FILE__,__FUNCTION__,"TCP: connect not possible %s\n",getDescript().c_str()))
		close(s);
        	return (-1);
        }

	fd=s;
	toState(CONNECT);
	LOG((__FILE__,__FUNCTION__,10,"TCP: socket is connected %s\n", getDescript().c_str()))
       	return (fd);
}

int Connection::connectionConnectUDP(){        
        int    s;                       
        struct sockaddr_in l_addr; 
        struct sockaddr_in p_addr;  
	int port;
	
	if (state!=CLOSE){
		FAT((__FILE__,__FUNCTION__,"UDP: connect %s\n",getDescript().c_str()))
	}

	memset( &p_addr.sin_zero, 0, sizeof(p_addr.sin_zero) );
        p_addr.sin_family 	= AF_INET;
	p_addr.sin_addr.s_addr 	= p_ip; 
        p_addr.sin_port		= htons(p_port);
#if defined(HAVE_SIN_LEN)
        p_addr.sin_len 		= sizeof( p_addr );
#endif
	if (p_ip!=0) 	p_mask	= 0xffffffff;
	else  		p_mask	= 0;

        if ((s = socket(AF_INET, SOCK_DGRAM,0))<0) {
		FAT((__FILE__,__FUNCTION__,"UDP: allocate socket errno: %d %s %s\n",errno,strerror(errno),getDescript().c_str()))
	}

	memset( &l_addr.sin_zero, 0, sizeof(l_addr.sin_zero) );
        l_addr.sin_family 	= AF_INET;
	if (l_port==-1){
		port=Connection::getPort();
                if (RTCP) {
                	port=Connection::getPort();
                	if ((port%2)==0) port=Connection::getPort();
                }
		l_addr.sin_port 	= htons(port);
	}
	else 	l_addr.sin_port         = htons(l_port);
        l_addr.sin_addr.s_addr 	= LOCAL_IP; 
#if defined(HAVE_SIN_LEN)
        l_addr.sin_len 		= sizeof( l_addr );
#endif
        while (bind(s,(struct sockaddr*)&l_addr,sizeof(l_addr))<0){
		if ((errno==EADDRINUSE) && (l_port==-1) && (tryPort<MAXTRY)) {
			LOG((__FILE__,__FUNCTION__,10,"UDP: bind errno: %d %s %s\n",errno,strerror(errno),getDescript().c_str()))
			port=Connection::getPort();
                	if (RTCP) {
                		port=Connection::getPort();
                		if ((port%2)==0) port=Connection::getPort();
                	}
			l_addr.sin_port	= htons(port);
			++tryPort;
		}
		else if (errno==EADDRINUSE) {
			LOG((__FILE__,__FUNCTION__,10,"UDP: bind errno: %d %s %s\n",errno,strerror(errno),getDescript().c_str()))
			close(s);
                	return (-1);
		}
		else FAT((__FILE__,__FUNCTION__,"UDP: bind errno: %d %s %s\n",errno,strerror(errno),getDescript().c_str()))
	}
	tryPort=0;
	l_ip    = l_addr.sin_addr.s_addr;
        l_port  = ntohs(l_addr.sin_port);
        if (l_ip!=0) l_mask = 0xffffffff;
        else  	     l_mask = 0;

#ifdef _USE_FIREWALL_CONTROL_
        if(connectionGetChannelTCPUDP()<0){
                ERR((__FILE__,__FUNCTION__,"could not configure firewall for incomming connection %s\n",getDescript().c_str()))
        }
#endif

	if(hardUDP){
        	if (connect(s, (struct sockaddr *)&p_addr, sizeof(p_addr)) < 0) {
                	ERR((__FILE__,__FUNCTION__,"UDP: connect not possible %s\n",getDescript().c_str()))
			close(s);
                	return (-1);
        	}
	}
	fd=s;
	toState(CONNECT);
	LOG((__FILE__,__FUNCTION__,10,"UDP: socket is connected %s\n",getDescript().c_str()))
       	return (fd);
}

int Connection::connectionConnect(){
	if ((ipproto)==6 ) return connectionConnectTCP();
	if ((ipproto)==17) return connectionConnectUDP();
	return(-1);
}

int Connection::connectionSetNonBlockTCPUDP(){
 	if (fcntl( fd, F_SETFL, (fcntl(fd, F_GETFL) | O_NONBLOCK ))){
                FAT((__FILE__,__FUNCTION__,"setting socket O_NONBLOCK errno: %d %s %s\n",errno,strerror(errno),getDescript().c_str()))
	}
        LOG((__FILE__,__FUNCTION__,10,"socket set non blocking %s\n",getDescript().c_str()))
	return(1);
}

int Connection::connectionSetNonBlock(){
	return connectionSetNonBlockTCPUDP();
}

int Connection::connectionSetLingerTCPUDP(){
	struct  linger l;
	l.l_onoff=1;
	l.l_linger=5;
 	if (setsockopt( fd, SOL_SOCKET, SO_LINGER, (struct linger *)&l, sizeof(l))){
                FAT((__FILE__,__FUNCTION__,"setting socket SO_LINGER errno: %d %s %s\n",errno,strerror(errno),getDescript().c_str()))
	}
        LOG((__FILE__,__FUNCTION__,10,"socket set SO_LINGER %s\n",getDescript().c_str()))
	return(1);
}

int Connection::connectionSetLinger(){
	return connectionSetLingerTCPUDP();
}
int Connection::connectionSetReuseAddrTCPUDP(){
	const int on=1;
 	if (setsockopt( fd, SOL_SOCKET, SO_REUSEADDR, (char*)&on , sizeof(on))){
                FAT((__FILE__,__FUNCTION__,"setting socket SO_REUSEADDR errno: %d %s %s\n",errno,strerror(errno),getDescript().c_str()))
	}
        LOG((__FILE__,__FUNCTION__,10,"socket set SO_REUSEADDR %s\n",getDescript().c_str()))
	return(0);
}

int Connection::connectionSetReuseAddr(){
	return connectionSetReuseAddrTCPUDP();	
}

int Connection::connectionCloseTCP(){
	if (state==CLOSE) return(1);
    	LOG((__FILE__,__FUNCTION__,10,"shutting down connection %s\n",getDescript().c_str()))
	if (fd>=0){
		shutdown(fd,SHUT_RDWR); 
   		close(fd);
	}
#ifdef _USE_FIREWALL_CONTROL_
	connectionDelAllChannelTCPUDP();
#endif
	toState(CLOSE);
	connectionInitTCPUDP();
	return(1);
}

int Connection::connectionCloseUDP(){
	if (state==CLOSE) return(1);
   	LOG((__FILE__,__FUNCTION__,10,"shutting down connection %s\n",getDescript().c_str()))
	if (fd>=0) {
		close(fd);
	}
#ifdef _USE_FIREWALL_CONTROL_
	connectionDelAllChannelTCPUDP();
#endif
	toState(CLOSE);
	connectionInitTCPUDP();
	return(1);
}

int Connection::connectionClose(){
	if ((ipproto)==6 ) return connectionCloseTCP();
	if ((ipproto)==17) return connectionCloseUDP();
	return(-1);
}

int Connection::connectionGetStateTCPUDP(){
	return(state);
}

int Connection::connectionGetState(){
	return connectionGetStateTCPUDP();
}

int Connection::zeroBufferTCPUDP(){
	/* buffer is not zeroed at the moment */
	packet_len=0;
	buffer[0] = '\0';

	return(0);
}
int Connection::zeroBuffer(){
	return zeroBufferTCPUDP();
}

int Connection::get_fd(){
	return fd;
}

int Connection::set_fd(int fd_t){
	fd=fd_t;
	return fd;
}

int Connection::getPort(){
        MUTEX_AUTO_LOCK(connectionLock);
        currentport++;
        if (currentport>MAXPORT) {
                        currentport=MINPORT;
        }
        return currentport;
}

int Connection::gethostip(const char *name){
	MUTEX_AUTO_LOCK(connectionLock);
	struct hostent  *phe;
	struct sockaddr_in addr;
	if ((phe=gethostbyname(name))) {
		memcpy(&addr.sin_addr, phe->h_addr, phe->h_length);
		return(addr.sin_addr.s_addr);
	}
	return(0);
}


#ifdef _USE_FIREWALL_CONTROL_
int Connection::connectionGetChannelTCPUDP(){
        Fwfil *fwfilp;

        fwfilp = new Fwfil(this,true);
        if(fwhndl.getChannel(fwfilp)<0){
                FAT((__FILE__,__FUNCTION__,"fw rule handling %s\n",getDescript().c_str()))
        }
        fwfilp = new Fwfil(this,false);
        if(fwhndl.getChannel(fwfilp)<0){
                FAT((__FILE__,__FUNCTION__,"fw rule handling %s\n",getDescript().c_str()))
        }

        return(0);
}
int Connection::connectionDelAllChannelTCPUDP(){
        if(fwhndl.delAllChannel()<0){
                FAT((__FILE__,__FUNCTION__,"fw rule handling %s\n",getDescript().c_str()))
        }
        return(0);
}
#endif
