/* Copyright (C) 2000 KOM/Darmstadt University of Technology
 *
 * You are allowed to use all other parts of the code under the following terms:
 *
 * For non-commercial use, code may be used in unmodified form provided
 * that this copyright notice and this permission notice appear in
 * supporting documentation.
 *
 * This software is provided "as is" and without any express or implied
 * warranties, including, without limitation, the implied warranty of
 * fitness for a particular purpose.
 *
 * The code may be subjected to the GNU General Public License, Version 2,
 * and re-distributed under the terms of this license.
 *
 * Commercial use other than under the terms of the GNU General Public
 * License is allowed only after express negotiation of conditions
 * with the authors.
 */
#ifndef _Connection_h_
#define _Connection_h_ 1

#include "Global.h"
#include "String.h"
#include "Thread.h"


#define _USE_FIREWALL_CONTROL_ 1
#ifdef _USE_FIREWALL_CONTROL_
#include "Fwhndl.h"
#endif


#define MINPORT 	10000
#define PORTRANGE	((MAXPROXIES*MAXSESSIONS*MAXCONNECTIONS)+MAXLISTENERS+50)
#define MAXPORT 	(MINPORT+PORTRANGE-1) 
#define MAXTRY		30

enum states {CLOSE, LISTEN, CONNECT};

static Mutex connectionLock;
static int   currentport=MINPORT;

class Proxy;
class Connection {
	Proxy*		systemFD;
	bool		regFD;
	int		LOCAL_IP;

#ifdef _USE_FIREWALL_CONTROL_
	Fwhndl		fwhndl;
	int connectionGetChannelTCPUDP();
        int connectionDelAllChannelTCPUDP();
#endif


        int	        queue_len;              /*  */
        char            queue[BUFFERSIZE];      /*  */
        states          state;                  /* marks the state of the connection */
        int             fd;                     /* if fd=-1 fd not valid */
	int		tryPort;

	int connectionInitTCPUDP();
	int connectionCloneTCPUDP(Connection *clone);

	int connectionReadTCP(int len);
	int connectionReadUDP(int len);
	
	int connectionWriteTCP(Connection *cin);
	int connectionWriteUDP(Connection *cin);

	int connectionListenUDP();
	int connectionListenTCP();

	Connection* connectionAcceptUDP();
	Connection* connectionAcceptTCP();

	int connectionConnectUDP();
	int connectionConnectTCP();

	int connectionCloseTCP();
	int connectionCloseUDP();

	int connectionSetNonBlockTCPUDP();
	int connectionSetLingerTCPUDP();
	int connectionSetReuseAddrTCPUDP();
	int connectionGetStateTCPUDP();
	int zeroBufferTCPUDP();

	static int getPort();
public:

        int             packet_len;             /* points to the last valid char in the buffer */
        char            buffer[BUFFERSIZE];     /* sender/reader buffer */
        String		d_name;
        int		d_number;
        int      	ipproto; 
        int		l_ip;			/* local ip     */
        int		l_mask;			/* local mask   */
        int		l_port;			/* local port   */
        int		p_ip;			/* foreign ip   */
        int		p_mask;			/* foreign mask */
        int		p_port;			/* foreign port */
        bool            direction;		/* valid vor TCP, direction of first SYN 0=local->peer 1=peer->local */
	bool		hardUDP; 		/* bind and connect for UDP */
	bool		RTCP; 			/* the port will be bound to the next free odd port, space for the even RTP port will be left */
	bool		initiator;		/* 0=this socket is connected with the initiator of a session, 1=oposite */

	Connection();
	Connection(Proxy* systemFD_t);
	Connection(Connection *c);
	Connection(Connection *c,Proxy* systemFD_t);
	~Connection();

	int get_fd();
	int set_fd(int fd);
	int connectionGetState();

	String getDescript();

	int connectionRead(int len);
	int connectionWrite(Connection *cin);
	int connectionListen();
	Connection* connectionAccept();
	int connectionConnect();
	int connectionClose();

	int connectionSetNonBlock();
	int connectionSetLinger();
	int connectionSetReuseAddr();

	int zeroBuffer();
	void setSystemFD(Proxy* systemFD_t);
	bool toState(states s);

        static int gethostip(const char *name);

};





#endif
