/* Copyright (C) 2000 KOM/Darmstadt University of Technology
 *
 * You are allowed to use all other parts of the code under the following terms:
 *
 * For non-commercial use, code may be used in unmodified form provided
 * that this copyright notice and this permission notice appear in
 * supporting documentation.
 *
 * This software is provided "as is" and without any express or implied
 * warranties, including, without limitation, the implied warranty of
 * fitness for a particular purpose.
 *
 * The code may be subjected to the GNU General Public License, Version 2,
 * and re-distributed under the terms of this license.
 *
 * Commercial use other than under the terms of the GNU General Public
 * License is allowed only after express negotiation of conditions
 * with the authors.
 */




#include "FirewallControl.h"
#include "Log.h"
#include "SignalHandler.h"
#include "Fwfil.h"
#include "Connection.h"

#include <unistd.h> /* sleep */
#include <signal.h> /* signals */


FirewallControl::FirewallControl() : size(0){
	INF((__FILE__,__FUNCTION__,"firewall control started\n"))
	for(int i=0;i<MAXFWRULES;++i){
                fwfilList[i]=NULL;
        }
}

FirewallControl::~FirewallControl() {
	INF((__FILE__,__FUNCTION__,"waiting for firewall control thread to terminate\n"))
	join();
	fw_del_chl();
	INF((__FILE__,__FUNCTION__,"firewall control terminated\n"))
}

int FirewallControl::fw_del_fil(Fwfil*  fwfilp){ /* remove one rule */
	return fw_del_fil(fwfilp->id);
}

int FirewallControl::fw_del_fil(int id){ /* remove one rule */
	if ((id<0)||(id>MAXFWRULES)) FAT((__FILE__,__FUNCTION__,"filter id out of scope: %d\n",id))
	if (fwfilList[id]!=NULL){
		delete (fwfilList[id]);
		fwfilList[id]=NULL;
		--size;
		return(0);
	}
        ERR((__FILE__,__FUNCTION__,"could not find session in list\n"))
        return(-1);
}

int FirewallControl::fw_add_fil(Fwfil*  fwfilp){ /* add one rule  */
        for(int i=0;i<MAXFWRULES;++i){
                if (fwfilList[i]==NULL){
			fwfilList[i]=fwfilp;
			fwfilp->id=i;
                        ++size;
                        return(0);
                }
        }
        FAT((__FILE__,__FUNCTION__,"rule list reached MAXFWRULES, could not handle new rule\n"))
        return(-1);
}

int FirewallControl::fw_del_chl(){
	MUTEX_AUTO_LOCK(fwcLock);
	LOG((__FILE__,__FUNCTION__,9,"flushing rules\n"))
        for(int i=0;i<MAXFWRULES;++i){
                if (fwfilList[i]!=NULL){
			fwfilList[i]->log(9);
			if (fw_del_chl_x(fwfilList[i])<0){
                        	WRN((__FILE__,__FUNCTION__,"could not remove rule from filter device\n"))
                	}
                	if (fw_del_fil(fwfilList[i])<0){
                        	FAT((__FILE__,__FUNCTION__,"could not remove rule from filter cache\n"))
                	}
		}
	}
	LOG((__FILE__,__FUNCTION__,9,"rules flushed\n"))
	return(0);
}
int FirewallControl::fw_del_chl(int id){
	MUTEX_AUTO_LOCK(fwcLock);
	if ((id<0)||(id>MAXFWRULES)) FAT((__FILE__,__FUNCTION__,"filter id out of scope: %d\n",id))
	LOG((__FILE__,__FUNCTION__,9,"flushing rule, ID: %d\n", id))
	if (fwfilList[id]!=NULL){
		fwfilList[id]->log(9);
		if (fw_del_chl_x(fwfilList[id])<0){
			WRN((__FILE__,__FUNCTION__,"could not remove rule from filter device\n"))
		}
		if (fw_del_fil(fwfilList[id])<0){
			FAT((__FILE__,__FUNCTION__,"could not remove rule from filter cache\n"))
		}
	}
	LOG((__FILE__,__FUNCTION__,9,"rule flushed\n"))
	return(0);
}

int FirewallControl::fw_del_chl(Fwfil *fwfilp){
	return fw_del_chl(fwfilp->id);
}

int FirewallControl::fw_get_chl(Fwfil *fwfilp){
	MUTEX_AUTO_LOCK(fwcLock);
	LOG((__FILE__,__FUNCTION__,9,"adding rule\n"))
	if (fw_add_fil(fwfilp)<0){
		FAT((__FILE__,__FUNCTION__,"could not add rule to filter cache\n"))
	}
	if (fw_get_chl_x(fwfilp)<0){
		WRN((__FILE__,__FUNCTION__,"could not add rule to filter device\n"))
	}
	fwfilp->log(9);
	LOG((__FILE__,__FUNCTION__,9,"rule added\n"))
	return (0);
}

int FirewallControl::fw_get_dst(Connection *sc,Connection *dc){
	MUTEX_AUTO_LOCK(fwcLock);
	if (fw_get_dst_x(sc,dc)<0){
		return(-1);
	}
	return (0);
}

int FirewallControl::log(int loglevel){
        LOG((__FILE__,__FUNCTION__,loglevel,"firewallcontrol       thread# %d filtersize# %d/%d type# %s\n",myID(),size,MAXFWRULES,type.c_str()))
        return(0);
}

void FirewallControl::main() {
        LOG((__FILE__,__FUNCTION__,10,"starting FirewallControl loop\n"))
	SignalHandler::unblock(SIGUSR2);
	while ( !end ) {
                sleep(1);
		/* here should be a check for timed out rules */
        }
	LOG((__FILE__,__FUNCTION__,10,"leaving FirewallControl loop\n"))
}


