/* Copyright (C) 2000 KOM/Darmstadt University of Technology
 *
 * You are allowed to use all other parts of the code under the following terms:
 *
 * For non-commercial use, code may be used in unmodified form provided
 * that this copyright notice and this permission notice appear in
 * supporting documentation.
 *
 * This software is provided "as is" and without any express or implied
 * warranties, including, without limitation, the implied warranty of
 * fitness for a particular purpose.
 *
 * The code may be subjected to the GNU General Public License, Version 2,
 * and re-distributed under the terms of this license.
 *
 * Commercial use other than under the terms of the GNU General Public
 * License is allowed only after express negotiation of conditions
 * with the authors.
 */

#include "Listener.h"
#include "Session.h"
#include "Proxy.h"
#include "Log.h"
#include "Conf.h"
#include "FirewallControl.h"
#include "SignalHandler.h"
#include "ProxyList.h"

#include <signal.h>     /* signals */
#include <unistd.h>     /* sleep */

Listener::Listener(String type_t):c_l(NULL),c_s(NULL){
	type=type_t;	
        INF((__FILE__,__FUNCTION__,"listener started (type=%s)\n",type.c_str()))
	c_l = new Connection();
	c_l->d_name	= type+"-CONTROL-SRC";
	c_l->l_ip	= Global::getGlobal()->getConf()->getValAsIP("LOCALIP");
	c_l->l_mask	= 0xffffffff;
	c_l->l_port	= Global::getGlobal()->getConf()->getValAsInt(type+"_PORT");
	c_l->ipproto	= Global::getGlobal()->getConf()->getValAsInt(type+"_PROTO");
	c_l->direction	= 1;
}

Listener::~Listener() {
	this->stop();
	INF((__FILE__,__FUNCTION__,"waiting for listener thread to terminate\n"))
	join();
	delete c_l;
        INF((__FILE__,__FUNCTION__,"listener terminated (type=%s)\n",type.c_str()))
}

int Listener::queueInProxy(){
	Session    *s;

	if(Global::getGlobal()->getProxyList()->hasSpaceLeft()){
		LOG((__FILE__,__FUNCTION__,5,"new session\n"))
		s=new Session(type,c_s); 
		if((Global::getGlobal()->getProxyList()->queueSession(s))<0){
			ERR((__FILE__,__FUNCTION__,"queueing new session\n"))
			delete s;
			return (-1);
		}
		return(0);
	}
	return(-1);
}

int Listener::log(int loglevel){
        LOG((__FILE__,__FUNCTION__,loglevel,"listener              thread# %d port# %d type# %s\n",myID(),Global::getGlobal()->getConf()->getValAsInt(type+"_PORT"),type.c_str()))
        return(0);
}


void Listener::main() {
	LOG((__FILE__,__FUNCTION__,10,"starting Listener loop\n"))

	SignalHandler::unblock(SIGUSR2);

	if ((c_l->connectionListen())<0){
		ERR((__FILE__,__FUNCTION__,"listen\n"))
		this->stop();
	}

        while ( !end ) {
		if ((c_s=c_l->connectionAccept())==NULL){
                        ERR((__FILE__,__FUNCTION__,"accept\n"))
                        this->stop();
                }
		if ( c_s==c_l ) {
			c_s==NULL;
                        ERR((__FILE__,__FUNCTION__,"accept: the listener was terminated (UDP used?)\n"))
                        this->stop();
		}
		if ( c_s!=NULL ) {
			if (queueInProxy()<0){
				WRN((__FILE__,__FUNCTION__,"could not handle new connection, sleeping 1 sec\n"))
				sleep(1);
			}
		}
        }
	LOG((__FILE__,__FUNCTION__,10,"leaving Listener loop\n"))
}


