/* Copyright (C) 2000 KOM/Darmstadt University of Technology
 *
 * You are allowed to use all other parts of the code under the following terms:
 *
 * For non-commercial use, code may be used in unmodified form provided
 * that this copyright notice and this permission notice appear in
 * supporting documentation.
 *
 * This software is provided "as is" and without any express or implied
 * warranties, including, without limitation, the implied warranty of
 * fitness for a particular purpose.
 *
 * The code may be subjected to the GNU General Public License, Version 2,
 * and re-distributed under the terms of this license.
 *
 * Commercial use other than under the terms of the GNU General Public
 * License is allowed only after express negotiation of conditions
 * with the authors.
 */

#include "ListenerList.h"
#include "SignalHandler.h"
#include "Log.h"
#include "Conf.h"

#include <signal.h>     /* signals */
#include <unistd.h>     /* sleep */

ListenerList::ListenerList():size(0){
	INF((__FILE__,__FUNCTION__,"listenerList started\n"))

	for(int i=0;i<MAXLISTENERS;++i){
                listenerList[i]=NULL;
        }
	for (int i=0;i<MAXLISTENERS;++i){
		String listener;
		listener = listener + "LISTENER_" + i;  
		if ((Global::getGlobal()->getConf()->getValAsString(listener))!=String("")) {
			addListener(new Listener(Global::getGlobal()->getConf()->getValAsString(listener)));
		}
        }
}

ListenerList::~ListenerList(){
        INF((__FILE__,__FUNCTION__,"waiting for listenerList thread to terminate\n"))
	join();
	delListener();
	INF((__FILE__,__FUNCTION__,"listenerList terminated\n"))
}

int ListenerList::addListenerInternal(Listener* l){
	for(int i=0;i<MAXLISTENERS;++i){
		if (listenerList[i]==NULL) {
			l->run();
			listenerList[i]=l;
			++size;
			return(0);
		}
	}
	WRN((__FILE__,__FUNCTION__,"listenerList overloaded, could not handle new listener\n"))
        return(-1);
}

int ListenerList::addListener(Listener* l){
        MUTEX_AUTO_LOCK(listenerListLock);
        return addListenerInternal(l);
}

int ListenerList::delListenerInternal(Listener* l){
	for(int i=0;i<MAXLISTENERS;++i){
                if (listenerList[i]==l){
                        delete(l);
                        listenerList[i]=NULL;
                        --size;
                        return(0);
                }
        }
        ERR((__FILE__,__FUNCTION__,"could not find listener in listenerList\n"))
        return(-1);
}

int ListenerList::delListener(Listener* l){
        MUTEX_AUTO_LOCK(listenerListLock);
	return delListenerInternal(l);
}

int ListenerList::delListenerInternal(){
        for(int i=0;i<MAXLISTENERS;++i){
                if (listenerList[i]!=NULL){
                        delete listenerList[i];
                        listenerList[i]=NULL;
                }
        }
        size=0;
        return(0);
}

int ListenerList::delListener(){
        MUTEX_AUTO_LOCK(listenerListLock);
	return delListenerInternal();
}

int ListenerList::getSize(){
	return(size);
}

int ListenerList::log(int loglevel){
	MUTEX_AUTO_LOCK(listenerListLock);
	LOG((__FILE__,__FUNCTION__,loglevel,"listenerlist          thread# %d listeners# %d/%d\n",myID(),size,MAXLISTENERS))
	for(int i=0;i<MAXLISTENERS;++i){
		if (listenerList[i]!=NULL) listenerList[i]->log(loglevel);
	}
        return(0);
}

void ListenerList::main() {
        LOG((__FILE__,__FUNCTION__,10,"starting ListenerList loop\n"))
        SignalHandler::unblock(SIGUSR2);
        while ( !end ) {
		sleep(1);
	}
	LOG((__FILE__,__FUNCTION__,10,"leaving ListenerList loop\n"))
}


