/* Copyright (C) 2000 KOM/Darmstadt University of Technology
 *
 * You are allowed to use all other parts of the code under the following terms:
 *
 * For non-commercial use, code may be used in unmodified form provided
 * that this copyright notice and this permission notice appear in
 * supporting documentation.
 *
 * This software is provided "as is" and without any express or implied
 * warranties, including, without limitation, the implied warranty of
 * fitness for a particular purpose.
 *
 * The code may be subjected to the GNU General Public License, Version 2,
 * and re-distributed under the terms of this license.
 *
 * Commercial use other than under the terms of the GNU General Public
 * License is allowed only after express negotiation of conditions
 * with the authors.
 */




#include "Log.h"
#include <stdlib.h> /* exit */  
#include <unistd.h> /* getpid */
#include <iostream.h>

Log::Log(int logType_t, int logLevel_t, String logDir_t):logType(logType_t), logLevel(logLevel_t), logDir(logDir_t), fp(NULL) {
	if ((logType_t>2) || (logType_t<0)) {
		logType=0;
	}
	if (logLevel<0) logLevel=3;
	if ((logType!=0) && (fp==NULL)) openlog();

	printf("logger started with logtype: %d loglevel: %d logdir: %s\n",logType,logLevel,logDir.c_str());
}

Log::~Log() {
	if ((logType!=0) && (fp!=NULL)) closelog();
	printf("logger terminated\n");
}

int Log::setLogtarget(int logType_t,String logDir_t){
	MUTEX_AUTO_LOCK(logLock);
	if ((logType_t>2) || (logType_t<0)) {
		logType=0;
	}
	else {
		logType=logType_t;
	}
	logDir=logDir_t;
	closelog();
	if ((logType!=0) && (fp==NULL)) openlog();

	return(0);
}

int Log::setLoglevel(int logLevel_t){
	MUTEX_AUTO_LOCK(logLock);
	if (logLevel_t<0) {
		logLevel=3;
		return(-1);
	}
	else{
		logLevel=logLevel_t;
	}
	return(0);
}


int Log::openlog(){
	String fileName;
	char PID[10];

        sprintf(PID,"%d",getpid());
	fileName=logDir+"log_proxyd_pid"+PID;

	if ((fp = fopen(fileName.c_str(), "w"))==NULL) {
			printf("could not open %s, switching to console\n",fileName.c_str());
			logType=0;
			return(-1);
	}
	return (1);
}

int Log::closelog(){
	if ((logType!=0) && (fp!=NULL)) fclose(fp);
	return(0);
}

int Log::lograwprint(char *format,char *fmt,va_list ap){
        switch (logType) {
                case 0:
                        printf(format);
                        vprintf(fmt,ap);
                        fflush(stdout);
                        break;
                case 1:
                        fprintf(fp,format);
                        vfprintf(fp,fmt,ap);
                        fflush(fp);
                        break;
                case 2:
                        printf(format);
                        vprintf(fmt,ap);
                        fflush(stdout);
                        fprintf(fp,format);
                        vfprintf(fp,fmt,ap);
                        fflush(fp);
                        break;
                default:
                        break;
        }
        return (1);    
}


int Log::logprint(char *format,const char *file,const char *function,char *fmt,va_list ap){
	switch (logType) {
		case 0:
			printf(format,getpid(),file,function);	
			vprintf(fmt,ap);
			fflush(stdout);
			break;
		case 1:
			fprintf(fp,format,getpid(),file,function);	
			vfprintf(fp,fmt,ap);
			fflush(fp);	
			break;
		case 2:
			printf(format,getpid(),file,function);	
			vprintf(fmt,ap);
			fflush(stdout);
			fprintf(fp,format,getpid(),file,function);	
			vfprintf(fp,fmt,ap);
			fflush(fp);	
			break;
		default:
			break;
	}
 	return (1);	
}

int Log::fat(const char *file,const char *function,char *fmt, ...){
	MUTEX_AUTO_LOCK(logLock);
	va_list ap;
	va_start(ap,fmt);

	logprint("FATAL   PID %5.0d FILE %-10.10s FUNCTION %-10.10s : FATAL ERROR OCCURED\n error: ",file,function,fmt,ap);
	//kill(0, SIGTERM);
	exit(-1);
 	return (1);	
}
int Log::err(const char *file,const char *function,char *fmt, ...){
	MUTEX_AUTO_LOCK(logLock);
	va_list ap;
	va_start(ap,fmt);

	logprint("ERROR   PID %5.0d FILE %-10.10s FUNCTION %-10.10s : ",file,function,fmt,ap);
 	return (1);	
}
int Log::wrn(const char *file,const char *function,char *fmt, ...){
	MUTEX_AUTO_LOCK(logLock);
	va_list ap;
	va_start(ap,fmt);

	logprint("WARNING PID %5.0d FILE %-10.10s FUNCTION %-10.10s : ",file,function,fmt,ap);
 	return (1);	
}
int Log::inf(const char *file,const char *function,char *fmt, ...){
	MUTEX_AUTO_LOCK(logLock);
	va_list ap;
	va_start(ap,fmt);

	logprint("INFO    PID %5.0d FILE %-10.10s FUNCTION %-10.10s : ",file,function,fmt,ap);
 	return (1);	
}

int Log::log(const char *file,const char *function,int loglevel,char *fmt, ...){
	MUTEX_AUTO_LOCK(logLock);
	va_list ap;
	va_start(ap,fmt);

	if((logLevel)>=loglevel)
		logprint("        PID %5.0d FILE %-10.10s FUNCTION %-10.10s : ",file,function,fmt,ap);
 	return (1);	
}

int Log::lograw(char *fmt, ...){
	MUTEX_AUTO_LOCK(logLock);
	va_list ap;
	va_start(ap,fmt);


	lograwprint("",fmt,ap);

 	return (1);	
}
