/* Copyright (C) 2000 KOM/Darmstadt University of Technology
 *
 * You are allowed to use all other parts of the code under the following terms:
 *
 * For non-commercial use, code may be used in unmodified form provided
 * that this copyright notice and this permission notice appear in
 * supporting documentation.
 *
 * This software is provided "as is" and without any express or implied
 * warranties, including, without limitation, the implied warranty of
 * fitness for a particular purpose.
 *
 * The code may be subjected to the GNU General Public License, Version 2,
 * and re-distributed under the terms of this license.
 *
 * Commercial use other than under the terms of the GNU General Public
 * License is allowed only after express negotiation of conditions
 * with the authors.
 */

#include "ProxyList.h"
#include "SignalHandler.h"
#include "Log.h"
#include "Conf.h"

#include <signal.h>     /* signals */
#include <unistd.h>	/* sleep */

ProxyList::ProxyList():size(0),fillmethod(0){
	INF((__FILE__,__FUNCTION__,"proxyList started\n"))

	if ((Global::getGlobal()->getConf()->getValAsString("FILLMETHOD"))==String("FILLFIRST")) fillmethod=1;
	if ((Global::getGlobal()->getConf()->getValAsString("FILLMETHOD"))==String("FILLEQUAL")) fillmethod=2;

	if (fillmethod==0) fillmethod=2;

	for(int i=0;i<MAXPROXIES;++i){
                proxyList[i]=NULL;
        }
	for (int i=0;i<MAXPROXIES;++i){
		addProxy(new Proxy());
        }
}

ProxyList::~ProxyList(){
        INF((__FILE__,__FUNCTION__,"waiting for proxyList thread to terminate\n"))
	join();
	delProxy();
	INF((__FILE__,__FUNCTION__,"proxyList terminated\n"))
}

int ProxyList::queueSessionInProxy(Session *s,Proxy* p){
	int transferFD;

	LOG((__FILE__,__FUNCTION__,5,"queueing new session\n"));
	transferFD=p->getTransferFD();
	if (write(transferFD,&s,sizeof(s))!=sizeof(s)){
		FAT((__FILE__,__FUNCTION__,"queueing new session\n"));
		return(-1);
	}
	return(0);
}

int ProxyList::queueSessionFillEqual(Session *s){
	int min=-1;

	/*determine proxy with the slightest load*/
	for(int i=0;i<MAXPROXIES;++i){
		if (proxyList[i]!=NULL) {
			if (min==(-1)) min=i;
			if((proxyList[i]->getSize())<(proxyList[min]->getSize())) min=i;
		}
	}

	if ((proxyList[min]->hasSpaceLeft())) {
        	return queueSessionInProxy(s,proxyList[min]);
	}
	else {
		WRN((__FILE__,__FUNCTION__,"too many sessions\n"));
		return(-1);
	}
}

int ProxyList::queueSessionFillFirst(Session *s){
	for(int i=0;i<MAXPROXIES;++i){
		if ((proxyList[i]!=NULL) && (proxyList[i]->hasSpaceLeft())) {
			return queueSessionInProxy(s,proxyList[i]);
		}
	}

	WRN((__FILE__,__FUNCTION__,"too many sessions\n"));
	return(-1);
}

int ProxyList::queueSession(Session *s){
        MUTEX_AUTO_LOCK(proxyListLock);
	if (fillmethod==1) return queueSessionFillFirst(s);
	if (fillmethod==2) return queueSessionFillEqual(s);
	FAT((__FILE__,__FUNCTION__,"unknown FILLMETHOD\n"));
        return (-1);
}

int ProxyList::addProxyInternal(Proxy* p){
	for(int i=0;i<MAXPROXIES;++i){
		if (proxyList[i]==NULL) {
			p->run();
			proxyList[i]=p;
			++size;
			return(0);
		}
	}
	WRN((__FILE__,__FUNCTION__,"proxyList overloaded, could not handle new proxy\n"))
        return(-1);
}

int ProxyList::addProxy(Proxy* p){
        MUTEX_AUTO_LOCK(proxyListLock);
        return addProxyInternal(p);
}

int ProxyList::delProxyInternal(Proxy* p){
	for(int i=0;i<MAXPROXIES;++i){
                if (proxyList[i]==p){
                        delete(p);
                        proxyList[i]=NULL;
                        --size;
                        return(0);
                }
        }
        ERR((__FILE__,__FUNCTION__,"could not find proxy in proxyList\n"))
        return(-1);
}

int ProxyList::delProxy(Proxy* p){
        MUTEX_AUTO_LOCK(proxyListLock);
	return delProxyInternal(p);
}

int ProxyList::delProxyInternal(){
        for(int i=0;i<MAXPROXIES;++i){
                if (proxyList[i]!=NULL){
                        delete proxyList[i];
                        proxyList[i]=NULL;
                }
        }
        size=0;
        return(0);
}

int ProxyList::delProxy(){
        MUTEX_AUTO_LOCK(proxyListLock);
	return delProxyInternal();
}

int ProxyList::getSize(){
	return(size);
}

bool ProxyList::hasSpaceLeft(){
	MUTEX_AUTO_LOCK(proxyListLock);
	for(int i=0;i<MAXPROXIES;++i){
		if ((proxyList[i]!=NULL) && (proxyList[i]->hasSpaceLeft())) return (true); 
	}
        return(false);
}

int ProxyList::log(int loglevel){
	MUTEX_AUTO_LOCK(proxyListLock);
	LOG((__FILE__,__FUNCTION__,loglevel,"proxylist             thread# %d proxies# %d/%d\n",myID(),size,MAXPROXIES))
	for(int i=0;i<MAXPROXIES;++i){
		if (proxyList[i]!=NULL) proxyList[i]->log(loglevel);
	}
        return(0);
}

void ProxyList::main() {
        LOG((__FILE__,__FUNCTION__,10,"starting ProxyList loop\n"))
        SignalHandler::unblock(SIGUSR2);
        while ( !end ) {
		sleep(1);
	}
	LOG((__FILE__,__FUNCTION__,10,"leaving ProxyList loop\n"))
}


