/* Copyright (C) 2000 KOM/Darmstadt University of Technology
 *
 * You are allowed to use all other parts of the code under the following terms:
 *
 * For non-commercial use, code may be used in unmodified form provided
 * that this copyright notice and this permission notice appear in
 * supporting documentation.
 *
 * This software is provided "as is" and without any express or implied
 * warranties, including, without limitation, the implied warranty of
 * fitness for a particular purpose.
 *
 * The code may be subjected to the GNU General Public License, Version 2,
 * and re-distributed under the terms of this license.
 *
 * Commercial use other than under the terms of the GNU General Public
 * License is allowed only after express negotiation of conditions
 * with the authors.
 */

#include "Proxyd.h"
#include "Proxy.h"
#include "Log.h"
#include "Conf.h"
#include "Listener.h"
#include "FirewallControl.h"
#include "SignalHandler.h"
#include "ProxyList.h"
#include "ListenerList.h"

#include <unistd.h>     /* sleep */
#include <dlfcn.h> /* dl functions */


#ifdef STATIC_PROXY
#include "testfw/FirewallControlTestfw.h"
#endif

int proxyd_start(const char* configfile){
        Global::getGlobal()->setLog(new Log(0,3,""));

        INF((__FILE__,__FUNCTION__,"starting conf\n"))
        Global::getGlobal()->setConf(new Conf(configfile));
        Global::getGlobal()->getConf()->read();
        Global::getGlobal()->getLog()->setLogtarget(Global::getGlobal()->getConf()->getValAsInt("LOGTARGET"),Global::getGlobal()->getConf()->getValAsString("LOGDIR"));
        Global::getGlobal()->getLog()->setLoglevel(Global::getGlobal()->getConf()->getValAsInt("LOGLEVEL"));
	//Global::getGlobal()->getConf()->log();

	/* print options */
        INF((__FILE__,__FUNCTION__,"----------------------------------------\n"))
        INF((__FILE__,__FUNCTION__,"system:                %s\n",VERSION))
	INF((__FILE__,__FUNCTION__,"local IP address used: %s\n",(Global::getGlobal()->getConf()->getValAsString("LOCALIP")).c_str()))
	INF((__FILE__,__FUNCTION__,"port range used:       %d - %d\n",MINPORT,MAXPORT))
	INF((__FILE__,__FUNCTION__,"firewall module:       %s\n",(Global::getGlobal()->getConf()->getValAsString("FIREWALL")).c_str()))
	String proxies;
	for (int i=0;i<MAXLISTENERS;++i){
                String listener;
                listener = listener + "LISTENER_" + i;
                if ((Global::getGlobal()->getConf()->getValAsString(listener))!=String("")) {
                        proxies=proxies+" "+(Global::getGlobal()->getConf()->getValAsString(listener));
                }
        }
	for (int i=0;i<MAXLISTENERS;++i){
                String listener;
                listener = listener + "SESSION_" + i;
                if ((Global::getGlobal()->getConf()->getValAsString(listener))!=String("")) {
                        proxies=proxies+" "+(Global::getGlobal()->getConf()->getValAsString(listener));
                }
        }
	INF((__FILE__,__FUNCTION__,"proxy modules:        %s\n",proxies.c_str()))
        INF((__FILE__,__FUNCTION__,"----------------------------------------\n"))

	String firewall;
	firewall = Global::getGlobal()->getConf()->getValAsString("FIREWALL");
        INF((__FILE__,__FUNCTION__,"starting firewall controler (type=%s)\n",firewall.c_str()))
#ifdef STATIC_PROXY
        Global::getGlobal()->setFirewallControl(new FirewallControlTestfw(firewall));
#else
        String dll;
        dll = Global::getGlobal()->getConf()->getValAsString(firewall+"_PATH") + "lib" + Global::getGlobal()->getConf()->getValAsString(firewall+"_NAME") + ".so";
        void *hndl = dlopen(dll.c_str(), RTLD_NOW);
        if(hndl == NULL){
                ERR((__FILE__,__FUNCTION__,"can not load firewall control lib: %s %s\n",dll.c_str(), dlerror()))
                return(-1);
        }
        void *mkr = dlsym(hndl, "maker");
        Global::getGlobal()->setFirewallControl(((FWMakerType)(mkr))(firewall));
#endif
        Global::getGlobal()->getFirewallControl()->run();
	
        INF((__FILE__,__FUNCTION__,"starting proxylist\n"))
        Global::getGlobal()->setProxyList(new ProxyList());
        Global::getGlobal()->getProxyList()->run();

        INF((__FILE__,__FUNCTION__,"starting predefined sessions\n"))
	for (int i=0;i<MAXLISTENERS;++i){
                String listener;
                listener = listener + "SESSION_" + i;
                if ((Global::getGlobal()->getConf()->getValAsString(listener))!=String("")) {
			Session    *s;
        		if(Global::getGlobal()->getProxyList()->hasSpaceLeft()){
                		LOG((__FILE__,__FUNCTION__,5,"new session\n"))
                		s=new Session(Global::getGlobal()->getConf()->getValAsString(listener),new Connection());
                		if((Global::getGlobal()->getProxyList()->queueSession(s))<0){
                        		ERR((__FILE__,__FUNCTION__,"queueing new session\n"))
                        		delete s;
                		}
				else {	
					INF((__FILE__,__FUNCTION__,"session started (type=%s)\n",(Global::getGlobal()->getConf()->getValAsString(listener)).c_str()))
				}
        		}

                }
        }

        INF((__FILE__,__FUNCTION__,"starting listener\n"))
        Global::getGlobal()->setListenerList(new ListenerList());
        Global::getGlobal()->getListenerList()->run();


	return(0);
}

int proxyd_stop(){
       	INF((__FILE__,__FUNCTION__,"shutting down system\n"))
	if (Global::getGlobal()->getListenerList()!=NULL){
        	INF((__FILE__,__FUNCTION__,"shutting down listenerlist\n"))
		Global::getGlobal()->getListenerList()->stop();
		Global::getGlobal()->setListenerList(NULL);
	}

	if (Global::getGlobal()->getProxyList()!=NULL){
        	INF((__FILE__,__FUNCTION__,"shutting down proxylist\n"))
		Global::getGlobal()->getProxyList()->stop();
		Global::getGlobal()->setProxyList(NULL);
	}

	if (Global::getGlobal()->getFirewallControl()!=NULL){
        	INF((__FILE__,__FUNCTION__,"shutting down firewall control\n"))
		Global::getGlobal()->getFirewallControl()->stop();
		Global::getGlobal()->setFirewallControl(NULL);
	}

	if (Global::getGlobal()->getConf()!=NULL){
        	INF((__FILE__,__FUNCTION__,"shutting down conf\n"))
		Global::getGlobal()->setConf(NULL);
	}

	if (Global::getGlobal()->getLog()!=NULL){
        	INF((__FILE__,__FUNCTION__,"shutting down log\n"))
		Global::getGlobal()->setLog(NULL);
	}

	return(0);
}

void usage(const char* program){
	printf("usage: %s [-f file]\n",program);
}

int main(int argc, char **argv){
	int c;
	const char* configfile = "./proxyd.conf";
	

	/* read arguments */
	while ((c = getopt(argc, argv, "Df:")) != -1) {
                switch (c) {
                        case 'f' :
                                configfile = optarg;
                                break;
                        default  :
				usage( argv[0] );
				return(0);
                }
	}


	/* block signal handlers for all following threads */
	SignalHandler::install();
	SignalHandler::blockAll();

	if (proxyd_start(configfile)<0) {
		proxyd_stop();
		//dmalloc_shutdown();
		/* exit */
		return 0;
	}

	/* unblock signal handlers for main*/
	SignalHandler::unblockAll();

	while (!(Global::getGlobal()->getEnd())) {
		sleep(600);
		if (Global::getGlobal()->getListenerList()!=NULL)         {
			Global::getGlobal()->getListenerList()->log(10);
		}
       		if (Global::getGlobal()->getFirewallControl()!=NULL)  {
			Global::getGlobal()->getFirewallControl()->log(10);
		}	
		if (Global::getGlobal()->getProxyList()!=NULL)        {
			Global::getGlobal()->getProxyList()->log(10);
		} 
        }

	proxyd_stop();
	//dmalloc_shutdown();
	/* exit */
	return 0;
}

