/* Copyright (C) 2000 KOM/Darmstadt University of Technology
 *
 * You are allowed to use all other parts of the code under the following terms:
 *
 * For non-commercial use, code may be used in unmodified form provided
 * that this copyright notice and this permission notice appear in
 * supporting documentation.
 *
 * This software is provided "as is" and without any express or implied
 * warranties, including, without limitation, the implied warranty of
 * fitness for a particular purpose.
 *
 * The code may be subjected to the GNU General Public License, Version 2,
 * and re-distributed under the terms of this license.
 *
 * Commercial use other than under the terms of the GNU General Public
 * License is allowed only after express negotiation of conditions
 * with the authors.
 */


#include "Session.h"
#include "Proxy.h"
#include "Log.h"
#include "Conf.h"
#include "String.h"

#include <dlfcn.h> /* dl functions */

#ifdef STATIC_PROXY
#include "test/TestParser.h"
#endif


Session::Session(String type_t):proxy(NULL), parser(NULL), sessionID(-1) , sessionEnd(false) {
	type=type_t;
	data.type=type;
	TimeValue::getCurrentSystemTime(startTime);
	if((Global::getGlobal()->getConf()->getValAsInt(type + "_TIMEOUT"))<0) {
		timeoutTime=TimeValue(0,0);
	}
	else {
		timeoutTime=TimeValue(Global::getGlobal()->getConf()->getValAsInt(type + "_TIMEOUT"),0);
	}
	for(int i=0;i<MAXCONNECTIONS;++i){
		con[i][0]=NULL;
		con[i][1]=NULL;
	}
        sessionInit();
}

Session::Session(String type_t,Connection *c):proxy(NULL), parser(NULL), sessionID(-1) , sessionEnd(false) {
	type=type_t;
	data.type=type;
	TimeValue::getCurrentSystemTime(startTime);
	if((Global::getGlobal()->getConf()->getValAsInt(type + "_TIMEOUT"))<0) {
		timeoutTime=TimeValue(0,0);
	}
	else {
		timeoutTime=TimeValue(Global::getGlobal()->getConf()->getValAsInt(type + "_TIMEOUT"),0);
	}
	for(int i=0;i<MAXCONNECTIONS;++i){
		con[i][0]=NULL;
		con[i][1]=NULL;
	}
	con[0][0]=c;
	data.srcIP=c->p_ip;
	con[0][1]=new Connection();
        sessionInit();
}

Session::~Session() {
	if (parser!=NULL){
                LOG((__FILE__,__FUNCTION__,5,"shutting down parser\n"))
                delete parser;
        }
	for(int i=0;i<MAXCONNECTIONS;++i){
		delConnection(i);
	}
        LOG((__FILE__,__FUNCTION__,5,"Session# %d deleted\n",sessionID))
}

String Session::getDescript(){
        /* [session %d in thread# %d] */
        String d;

        d = d + "[session: "     + sessionID;
	if (proxy!=NULL) d = d + " in proxy thread "    + proxy->myID();
	else		 d = d + " ";
	d = d + " timeout:("    + timeoutTime.tv_sec;
        d = d + ":"             + timeoutTime.tv_usec;
        d = d + ")]";

        return (d);
}


void Session::setSessionEnd(){
        sessionEnd=true;
        return;
}

bool Session::getSessionEnd(){
        return sessionEnd;
}

int Session::getSessionID(){
        return sessionID;
}
int Session::setSessionID(int ID){
	sessionID=ID;
        return sessionID;
}

const TimeValue& Session::getStartTime(){
        return startTime;
}

const TimeValue& Session::getTimeoutTime(){
        return timeoutTime;
}

void Session::setProxy(Proxy* proxy_t){
	proxy=proxy_t;
	for(int i=0;i<MAXCONNECTIONS;++i){
		if (con[i][0]!=NULL) con[i][0]->setSystemFD(proxy);
		if (con[i][1]!=NULL) con[i][1]->setSystemFD(proxy);
	}
}
int Session::addConnection(){
	for(int i=0;i<MAXCONNECTIONS;++i){
		if ((con[i][0]==NULL) && (con[i][1]==NULL)) {
			con[i][0]=new Connection();
			con[i][1]=new Connection();
			con[i][0]->setSystemFD(proxy);
			con[i][1]->setSystemFD(proxy);
			return(i);
		}
	}
	return(-1);
}

int Session::delConnection(int i){
	if (con[i][0]!=NULL) {delete con[i][0];con[i][0]=NULL;}
	if (con[i][1]!=NULL) {delete con[i][1];con[i][1]=NULL;}
	return(0);
}

int Session::sessionInit(){
#ifdef STATIC_PROXY
	parser=new TestParser;
#else
	String dll;
        dll = Global::getGlobal()->getConf()->getValAsString(type+"_PATH") + "lib" + Global::getGlobal()->getConf()->getValAsString(type+"_NAME") + ".so";
        void *hndl = dlopen(dll.c_str(), RTLD_NOW);
        if(hndl == NULL){
                FAT((__FILE__,__FUNCTION__,"can not load parser lib: %s %s\n",dll.c_str(), dlerror()))
                return(-1);
        }
        void *mkr = dlsym(hndl, "maker");
        parser = ((PMakerType)(mkr))(this,type);
#endif
	if((parser->connectDestinationFAST())<0) sessionEnd=true;
	return(0);
}

int Session::sessionRead(fd_set *s_fd_set){
        int ret=0;

        for(int j=0;j<MAXCONNECTIONS;++j){
                if ((con[j][0]!=NULL) && (con[j][1]!=NULL) && (con[j][0]->connectionGetState()==CONNECT) && (FD_ISSET((con[j][0]->get_fd()),s_fd_set)) !=0){
                        if ((ret=(sessionRelaySD(j)))<=0){
                                if (ret==0){
                                        if (j==0) {
                                                LOG((__FILE__,__FUNCTION__,5,"control channel closed\n"))
						delConnection(0);
                                                setSessionEnd();
                                        }
                                        else {
                                                LOG((__FILE__,__FUNCTION__,5,"channel closed\n"))
						delConnection(j);
                                        }
                                }
                                else {
                                        ERR((__FILE__,__FUNCTION__,"sessionRead (sessionID: %d con[%d,0] %s)\n",sessionID,j,con[j][0]->getDescript().c_str()))
                                        setSessionEnd();
                                }
                        }
                }
                else if ((con[j][0]!=NULL) && (con[j][1]!=NULL) && (con[j][1]->connectionGetState()==CONNECT) && (FD_ISSET((con[j][1]->get_fd()),s_fd_set)) !=0){
                        if ((ret=(sessionRelayDS(j)))<=0){
                                if (ret==0){
                                        if (j==0) {
                                                LOG((__FILE__,__FUNCTION__,5,"control channel closed\n"))
						delConnection(0);
                                                setSessionEnd();
                                        }
                                        else {
                                                LOG((__FILE__,__FUNCTION__,5,"channel closed\n"))
						delConnection(j);
                                        }
                                }
                                else {
                                        ERR((__FILE__,__FUNCTION__,"sessionRead (sessionID: %d con[%d,1] %s)\n",sessionID,j,con[j][1]->getDescript().c_str()))
                                        setSessionEnd();
                                }
                        }
                }
                else if ((con[j][0]!=NULL) && (con[j][1]!=NULL) && (con[j][0]->connectionGetState()==LISTEN) && (FD_ISSET((con[j][0]->get_fd()),s_fd_set)) !=0){
			Connection *c_l;
                        c_l=(con[j][0]->connectionAccept());
                        if (c_l==NULL){
                                ERR((__FILE__,__FUNCTION__,"accept\n"))
                                setSessionEnd();
                        }
                        else {
				if ( con[j][0]!=c_l ) { 
                                	delete con[j][0];
                                	con[j][0]=c_l;
				}
                        }
                        if((con[j][1]->connectionGetState())!=CONNECT) con[j][1]->connectionConnect();
                        if((con[j][1]->connectionGetState())!=CONNECT){
                                ERR((__FILE__,__FUNCTION__,"could not connect destination\n"))
                                setSessionEnd();
                        }
                }
        }
        return(0);
}

int Session::sessionRelaySD(int i){
        int ret;
        LOG((__FILE__,__FUNCTION__,10,"starting proxy data... %s\n",con[i][0]->getDescript().c_str()))

        if ((ret=read(con[i][0]))<=0){
		if (ret==0) return(0);
                return(-1);
        }
	data.read=data.read+ret;
        if ((ret=parse0(con[i][0]))<=0){
		if (ret==0) return(1);
                return(-1);
        }
        if ((ret=write(con[i][1],con[i][0]))<=0){
		if (ret==0) return(0);
                return(-1);
        }
	data.write=data.write+ret;

        LOG((__FILE__,__FUNCTION__,15,"write, length: %d\n",ret))
        LOG((__FILE__,__FUNCTION__,10,"... finished proxy data\n"))
        return(1);
}

int Session::sessionRelayDS(int i){
        int ret;
        LOG((__FILE__,__FUNCTION__,10,"starting proxy data... %s\n",con[i][1]->getDescript().c_str()))

        if ((ret=read(con[i][1]))<=0){
		if (ret==0) return(0);
                return(-1);
        }
	data.read=data.read+ret;
        if ((ret=parse1(con[i][1]))<=0){
		if (ret==0) return(1);
                return(-1);
        }
        if ((ret=write(con[i][0],con[i][1]))<=0){
		if (ret==0) return(0);
                return(-1);
        }
	data.write=data.write+ret;

        LOG((__FILE__,__FUNCTION__,15,"write, length: %d\n",ret))
        LOG((__FILE__,__FUNCTION__,10,"... finished proxy data\n"))
        return(1);
}


int Session::read(Connection* c) {
        int ret=(-1);
        switch (c->d_number) {
                case 0:
                        if ((ret=parser->read_0(c))<0) return(-1);
                        break;
                case 1:
                        if ((ret=parser->read_1(c))<0) return(-1);
                        break;
                case 2:
                        if ((ret=parser->read_2(c))<0) return(-1);
                        break;
                case 3:
                        if ((ret=parser->read_3(c))<0) return(-1);
                        break;
                default:
                        if ((ret=parser->read_default(c))<0) return(-1);
                        break;
        }
        return(ret);
}
int Session::parse0(Connection* c) {
        int ret=(-1);
        switch (c->d_number) {
                case 0:
                        if ((ret=parser->parse_00(c))<0) WRN((__FILE__,__FUNCTION__,"error parsing data\n"))
                        break;
                case 1:
                        if ((ret=parser->parse_10(c))<0) WRN((__FILE__,__FUNCTION__,"error parsing data\n"))
                        break;
                case 2:
                        if ((ret=parser->parse_20(c))<0) WRN((__FILE__,__FUNCTION__,"error parsing data\n"))
                        break;
                case 3:
                        if ((ret=parser->parse_30(c))<0) WRN((__FILE__,__FUNCTION__,"error parsing data\n"))
                        break;
                default:
                        if ((ret=parser->parse_default0(c))<0) WRN((__FILE__,__FUNCTION__,"error parsing data\n"))
                        break;
        }
        return(ret);
}
int Session::parse1(Connection* c) {
        int ret=(-1);
        switch (c->d_number) {
                case 0:
                        if ((ret=parser->parse_01(c))<0) WRN((__FILE__,__FUNCTION__,"error parsing data\n"))
                        break;
                case 1:
                        if ((ret=parser->parse_11(c))<0) WRN((__FILE__,__FUNCTION__,"error parsing data\n"))
                        break;
                case 2:
                        if ((ret=parser->parse_21(c))<0) WRN((__FILE__,__FUNCTION__,"error parsing data\n")) 
                        break;
                case 3:
                        if ((ret=parser->parse_31(c))<0) WRN((__FILE__,__FUNCTION__,"error parsing data\n"))
                        break;
                default:
                        if ((ret=parser->parse_default1(c))<0) WRN((__FILE__,__FUNCTION__,"error parsing data\n"))
                        break;
        }
        return(ret);
}
int Session::write(Connection* dc, Connection* sc) {
        int ret=(-1);
        switch (dc->d_number) {
                case 0:
                        if ((ret=parser->write_0(dc,sc))<0) return(-1);
                        break;
                case 1:
                        if ((ret=parser->write_1(dc,sc))<0) return(-1);
                        break;
                case 2:
                        if ((ret=parser->write_2(dc,sc))<0) return(-1);
                        break;
                case 3:
                        if ((ret=parser->write_3(dc,sc))<0) return(-1);
                        break;
                default:
                        if ((ret=parser->write_default(dc,sc))<0) return(-1);
                        break;
        }
        return(ret);
}


