/* Copyright (C) 2000 KOM/Darmstadt University of Technology
 *
 * You are allowed to use all other parts of the code under the following terms:
 *
 * For non-commercial use, code may be used in unmodified form provided
 * that this copyright notice and this permission notice appear in
 * supporting documentation.
 *
 * This software is provided "as is" and without any express or implied
 * warranties, including, without limitation, the implied warranty of
 * fitness for a particular purpose.
 *
 * The code may be subjected to the GNU General Public License, Version 2,
 * and re-distributed under the terms of this license.
 *
 * Commercial use other than under the terms of the GNU General Public
 * License is allowed only after express negotiation of conditions
 * with the authors.
 */
#ifndef _Session_h_
#define _Session_h_ 1

#include "Global.h"
#include "Connection.h"
#include "Parser.h"
#include "TimeValue.h"
#include "String.h"
#include "SessionData.h"

class Proxy;

class Session {
	Proxy*  	proxy;
	TimeValue	startTime;
	TimeValue	timeoutTime;
        int             sessionID;                      /* a unique ID representing this session */
        bool            sessionEnd;                     /* when set to true the session is removed by the server (garbage collector flag) */
	String		type;

        /* the parser object contains the finite state machine of the used protocol (application logic) */
        Parser*         parser;                         /* a pointer to the parser that is used */
        int             read(Connection *c);            /* interface to the appropriate read function in the parser */
        int             parse0(Connection *c);           /* interface to the appropriate parse function in the parser */
        int             parse1(Connection *c);           /* interface to the appropriate parse function in the parser */
	int 		write(Connection* dc, Connection* sc);

	int 		sessionRelaySD(int i);		/* process data received for on a connection */
	int 		sessionRelayDS(int i);		/* process data received for on a connection */

public:
	SessionData	data;
	Connection	*con[MAXCONNECTIONS][2]; 	/* con[x,0] = src con[x,1] = dst */

	Session(String type_t);
	Session(String type_t,Connection *c);
	~Session();

	String getDescript();

        void setSessionEnd();
        bool getSessionEnd();
        int  getSessionID();
        int  setSessionID(int ID);
	int addConnection();
	int delConnection(int i);

	const TimeValue& getStartTime();
	const TimeValue& getTimeoutTime();

	int sessionRead(fd_set *s_fd_set);  		/* process data for the session if necessary (socket operation)*/
	
	void setProxy(Proxy* parent_t);
	int sessionInit();
	int log(bool remove);
};


#endif
