/* Copyright (C) 2000 KOM/Darmstadt University of Technology
 *
 * You are allowed to use all other parts of the code under the following terms:
 *
 * For non-commercial use, code may be used in unmodified form provided
 * that this copyright notice and this permission notice appear in
 * supporting documentation.
 *
 * This software is provided "as is" and without any express or implied
 * warranties, including, without limitation, the implied warranty of
 * fitness for a particular purpose.
 *
 * The code may be subjected to the GNU General Public License, Version 2,
 * and re-distributed under the terms of this license.
 *
 * Commercial use other than under the terms of the GNU General Public
 * License is allowed only after express negotiation of conditions
 * with the authors.
 */

#include "SignalHandler.h"
#include "Log.h"
#include "Conf.h"
#include "ListenerList.h"
#include "ProxyList.h"
#include "FirewallControl.h"

#include <signal.h>     /* signals, pthread_sigmask */
#include <pthread.h>	/* pthread_sigmask */
#include <unistd.h>	/* sleep */
#include <string.h>	/* memset */

void SignalHandler::sighup ( int i )
{
        LOG((__FILE__,__FUNCTION__,10,"sighup\n"))
        INF((__FILE__,__FUNCTION__,"sighup received, reading config file...\n"))
	Global::getGlobal()->getConf()->read();
	Global::getGlobal()->getLog()->setLoglevel(Global::getGlobal()->getConf()->getValAsInt("LOGLEVEL"));
        INF((__FILE__,__FUNCTION__,"... new config file read!\n"))
}

void SignalHandler::sigusr1 ( int i )
{
	/* for the proxies to stop select */
        LOG((__FILE__,__FUNCTION__,10,"sigusr1\n"))
        INF((__FILE__,__FUNCTION__,"sigusr1 received, increasing loglevel...\n"))
	Global::getGlobal()->getLog()->setLoglevel(100);
}
void SignalHandler::sigusr2 ( int i )
{
	/* term signals for threads */
        LOG((__FILE__,__FUNCTION__,10,"sigusr2\n"))
}

void SignalHandler::sigint ( int i )
{
	LOG((__FILE__,__FUNCTION__,10,"sigint received by thread# %d\n",Thread::currentID()))
	
	if (Global::getGlobal()->getListenerList()!=NULL) 	Global::getGlobal()->getListenerList()->stop();
	if (Global::getGlobal()->getProxyList()!=NULL)		Global::getGlobal()->getProxyList()->stop();
	if (Global::getGlobal()->getFirewallControl()!=NULL)	Global::getGlobal()->getFirewallControl()->stop();

	Global::getGlobal()->setEnd(true);
}

void SignalHandler::sigkill ( int i )
{
	LOG((__FILE__,__FUNCTION__,10,"sigkill received by thread# %d\n",Thread::currentID()))
	Global::getGlobal()->setEnd(true);
}

void SignalHandler::sigterm ( int i )
{
	LOG((__FILE__,__FUNCTION__,10,"sigterm received by thread# %d\n",Thread::currentID()))
	Global::getGlobal()->setEnd(true);
}

void SignalHandler::sigsegv ( int i )
{
	printf("segmentation fault, program suspended, attach gdb!\n");
	//dmalloc_shutdown();
	while ( 1 ) sleep ( 1 ) ;
}

void SignalHandler::install(){
	struct sigaction  actint,acthup,actkill,actterm,actusr1,actusr2;

	memset( &actint, 0, sizeof(actint) );
	actint.sa_handler = SignalHandler::sigint;
	sigaction(SIGINT, &actint, NULL);

	memset( &acthup, 0, sizeof(actint) );
	acthup.sa_handler = SignalHandler::sighup;
	sigaction(SIGHUP, &acthup, NULL);

	memset( &actkill, 0, sizeof(actint) );
	actkill.sa_handler = SignalHandler::sigkill;
	sigaction(SIGKILL, &actkill, NULL);

	memset( &actterm, 0, sizeof(actint) );
	actterm.sa_handler = SignalHandler::sigterm;
	sigaction(SIGTERM, &actterm, NULL);

	memset( &actusr1, 0, sizeof(actint) );
	actusr1.sa_handler = SignalHandler::sigusr1;
	sigaction(SIGUSR1, &actusr1, NULL);

	memset( &actusr2, 0, sizeof(actint) );
	actusr2.sa_handler = SignalHandler::sigusr2;
	sigaction(SIGUSR2, &actusr2, NULL);

	/*
        struct sigaction  actsegv;
	memset( &actsegv, 0, sizeof(actint) );
	actsegv.sa_handler = SignalHandler::sigsegv;
	sigaction(SIGSEGV, &actsegv, NULL);
	*/
}

void SignalHandler::block(int sig){
	sigset_t sigs;

	sigemptyset(&sigs);
        sigaddset(&sigs, sig);
	pthread_sigmask(SIG_BLOCK, &sigs, NULL);
}

void SignalHandler::unblock(int sig){
	sigset_t sigs;

	sigemptyset(&sigs);
        sigaddset(&sigs, sig);
	pthread_sigmask(SIG_UNBLOCK, &sigs, NULL);
}

void SignalHandler::blockAll(){
	block(SIGINT);
	block(SIGHUP);
	block(SIGKILL);
	block(SIGTERM);
	block(SIGUSR1);
	block(SIGUSR2);
}

void SignalHandler::unblockAll(){
	unblock(SIGINT);
	unblock(SIGHUP);
	unblock(SIGKILL);
	unblock(SIGTERM);
	unblock(SIGUSR1);
	unblock(SIGUSR2);
}
