/* Copyright (C) 2000 KOM/Darmstadt University of Technology
 *
 * You are allowed to use all other parts of the code under the following terms:
 *
 * For non-commercial use, code may be used in unmodified form provided
 * that this copyright notice and this permission notice appear in
 * supporting documentation.
 *
 * This software is provided "as is" and without any express or implied
 * warranties, including, without limitation, the implied warranty of
 * fitness for a particular purpose.
 *
 * The code may be subjected to the GNU General Public License, Version 2,
 * and re-distributed under the terms of this license.
 *
 * Commercial use other than under the terms of the GNU General Public
 * License is allowed only after express negotiation of conditions
 * with the authors.
 */

#include "String.h"
#include <string.h> /*strncpy */
#include <stdio.h>  /*sprintf */
#include <sys/socket.h> /*inet_ntoa*/
#include <netinet/in.h> /*inet_ntoa*/
#include <arpa/inet.h> /*inet_ntoa*/
#include <stdlib.h> /*atoi */


P_String::P_String() : len(0) { 
	rep[0] = 0; 
}
P_String::P_String( const P_String& s ) : len(s.len) {
	strncpy( rep, s.rep, len ); 
	rep[len] = 0;
}
P_String::P_String( const char* s ) {
	len = getLengthFromCharP(s);
	strncpy( rep, s, len ); 
	rep[len] = 0;
}
P_String::P_String( const char c ) {
	len = 1;
	rep[0] = c; 
	rep[1] = 0;
}
P_String::~P_String() {}

unsigned int P_String::getLengthFromCharP(const char* s) {
	unsigned int l = strlen(s);
	if (l >= maxP_StringLength) l = maxP_StringLength-1;
	return l;
}
P_String& P_String::operator=( const P_String& s ) {
	len = s.len;
	strncpy( rep, s.rep, len ); rep[len] = 0;
	return *this;
}
P_String& P_String::operator=( const char* s ) {
	len = getLengthFromCharP(s);
	strncpy( rep, s, len ); rep[len] = 0;
	return *this;
}
P_String& P_String::operator=( const char c ) {
	len = 1;
	rep[0] = c; 
	rep[1] = 0;
	return *this;
}
P_String& P_String::operator+=( const P_String& s ) {
	strncpy( rep + len, s.rep, s.len );
	len += s.len;
	rep[len] = 0;
	return *this;
}
P_String& P_String::operator+=( const char* s ) {
	unsigned int slen = getLengthFromCharP(s);
	strncpy( rep + len, s, slen );
	len += slen;
	rep[len] = 0;
	return *this;
}
P_String P_String::operator+( const P_String& s1 ) {
	P_String s = *this;
	s += s1;
	return s;
}
P_String& P_String::operator=( int i ) {
	sprintf(rep,"%d",i); 
	len = getLengthFromCharP(rep); 
	rep[len] = 0;
	return *this;
}
P_String& P_String::operator+=( int i ) {
	P_String s1;
	s1 = i;
	*this += s1; 
	return *this;
}
P_String P_String::operator+( int i ) {
	*this += i;
	return *this;
}
const char* P_String::c_str() const { 
	return rep; 
}
bool P_String::operator==( const P_String& s ) const {
	if ( len == s.len ) return strncmp( rep, s.rep, len ) == 0;
	return false;
}
bool P_String::operator==( const char* s ) const {
	unsigned int slen = getLengthFromCharP(s);
	if ( len == slen ) return strncmp( rep, s, len ) == 0;
	return false;
}
bool P_String::leftequal( const P_String& s ) const {
	return strncmp( rep, s.rep, s.len ) == 0;
}
bool P_String::leftequal( const char* s ) const {
	unsigned int slen = getLengthFromCharP(s);
	return strncmp( rep, s, slen ) == 0;
}
bool P_String::operator!=( const P_String& s ) const { 
	return !P_String::operator==(s); 
}
bool P_String::operator!=( const char* s ) const { 
	return !P_String::operator==(s); 
}
bool P_String::operator<( const P_String& s ) const {
	return strncmp( rep, s.rep, len ) < 0;
}
bool P_String::operator<( const char* s ) const { 
	unsigned int slen = getLengthFromCharP(s);
	int result = strncmp( rep, s, slen );
	if ( result == 0 ) return len < slen;
	return result < 0;
}
bool P_String::empty() const { 
	return len == 0; 
}
unsigned int P_String::size() const { 
	return len; 
}
char P_String::operator[](int i) const { 
	return rep[i]; 
}
P_String P_String::IPtoString(int ip){
	MUTEX_AUTO_LOCK(stringLock);
        static struct in_addr ia;
        ia.s_addr = ip;
        return( inet_ntoa(ia) );
}
int P_String::c_int(){
       	MUTEX_AUTO_LOCK(stringLock);
	return(atoi(rep));
}
	
