/* Copyright (C) 2000 KOM/Darmstadt University of Technology
 *
 * You are allowed to use all other parts of the code under the following terms:
 *
 * For non-commercial use, code may be used in unmodified form provided
 * that this copyright notice and this permission notice appear in
 * supporting documentation.
 *
 * This software is provided "as is" and without any express or implied
 * warranties, including, without limitation, the implied warranty of
 * fitness for a particular purpose.
 *
 * The code may be subjected to the GNU General Public License, Version 2,
 * and re-distributed under the terms of this license.
 *
 * Commercial use other than under the terms of the GNU General Public
 * License is allowed only after express negotiation of conditions
 * with the authors.
 */

#include "Thread.h"

#include <pthread.h>	/* thread */
#include <signal.h>	/* signals */
#include <errno.h>	/* errno */


Thread::Thread() : ID(0), end(false) {
}
Thread::~Thread() { 
	join(); 
}
void* Thread::runner( void* t ) { 
	((Thread*)t)->main(); return NULL; 
}
void Thread::run() { 
	if (!ID) {
		pthread_create( &ID, NULL, Thread::runner, this ); 
	}
}
void Thread::join() { 
	if (ID) { 
		pthread_join( ID, NULL ); ID = 0; 
	} 
}
void Thread::kill() { 
	if (ID) {
		pthread_kill( ID, SIGTERM ); 
	}
}
void Thread::alarm() { 
	if (ID) {
		pthread_kill( ID, SIGALRM ); 
	}
}
unsigned int Thread::myID() { 
	return (unsigned int)ID; 
}
unsigned int Thread::currentID() { 
	return (unsigned int)pthread_self(); 
}
void Thread::stop() {
	if (!end) {
		end=true;
		pthread_kill(ID,SIGUSR2);
	}
}	

Mutex::Mutex() : initialized(true) {
	pthread_mutex_init( &theLock, NULL );
}
Mutex::~Mutex() {
	initialized = false;
	pthread_mutex_unlock( &theLock );
	while ( pthread_mutex_destroy( &theLock ) == EBUSY ) {
		pthread_mutex_lock( &theLock );
		pthread_mutex_unlock( &theLock );
	}
}
bool Mutex::lock() {
	bool retval = pthread_mutex_lock( &theLock ) == 0 && initialized;
	if ( !retval ) pthread_mutex_unlock( &theLock );
	return retval;
}

void Mutex::unlock() {
	pthread_mutex_unlock( &theLock );
}
	
MutexLock::MutexLock( Mutex& mutex ) : mutex(mutex) { 
	mutex.lock(); 
}
MutexLock::~MutexLock() { 
	mutex.unlock(); 
}


