/* Copyright (C) 2000 KOM/Darmstadt University of Technology
 *
 * You are allowed to use all other parts of the code under the following terms:
 *
 * For non-commercial use, code may be used in unmodified form provided
 * that this copyright notice and this permission notice appear in
 * supporting documentation.
 *
 * This software is provided "as is" and without any express or implied
 * warranties, including, without limitation, the implied warranty of
 * fitness for a particular purpose.
 *
 * The code may be subjected to the GNU General Public License, Version 2,
 * and re-distributed under the terms of this license.
 *
 * Commercial use other than under the terms of the GNU General Public
 * License is allowed only after express negotiation of conditions
 * with the authors.
 */

#include "TimeValue.h"
#include <unistd.h>
#include <stdio.h>

TimeValue::TimeValue( int sec = 0, int usec = 0 ) {
	this->tv_sec = sec; this->tv_usec = usec;
}
TimeValue::TimeValue( const TimeValue& t ) {
	this->tv_sec = t.tv_sec; this->tv_usec = t.tv_usec;
}

TimeValue& TimeValue::operator=( const TimeValue& t ) {
	this->tv_sec = t.tv_sec; this->tv_usec = t.tv_usec;
	return *this;
}
TimeValue& TimeValue::operator+= ( const TimeValue& t ) {
	tv_sec += t.tv_sec;
	tv_usec += t.tv_usec;
	if ( tv_usec >= USECS_PER_SEC ) {
		tv_sec += 1;
		tv_usec -= USECS_PER_SEC;
	}
	return *this;
}

TimeValue& TimeValue::operator-= ( const TimeValue& t ) {
	tv_sec -= t.tv_sec;
	tv_usec -= t.tv_usec;
	if ( tv_sec > 0 ) {
		if ( tv_usec < 0 ) {
			tv_sec -= 1;
			tv_usec += USECS_PER_SEC;
		}
	} 
	else if ( tv_sec < 0 ) {
		if ( tv_usec > 0 ) {
			tv_sec += 1;
			tv_usec -= USECS_PER_SEC;
		}
	}
	return *this;
}
TimeValue::operator bool() const {
	return tv_sec || tv_usec;
}

void TimeValue::getCurrentSystemTime( timerep& t ) {
	MUTEX_AUTO_LOCK(timeValueLock);
        gettimeofday( &t, NULL );
}

