/* Copyright (C) 2000 KOM/Darmstadt University of Technology
 *
 * You are allowed to use all other parts of the code under the following terms:
 *
 * For non-commercial use, code may be used in unmodified form provided
 * that this copyright notice and this permission notice appear in
 * supporting documentation.
 *
 * This software is provided "as is" and without any express or implied
 * warranties, including, without limitation, the implied warranty of
 * fitness for a particular purpose.
 *
 * The code may be subjected to the GNU General Public License, Version 2,
 * and re-distributed under the terms of this license.
 *
 * Commercial use other than under the terms of the GNU General Public
 * License is allowed only after express negotiation of conditions
 * with the authors.
 */
#ifndef _TimeValue_h_
#define _TimeValue_h_ 1

#include "Thread.h"
#include <sys/time.h>

#define USECS_PER_SEC  1000000

static Mutex timeValueLock;

// internal represantation of time values
typedef timeval timerep;

// timerep: tv_sec <-> seconds,  tv_usec <-> microseconds
class TimeValue : public timerep {
public:
	TimeValue( int sec = 0, int usec = 0 );
        TimeValue( const TimeValue& t );

	// operators
	TimeValue& operator=( const TimeValue& t );
        TimeValue& operator+= ( const TimeValue& t );
        TimeValue& operator-= ( const TimeValue& t );
	operator bool() const;

	static void getCurrentSystemTime( timerep& t ); 
};
extern inline bool operator>( const TimeValue& t1 , const TimeValue& t2 ) {
        if (t1.tv_sec>t2.tv_sec) return true;
        if (t1.tv_sec<t2.tv_sec) return false;
        if (t1.tv_usec>t2.tv_usec) return true;
        return false;
}

extern inline bool operator>=( const TimeValue& t1 , const TimeValue& t2 ) {
        if (t1.tv_sec>t2.tv_sec) return true;
        if (t1.tv_sec<t2.tv_sec) return false;
        if (t1.tv_usec>t2.tv_usec) return true;
        if ((t1.tv_sec==t2.tv_sec) && (t1.tv_usec==t2.tv_usec)) return true;
        return false;
}

extern inline bool operator<( const TimeValue& t1 , const TimeValue& t2 ) {
        if (t1.tv_sec<t2.tv_sec) return true;
        if (t1.tv_sec>t2.tv_sec) return false;
        if (t1.tv_usec<t2.tv_usec) return true;
        return false;
}

extern inline bool operator<=( const TimeValue& t1 , const TimeValue& t2 ) {
        if (t1.tv_sec<t2.tv_sec) return true;
        if (t1.tv_sec>t2.tv_sec) return false;
        if (t1.tv_usec<t2.tv_usec) return true;
        if ((t1.tv_sec==t2.tv_sec) && (t1.tv_usec==t2.tv_usec)) return true;
        return false;
}
extern inline TimeValue operator-( const TimeValue& t1 , const TimeValue& t2 ) {
        TimeValue t = t1;
        return t -= t2;
}
extern inline TimeValue operator+( const TimeValue& t1 , const TimeValue& t2 ) {
        TimeValue t = t1;
        return t += t2;
}
extern inline TimeValue operator/ ( const TimeValue& t1, int i ) {
        if (i<=0) return TimeValue(0,0);
        long long usec = ((long long)(t1.tv_sec) * (long long)(USECS_PER_SEC) + t1.tv_usec) / (long long)(i);
        return TimeValue( usec / (long long)(USECS_PER_SEC), usec % (long long)(USECS_PER_SEC) );
}
extern inline int operator/ ( const TimeValue& t1, const TimeValue& t2 ) {
        if (!t2) return (-1);
        return ((long long)t1.tv_sec * (long long)USECS_PER_SEC + t1.tv_usec) / ((long long)t2.tv_sec * (long long)USECS_PER_SEC + t2.tv_usec);
}

extern inline TimeValue operator* ( const TimeValue& t1, int i ) {
        long long usec = (long long) t1.tv_usec * (long long) i;
        return TimeValue((long long) t1.tv_sec * (long long) i + usec / (long long) USECS_PER_SEC, usec % (long long) USECS_PER_SEC );
}


#endif
