/* Copyright (C) 2000 KOM/Darmstadt University of Technology
 *
 * You are allowed to use all other parts of the code under the following terms:
 *
 * For non-commercial use, code may be used in unmodified form provided
 * that this copyright notice and this permission notice appear in
 * supporting documentation.
 *
 * This software is provided "as is" and without any express or implied
 * warranties, including, without limitation, the implied warranty of
 * fitness for a particular purpose.
 *
 * The code may be subjected to the GNU General Public License, Version 2,
 * and re-distributed under the terms of this license.
 *
 * Commercial use other than under the terms of the GNU General Public
 * License is allowed only after express negotiation of conditions
 * with the authors.
 */





#include "FTPParser.h"
#include "../Log.h"
#include "../Conf.h"
#include "../FirewallControl.h"
#include "../Fwfil.h"
#include "../Session.h"


FTPParser::FTPParser(Session* parent_t,String type_t) {
	LOG((__FILE__,__FUNCTION__,15,"FTP: creating parser\n"))
	type=type_t;
	parent=parent_t;
	(CNTR_in)=&(parent->con[0][0]);
	(CNTR_out)=&(parent->con[0][1]);
	DATA_in=NULL;
	DATA_out=NULL;
	parent->data.dstUser="FTPSERVER";
}

FTPParser::~FTPParser() {
	LOG((__FILE__,__FUNCTION__,15,"FTP: deleting parser\n"))
}
int FTPParser::connectDestinationSPECIAL(String locator){
        return(-1);
}
int FTPParser::connectDestinationFAST(){
        return(connectDestination());
}


int FTPParser::cmd_user(char *parse_buffer){
        char cmd[5];
        char user[30];
        LOG((__FILE__,__FUNCTION__,8,"FTP:USER:\n"))
        if (sscanf(parse_buffer,"%s %s",&cmd,&user)<2){
		ERR((__FILE__,__FUNCTION__,"FTP:USER: parse error\n"))
		return(-1);
        }
	parent->data.srcUser=user;
        return(0);
}
int FTPParser::cmd_port(char *parse_buffer){
	Fwfil *fwfilp;
        char cmd[5];
        int  ip0,ip1,ip2,ip3,port0,port1;
        int       srchost;
        int       srcmask;
        int       srcport;
        int       dsthost;
        int       dstmask;
        int       dstport;


        LOG((__FILE__,__FUNCTION__,8,"FTP:PORT: (active mode)\n"))
        if (sscanf(parse_buffer,"%s %d,%d,%d,%d,%d,%d",&cmd,&ip0,&ip1,&ip2,&ip3,&port0,&port1)<7){
		ERR((__FILE__,__FUNCTION__,"FTP:PORT: parse error\n"))
		return(-1);
        }

        if ((Global::getGlobal()->getConf()->getValAsInt(type+"_DATAFASTPATH"))==0){
		int data;
		if (((DATA_in==NULL) && (DATA_out==NULL)) || (((*DATA_in)==NULL) && ((*DATA_out)==NULL))){
			if ((data=parent->addConnection())<0) {
                        	return(-1);
			}
			(DATA_in)=&(parent->con[data][0]);
			(DATA_out)=&(parent->con[data][1]);
                }
		(*DATA_out)->d_name     = String("FTP-DATA-DST");
		(*DATA_out)->l_ip       = Global::getGlobal()->getConf()->getValAsIP("LOCALIP");
		(*DATA_out)->l_mask     = 0xffffffff;
		(*DATA_out)->l_port     = -1;
		//(*DATA_out)->p_ip       = ((ip0)) | ((ip1) << 8) | ((ip2) << 16) | ((ip3) << 24);
		(*DATA_out)->p_ip       = (*CNTR_in)->p_ip;
		(*DATA_out)->p_mask     = 0xffffffff;
		(*DATA_out)->p_port     = port0*256+port1;
		(*DATA_out)->ipproto    = 6;
		(*DATA_out)->direction  = 0;
		(*DATA_out)->d_number   = 50;

		(*DATA_in)->d_name     = String("FTP-DATA-SRC");
		(*DATA_in)->l_ip       = Global::getGlobal()->getConf()->getValAsIP("LOCALIP"); 
		(*DATA_in)->l_mask     = 0xffffffff;
		(*DATA_in)->l_port     = -1;
		(*DATA_in)->p_ip       = (*CNTR_out)->p_ip;
		(*DATA_in)->p_mask     = 0xffffffff;
		(*DATA_in)->p_port     = -1;
		(*DATA_in)->ipproto    = 6;
		(*DATA_in)->direction  = 1;
		(*DATA_in)->d_number   = 50;
		
                if (((*DATA_in)->connectionListen())<0){
			ERR((__FILE__,__FUNCTION__,"FTP:PORT: listen on FTP-DATA-SRC\n"))
			return(-1);
                }
		ip0=((((*DATA_in)->l_ip)&0xff));
                ip1=((((*DATA_in)->l_ip)&0xff00) >>8);
                ip2=((((*DATA_in)->l_ip)&0xff0000) >>16);
                ip3=((((*DATA_in)->l_ip)&0xff000000)>>24);
		port0=((((*DATA_in)->l_port)&0xff00)>>8);
		port1=((((*DATA_in)->l_port)&0xff)     );
                sprintf(parse_buffer,"PORT %d,%d,%d,%d,%d,%d\0",ip0,ip1,ip2,ip3,port0,port1);
        	LOG((__FILE__,__FUNCTION__,8,"FTP:PORT: modified cmd: %s\n",parse_buffer))
        }
        else {
		srcport=20;
		srchost=(*CNTR_out)->p_ip;
		srcmask=0xffffffff;
		dstport=port0*256+port1;
		dsthost=(*CNTR_in)->p_ip;
		dstmask=0xffffffff;
		
        	fwfilp = new Fwfil("FTP-DATA",6,srchost,srcmask,FWFIL_PORTRANGE_EQUAL,srcport,dsthost,dstmask,FWFIL_PORTRANGE_EQUAL,dstport);
		if(fwhndl.getChannel(fwfilp)<0){
			ERR((__FILE__,__FUNCTION__,"FTP:PORT: could not configure firewall for incomming connection\n"))
			return(-1);
		}
        	fwfilp = new Fwfil("FTP-DATA",6,dsthost,dstmask,FWFIL_PORTRANGE_EQUAL,dstport,srchost,srcmask,FWFIL_PORTRANGE_EQUAL,srcport);
		if(fwhndl.getChannel(fwfilp)<0){
			ERR((__FILE__,__FUNCTION__,"FTP:PORT: could not configure firewall for incomming connection\n"))
			return(-1);
		}
        }
        return(0);
}

int FTPParser::cmd_pasv(char *parse_buffer){
	Fwfil *fwfilp;
        char cmd0[5];
        char cmd1[15];
        char cmd2[15];
        char cmd3[15];
        int  ip0,ip1,ip2,ip3,port0,port1;
        int       srchost;
        int       srcmask;
        int       srcport;
        int       dsthost;
        int       dstmask;
        int       dstport;

        LOG((__FILE__,__FUNCTION__,8,"FTP:PORT: (passive mode)\n"))
        if (sscanf(parse_buffer,"%s %s %s %s (%d,%d,%d,%d,%d,%d)",&cmd0,&cmd1,&cmd2,&cmd3,&ip0,&ip1,&ip2,&ip3,&port0,&port1)<10){
		ERR((__FILE__,__FUNCTION__,"FTP:PORT: parse error\n"))
		return(-1);
        }

        if ((Global::getGlobal()->getConf()->getValAsInt(type+"_DATAFASTPATH"))==0){
		int data;
		if (((DATA_in==NULL) && (DATA_out==NULL)) || (((*DATA_in)==NULL) && ((*DATA_out)==NULL))){
			if ((data=parent->addConnection())<0) {
                        	return(-1);
			}
			(DATA_in)=&(parent->con[data][0]);
			(DATA_out)=&(parent->con[data][1]);
                }
		(*DATA_out)->d_name     = String("FTP-DATA-DST");
		(*DATA_out)->l_ip       = Global::getGlobal()->getConf()->getValAsIP("LOCALIP");
		(*DATA_out)->l_mask     = 0xffffffff;
		(*DATA_out)->l_port     = -1;
		//(*DATA_out)->p_ip       = ((ip0)) | ((ip1) << 8) | ((ip2) << 16) | ((ip3) << 24);
		(*DATA_out)->p_ip       = (*CNTR_out)->p_ip;
		(*DATA_out)->p_mask     = 0xffffffff;
		(*DATA_out)->p_port     = port0*256+port1;
		(*DATA_out)->ipproto    = 6;
		(*DATA_out)->direction  = 0;
		(*DATA_out)->d_number   = 50;

		(*DATA_in)->d_name     = String("FTP-DATA-SRC");
		(*DATA_in)->l_ip       = Global::getGlobal()->getConf()->getValAsIP("LOCALIP"); 
		(*DATA_in)->l_mask     = 0xffffffff;
		(*DATA_in)->l_port     = -1;
		(*DATA_in)->p_ip       = (*CNTR_in)->p_ip;
		(*DATA_in)->p_mask     = 0xffffffff;
		(*DATA_in)->p_port     = -1;
		(*DATA_in)->ipproto    = 6;
		(*DATA_in)->direction  = 1;
		(*DATA_in)->d_number   = 50;
                if (((*DATA_in)->connectionListen())<0){
			ERR((__FILE__,__FUNCTION__,"FTP:PORT: listen on FTP-DATA-SRC\n"))
			return(-1);
                }
		ip0=((((*DATA_in)->l_ip)&0xff));
                ip1=((((*DATA_in)->l_ip)&0xff00) >>8);
                ip2=((((*DATA_in)->l_ip)&0xff0000) >>16);
                ip3=((((*DATA_in)->l_ip)&0xff000000)>>24);
		port0=((((*DATA_in)->l_port)&0xff00)>>8);
		port1=((((*DATA_in)->l_port)&0xff)     );
		
                sprintf(parse_buffer,"227 Entering Passive Mode (%d,%d,%d,%d,%d,%d)\0",ip0,ip1,ip2,ip3,port0,port1);
        	LOG((__FILE__,__FUNCTION__,8,"FTP:PORT: modified cmd: %s\n",parse_buffer))
        }
        else {
		srcport=1023;
		srchost=(*CNTR_out)->p_ip;
		srcmask=0xffffffff;

		dstport=port0*256+port1;
		dsthost=(*CNTR_in)->p_ip;
		dstmask=0xffffffff;
		
        	fwfilp = new Fwfil("FTP-DATA",6,srchost,srcmask,FWFIL_PORTRANGE_GREATERT,srcport,dsthost,dstmask,FWFIL_PORTRANGE_EQUAL,dstport);
		if(fwhndl.getChannel(fwfilp)<0){
			ERR((__FILE__,__FUNCTION__,"FTP:PORT: could not configure firewall for incomming connection\n"))
			return(-1);
		}
        	fwfilp = new Fwfil("FTP-DATA",6,dsthost,dstmask,FWFIL_PORTRANGE_EQUAL,dstport,srchost,srcmask,FWFIL_PORTRANGE_EQUAL,srcport);
		if(fwhndl.getChannel(fwfilp)<0){
			ERR((__FILE__,__FUNCTION__,"FTP:PORT: could not configure firewall for incomming connection\n"))
			return(-1);
		}
        }
        return(1);
}



int FTPParser::write_0(Connection *dc, Connection *sc){
        LOG((__FILE__,__FUNCTION__,15,"FTP: write_0\n"))
	if(dc->connectionWrite(sc)<0) {
                WRN((__FILE__,__FUNCTION__,"FTP: write on socket\n"))
                return(-1);
        }
        return(sc->packet_len);
}
int FTPParser::write_1(Connection *dc, Connection *sc){
        LOG((__FILE__,__FUNCTION__,15,"FTP: write_1\n"))
	return(-1);
}
int FTPParser::write_2(Connection *dc, Connection *sc){
        LOG((__FILE__,__FUNCTION__,15,"FTP: write_2\n"))
	return(-1);
}
int FTPParser::write_3(Connection *dc, Connection *sc){
        LOG((__FILE__,__FUNCTION__,15,"FTP: write_3\n"))
	return(-1);
}
int FTPParser::write_default(Connection *dc, Connection *sc){
        LOG((__FILE__,__FUNCTION__,15,"FTP: write_default\n"))
	if(dc->connectionWrite(sc)<0) {
                WRN((__FILE__,__FUNCTION__,"FTP: write on socket\n"))
                return(-1);
        }
        return(sc->packet_len);
}
	

int FTPParser::read_0(Connection *c){
        int     len=0;
        LOG((__FILE__,__FUNCTION__,15,"FTP: read_0\n"))
	c->zeroBuffer();
	while (1) {
                if ((len=c->connectionRead(1))<=0) return(len);
                if (((c->buffer)[(c->packet_len)-1]) == '\n') break;
                if ((c->packet_len)>=BUFFERSIZE) {
                        ERR((__FILE__,__FUNCTION__,"FTP: no \\n in message\n"))
			return(-1);
                }
        }
        LOG((__FILE__,__FUNCTION__,15,"FTP: read_0, length: %d\n",len))
        return(c->packet_len);
}
int FTPParser::read_1(Connection *c){
        LOG((__FILE__,__FUNCTION__,15,"FTP: read_1\n"))
	return(-1);
}
int FTPParser::read_2(Connection *c){
        LOG((__FILE__,__FUNCTION__,15,"FTP: read_2\n"))
	return(-1);
}
int FTPParser::read_3(Connection *c){
        LOG((__FILE__,__FUNCTION__,15,"FTP: read_3\n"))
	return(-1);
}
int FTPParser::read_default(Connection *c){
        int     len=0;
        LOG((__FILE__,__FUNCTION__,15,"FTP: read_0\n"))
	c->zeroBuffer();
        if ((len=c->connectionRead(BUFFERSIZE-1))<0) return(-1);
        LOG((__FILE__,__FUNCTION__,15,"FTP: read_0, length: %d\n",len))
        return(c->packet_len);
}

int FTPParser::parse_00(Connection *c){
        LOG((__FILE__,__FUNCTION__,15,"FTP: parse_00\n"))
	char parse_buffer[BUFFERSIZE];

        if ((c->packet_len)-2<0) return(-1);
        memcpy(parse_buffer,c->buffer,(c->packet_len)-2);
        parse_buffer[(c->packet_len)-2]='\0';
        LOG((__FILE__,__FUNCTION__,8,"FTP: content: %s\n",parse_buffer))
        if (String(parse_buffer).leftequal("PORT")) if (cmd_port(parse_buffer)<0) return(-1);
        if (String(parse_buffer).leftequal("USER")) if (cmd_user(parse_buffer)<0) return(-1);
        memcpy(c->buffer,parse_buffer,strlen(parse_buffer));
        c->packet_len=strlen(parse_buffer)+2;
        c->buffer[(c->packet_len)-2]='\r';
        c->buffer[(c->packet_len)-1]='\n';
        return(1);
}

int FTPParser::parse_10(Connection *c){
        LOG((__FILE__,__FUNCTION__,15,"FTP: parse_10\n"))
	return(1);
}
int FTPParser::parse_20(Connection *c){
        LOG((__FILE__,__FUNCTION__,15,"FTP: parse_20\n"))
	return(1);
}
int FTPParser::parse_30(Connection *c){
        LOG((__FILE__,__FUNCTION__,15,"FTP: parse_30\n"))
	return(1);
}
int FTPParser::parse_default0(Connection *c){
        LOG((__FILE__,__FUNCTION__,15,"FTP: parse_default0\n"))
	return(1);
}
int FTPParser::parse_01(Connection *c){
        LOG((__FILE__,__FUNCTION__,15,"FTP: parse_01\n"))
	char parse_buffer[BUFFERSIZE];

        if ((c->packet_len)-2<0) return(-1);
        memcpy(parse_buffer,c->buffer,(c->packet_len)-2);
        parse_buffer[(c->packet_len)-2]='\0';
        LOG((__FILE__,__FUNCTION__,15,"FTP: content: %s\n",parse_buffer))
        if (String(parse_buffer).leftequal("227")) if (cmd_pasv(parse_buffer)<0) return(-1);
        memcpy(c->buffer,parse_buffer,strlen(parse_buffer));
        c->packet_len=strlen(parse_buffer)+2;
        c->buffer[(c->packet_len)-2]='\r';
        c->buffer[(c->packet_len)-1]='\n';
	return(1);
}
int FTPParser::parse_11(Connection *c){
        LOG((__FILE__,__FUNCTION__,15,"FTP: parse_11\n"))
	return(1);
}
int FTPParser::parse_21(Connection *c){
        LOG((__FILE__,__FUNCTION__,15,"FTP: parse_21\n"))
	return(1);
}
int FTPParser::parse_31(Connection *c){
        LOG((__FILE__,__FUNCTION__,15,"FTP: parse_31\n"))
	return(1);
}
int FTPParser::parse_default1(Connection *c){
        LOG((__FILE__,__FUNCTION__,15,"FTP: parse_default1\n"))
	return(1);
}

