/* Copyright (C) 2000 KOM/Darmstadt University of Technology
 *
 * You are allowed to use all other parts of the code under the following terms:
 *
 * For non-commercial use, code may be used in unmodified form provided
 * that this copyright notice and this permission notice appear in
 * supporting documentation.
 *
 * This software is provided "as is" and without any express or implied
 * warranties, including, without limitation, the implied warranty of
 * fitness for a particular purpose.
 *
 * The code may be subjected to the GNU General Public License, Version 2,
 * and re-distributed under the terms of this license.
 *
 * Commercial use other than under the terms of the GNU General Public
 * License is allowed only after express negotiation of conditions
 * with the authors.
 */

#include "H323Call.h"
#include "../Connection.h"
#include "../Session.h"
#include "../Conf.h"
#include "H323MediaChannel.h"


H323Call::H323Call(){
	id		= -1;
	Q931_in		= NULL;
	Q931_out	= NULL;
	H245_in		= NULL;
	H245_out	= NULL;
	h245		= -1;
	q931		= -1;

	for (int i=0;i<MAXMEDIACHANNELS;++i){
		mediaChannel[i]=NULL;
	}
}

H323Call::~H323Call(){
	for (int i=0;i<MAXMEDIACHANNELS;++i){
		if (mediaChannel[i]!=NULL) delete mediaChannel[i];
	}
}

int H323Call::getMediaChannel(int id_a,int id_b){
	for (int i=0;i<MAXMEDIACHANNELS;++i){
               	if (mediaChannel[i]!=NULL){
                       	if(((mediaChannel[i]->id_a)==id_a) && ((mediaChannel[i]->id_b==id_b))) return(i);
               	}
       	}
	return(-1);
}

int H323Call::delMediaChannel(int id_a,int id_b){
	for (int i=0;i<MAXMEDIACHANNELS;++i){
        	if (mediaChannel[i]!=NULL){
			if(((mediaChannel[i]->id_a)==id_a) && ((mediaChannel[i]->id_b==id_b))) {
				delete mediaChannel[i];
				mediaChannel[i]=NULL;
				return(0);
			}
               	}
        }
	return(-1);
}

int H323Call::setMediaChannel(Session* parent){
	for (int i=0;i<MAXMEDIACHANNELS;++i){
               	if (mediaChannel[i]==NULL){
			mediaChannel[i]=new H323MediaChannel();
			if((mediaChannel[i]->setMediaChannel(parent))<0){
				delete mediaChannel[i];
				mediaChannel[i]=NULL;
				return(-1);
			}
			return(i);
               	}
       	}
	return(-1);
}

int H323Call::useMediaChannel(Session* parent,int id_a,int id_b){
	int ret=-1;
	if ((ret=getMediaChannel(id_a,id_b))>=0) return(ret);
	for (int i=0;i<MAXMEDIACHANNELS;++i){
               	if (mediaChannel[i]==NULL){
			mediaChannel[i]=new H323MediaChannel();
			if((mediaChannel[i]->useMediaChannel(parent,id_a,id_b))<0){
				delete mediaChannel[i];
				mediaChannel[i]=NULL;
				return(-1);
			}
			return(i);
               	}
       	}
	return(-1);
}

int H323Call::getH245Channel(){
	return(h245);
}

int H323Call::delH245Channel(){
        if (H245_in!=NULL)   {
                delete (*H245_in);
                (*H245_in)=NULL;
        }
        if (H245_out!=NULL)  {
                delete (*H245_out);
                (*H245_out)=NULL;
        }
	h245=-1;
	return(0);
}

int H323Call::setH245Channel(Session* parent){
	if ((H245_in==NULL) && (H245_out==NULL)){
		if ((h245=parent->addConnection())<0) {
                        delH245Channel(); 
                        return(-1);
                }
		H245_in  = &(parent->con[h245][0]);
                H245_out = &(parent->con[h245][1]);
	}
	return(h245);
}

int H323Call::useH245Channel(Session* parent){
	if (setH245Channel(parent)<0) return(-1);

        (*H245_out)->connectionClose();
	(*H245_out)->d_name     = String("H245-DST");
       	(*H245_out)->l_ip       = Global::getGlobal()->getConf()->getValAsIP("LOCALIP");
       	(*H245_out)->l_mask     = 0xffffffff;
       	(*H245_out)->l_port     = -1;
       	(*H245_out)->p_ip       = 0;
       	(*H245_out)->p_mask     = 0;
       	(*H245_out)->p_port     = -1; 
       	(*H245_out)->ipproto    = 6;
       	(*H245_out)->direction  = 0;
       	(*H245_out)->d_number   = 1;

        (*H245_in)->connectionClose();
       	(*H245_in)->d_name     = String("H245-SRC");
       	(*H245_in)->l_ip       = Global::getGlobal()->getConf()->getValAsIP("LOCALIP");
       	(*H245_in)->l_mask     = 0xffffffff;
       	(*H245_in)->l_port     = -1;
       	(*H245_in)->p_ip       = 0;
       	(*H245_in)->p_mask     = 0;
       	(*H245_in)->p_port     = -1;
       	(*H245_in)->ipproto    = 6;
       	(*H245_in)->direction  = 1;
       	(*H245_in)->d_number   = 1;
	if((*H245_in)->connectionListen()<0) return(-1);

	return (h245);
}

int H323Call::delQ931Channel(){
        if (Q931_in!=NULL)   {
                delete (*Q931_in);
                (*Q931_in)=NULL;
        }
        if (H245_out!=NULL)  {
                delete (*Q931_out);
                (*Q931_out)=NULL;
        }
	q931=-1;
	return(0);
}

int H323Call::setQ931Channel(Session* parent){
	if ((Q931_in==NULL) && (Q931_out==NULL)){
		Q931_in=&(parent->con[0][0]);
		Q931_out=&(parent->con[0][1]);
		q931=0;
	}
	return(q931);
}

int H323Call::useQ931Channel(Session* parent){
	if (setQ931Channel(parent)<0) return(-1);
	return(q931);
}

