/* Copyright (C) 2000 KOM/Darmstadt University of Technology
 *
 * You are allowed to use all other parts of the code under the following terms:
 *
 * For non-commercial use, code may be used in unmodified form provided
 * that this copyright notice and this permission notice appear in
 * supporting documentation.
 *
 * This software is provided "as is" and without any express or implied
 * warranties, including, without limitation, the implied warranty of
 * fitness for a particular purpose.
 *
 * The code may be subjected to the GNU General Public License, Version 2,
 * and re-distributed under the terms of this license.
 *
 * Commercial use other than under the terms of the GNU General Public
 * License is allowed only after express negotiation of conditions
 * with the authors.
 */





#include "H323Parser.h"
#include "../Log.h"
#include "../Conf.h"
#include "../Session.h"
#include "../Fwfil.h"

H323Parser::H323Parser(Session* parent_t,String type_t){
	LOG((__FILE__,__FUNCTION__,15,"H323: creating parser\n"))
	type=type_t;
	parent=parent_t;
	call.setQ931Channel(parent);
	call.setH245Channel(parent);
}

H323Parser::~H323Parser() {
	LOG((__FILE__,__FUNCTION__,15,"H323: deleting parser\n"))
	sendReleaseComplete(0);
	sendReleaseComplete(1);
	call.delQ931Channel();
	call.delH245Channel();
}

bool H323Parser::checkPDU(String pdutype){
        String check = type + "_ALLOW_" + pdutype;
        if ((Global::getGlobal()->getConf()->getValAsString(check))==String("YES")) return true;
	WRN((__FILE__,__FUNCTION__,"H323: drop pdu %s\n",pdutype.c_str()))
        return false;
}

bool H323Parser::checkOpenLogicalChannel(H245_OpenLogicalChannel *h245_openlogicalchannel){
        String check = type + "_ALLOW_CHANNEL_";
	if (h245_openlogicalchannel->m_forwardLogicalChannelParameters.m_dataType.GetTag()==H245_DataType::e_audioData){
		check = check + "audioData";
		if ((Global::getGlobal()->getConf()->getValAsString(check))==String("YES")) {
			LOG((__FILE__,__FUNCTION__,8,"H323:H245:REQUEST:OPENLOGICALCHANNEL: forwardLogicalChannelNumber = %d (audioData)\n",
				h245_openlogicalchannel->m_forwardLogicalChannelNumber.GetValue()))
			return true;
		}
		else LOG((__FILE__,__FUNCTION__,8,"H323:H245:REQUEST:OPENLOGICALCHANNEL: forwardLogicalChannelNumber = %d (audioData) not allowed by config!\n",
			h245_openlogicalchannel->m_forwardLogicalChannelNumber.GetValue()))
	}
	else if (h245_openlogicalchannel->m_forwardLogicalChannelParameters.m_dataType.GetTag()==H245_DataType::e_videoData){
		check = check + "videoData";
		if ((Global::getGlobal()->getConf()->getValAsString(check))==String("YES")) {
			LOG((__FILE__,__FUNCTION__,8,"H323:H245:REQUEST:OPENLOGICALCHANNEL: forwardLogicalChannelNumber = %d (videoData)\n",
				h245_openlogicalchannel->m_forwardLogicalChannelNumber.GetValue()))
			return true;
		}
		else LOG((__FILE__,__FUNCTION__,8,"H323:H245:REQUEST:OPENLOGICALCHANNEL: forwardLogicalChannelNumber = %d (videoData) not allowed by config!\n",
			h245_openlogicalchannel->m_forwardLogicalChannelNumber.GetValue()))
	}
	else if (h245_openlogicalchannel->m_forwardLogicalChannelParameters.m_dataType.GetTag()==H245_DataType::e_data){
		LOG((__FILE__,__FUNCTION__,8,"H323:H245:REQUEST:OPENLOGICALCHANNEL: forwardLogicalChannelNumber = %d (data (t.120)) not supported yet!\n",
			h245_openlogicalchannel->m_forwardLogicalChannelNumber.GetValue()))
	}
	else {
		WRN((__FILE__,__FUNCTION__,"UNKNOWN H245:REQUEST:OPENLOGICALCHANNEL: unknown DataType requested\n"))
	}
        return false;
}

int H323Parser::sendOpenLogicalChannelReject(bool direction,int num){
        H245_MultimediaSystemControlMessage        mscMsg;
        PPER_Stream                                h245streamOUT;
	Connection *c=NULL;

	if(!direction) c=*(call.H245_in);
	if(direction)  c=*(call.H245_out);
	if(c==NULL) return(-1);

        mscMsg.SetTag(H245_MultimediaSystemControlMessage::e_response);
	H245_ResponseMessage & h245_responsemessage=mscMsg;
	h245_responsemessage.SetTag(H245_ResponseMessage::e_openLogicalChannelReject);
	H245_OpenLogicalChannelReject & h245_openlogicalchannelreject=h245_responsemessage;
	h245_openlogicalchannelreject.m_forwardLogicalChannelNumber=num;
	h245_openlogicalchannelreject.m_cause.SetTag(H245_OpenLogicalChannelReject_cause::e_unspecified);
	
#ifdef _H323DEBUG_
        printf("\n===BEGIN H245 PDU DECODED ==============================\n");
	mscMsg.PrintOn(cout);
        printf("\n===END H245 PDU DECODED ================================\n");
#endif

	LOG((__FILE__,__FUNCTION__,8,"H323:H245:RESPONSE:OPENLOGICALCHANNELREJECT: forwardLogicalChannelNumber = %d created!\n",num))

        mscMsg.Encode(h245streamOUT);
	h245streamOUT.CompleteEncoding();

        c->packet_len=h245streamOUT.GetSize()+TPKT_LENGTH_OFFSET+2;

        c->buffer[TPKT_LENGTH_OFFSET]=  (unsigned char) (((c->packet_len) >> 8) & 0xFFFF) ;
        c->buffer[TPKT_LENGTH_OFFSET+1]=(unsigned char) (((c->packet_len)     ) & 0x00FF) ;
        for(u_int32_t i=0;i<h245streamOUT.GetSize();i++){
                c->buffer[TPKT_LENGTH_OFFSET+2+i]=h245streamOUT[i];
        }

	if(c->connectionWrite(c)<0) {
                WRN((__FILE__,__FUNCTION__,"H323:H245:RESPONSE:OPENLOGICALCHANNELREJECT: write on socket\n"))
                return(-1);
        }

	return(1);
}

int H323Parser::sendReleaseComplete(bool direction){
        Q931              		q931pdu;
        PPER_Stream                     q931streamOUT;
        H225_H323_UserInformation       uuInfo;
        PPER_Stream                     uuInfostreamOUT;
	Connection *c=NULL;

	if(!direction) c=*(call.Q931_in);
	if(direction)  c=*(call.Q931_out);
	if(c==NULL) return(-1);


	uuInfo.m_h323_uu_pdu.m_h323_message_body.SetTag(H225_H323_UU_PDU_h323_message_body::e_releaseComplete);
	q931pdu.BuildReleaseComplete(call.id,1);
	H225_ReleaseComplete_UUIE & h225_releasecomplete_uuie = uuInfo.m_h323_uu_pdu.m_h323_message_body;
	h225_releasecomplete_uuie.m_reason.SetTag(H225_ReleaseCompleteReason::e_noPermission);
	
#ifdef _H323DEBUG_
       	printf("\n===BEGIN UserUserField MODIFIED ========================\n");
        uuInfo.PrintOn(cout);
        printf("\n===END UserUserField MODIFIED ==========================\n");
#endif
	LOG((__FILE__,__FUNCTION__,8,"H323:Q931:RELEASECOMPLETE: created!\n"))

        uuInfo.Encode(uuInfostreamOUT);
	uuInfostreamOUT.CompleteEncoding();
        q931pdu.SetIE(Q931::UserUserIE, uuInfostreamOUT);
        q931pdu.Encode(q931streamOUT);

        c->packet_len=q931streamOUT.GetSize()+TPKT_LENGTH_OFFSET+2;

        c->buffer[TPKT_LENGTH_OFFSET]=  (unsigned char) (((c->packet_len) >> 8) & 0xFFFF) ;
        c->buffer[TPKT_LENGTH_OFFSET+1]=(unsigned char) (((c->packet_len)     ) & 0x00FF) ;
        for(u_int32_t i=0;i<q931streamOUT.GetSize();i++){
                c->buffer[TPKT_LENGTH_OFFSET+2+i]=q931streamOUT[i];
        }

	if(c->connectionWrite(c)<0) {
                WRN((__FILE__,__FUNCTION__,"H323:Q931:RELEASECOMPLETE: write on socket\n"))
                return(-1);
        }

	return(1);
}


int H323Parser::connectDestinationCalledPartyNumberPDU(String destination){
       	String locator;
	String prefix;
	String requestIP;
        String requestPORT;
	if(((*(call.Q931_out))->connectionGetState())!=CONNECT){
       		LOG((__FILE__,__FUNCTION__,8,"H323:Q931:LOCATOR:PDU:\n"))
		if (destination!=String("")) {
        		LOG((__FILE__,__FUNCTION__,8,"H323:Q931:LOCATOR:PDU: CalledPartyNumber contains %s\n",destination.c_str()))
			for (int i=0;i<MAXLOCATORS;++i){
                        	locator = type + "_LOCATOR" + "_PDU" + "_DIGIT" + "_" + i;
				prefix = Global::getGlobal()->getConf()->getValAsString(locator);
                        	if (prefix!=String("")) {
					LOG((__FILE__,__FUNCTION__,8,"H323:Q931:LOCATOR:PDU: testing %s = %s for %s\n",locator.c_str(),prefix.c_str(),destination.c_str()))
                                	if(destination.leftequal(prefix)){
						requestIP   = locator + "_IP";
						requestPORT = locator + "_PORT";
        					if (((Global::getGlobal()->getConf()->getValAsString(requestIP))!=String("")) && 
            		    				((Global::getGlobal()->getConf()->getValAsString(requestPORT))!=String(""))) {
        						(*(call.Q931_out))->p_ip        = Global::getGlobal()->getConf()->getValAsIP(requestIP);
        						(*(call.Q931_out))->p_mask      = 0xffffffff;
        						(*(call.Q931_out))->p_port      = Global::getGlobal()->getConf()->getValAsInt(requestPORT);
							(*(call.Q931_out))->ipproto     = (*(call.Q931_in))->ipproto;
       							if(((*(call.Q931_out))->connectionConnect())<0){
               							WRN((__FILE__,__FUNCTION__,"H323:Q931:LOCATOR:PDU: connecting destination for control channel\n"))
								return(-1);
       							}
							LOG((__FILE__,__FUNCTION__,8,"H323:Q931:LOCATOR:PDU: using destination %s:%d for %s\n",
								(String::IPtoString((*(call.Q931_out))->p_ip)).c_str(),(*(call.Q931_out))->p_port,destination.c_str()))
							parent->data.dstIP=(*(call.Q931_in))->p_ip;
							return(1);
						}
						else {
							WRN((__FILE__,__FUNCTION__,"H323:Q931:LOCATOR:PDU: configuration file incomplete: KEY=%s VAL=%s KEY=%s VAL=%s\n",
                                                		requestIP.c_str()  ,Global::getGlobal()->getConf()->getValAsString(requestIP).c_str(),
                                                		requestPORT.c_str(),Global::getGlobal()->getConf()->getValAsString(requestPORT).c_str()))
						}
					}
                        	}
			}
                        /* use default routing entry if present */
                        locator = type + "_LOCATOR" + "_PDU" + "_DIGIT" + "_DEFAULT";
                        LOG((__FILE__,__FUNCTION__,8,"H323:Q931:LOCATOR:PDU: using %s for %s\n",locator.c_str(),destination.c_str()))
                        requestIP   = locator + "_IP";
                        requestPORT = locator + "_PORT";
                        if (((Global::getGlobal()->getConf()->getValAsString(requestIP))!=String("")) &&
                            ((Global::getGlobal()->getConf()->getValAsString(requestPORT))!=String(""))) {
        				(*(call.Q931_out))->p_ip        = Global::getGlobal()->getConf()->getValAsIP(requestIP);
        				(*(call.Q931_out))->p_mask      = 0xffffffff;
        				(*(call.Q931_out))->p_port      = Global::getGlobal()->getConf()->getValAsInt(requestPORT);
					(*(call.Q931_out))->ipproto     = (*(call.Q931_in))->ipproto;
       					if(((*(call.Q931_out))->connectionConnect())<0){
               					WRN((__FILE__,__FUNCTION__,"H323:Q931:LOCATOR:PDU: connecting destination for control channel\n"))
						return(-1);
       					}
					LOG((__FILE__,__FUNCTION__,8,"H323:Q931:LOCATOR:PDU: using destination %s:%d for %s\n",
						(String::IPtoString((*(call.Q931_out))->p_ip)).c_str(),
						(*(call.Q931_out))->p_port,destination.c_str()))

					parent->data.dstIP=(*(call.Q931_out))->p_ip;
					return(1);
                        }
                        else {
                                WRN((__FILE__,__FUNCTION__,"H323:Q931:LOCATOR:PDU: default routing entry not found in configuration file: KEY=%s VAL=%s KEY=%s VAL=%s\n",
                                            requestIP.c_str()  ,Global::getGlobal()->getConf()->getValAsString(requestIP).c_str(),
                                            requestPORT.c_str(),Global::getGlobal()->getConf()->getValAsString(requestPORT).c_str()))
                        }
		}
		return(-1);	
	}
	else {
	   return(1);
	}
}
int H323Parser::connectDestinationDestinationAddressPDU(H225_ArrayOf_AliasAddress* destinationAddress){
       	String locator;
	String prefix;
	String requestIP;
        String requestPORT;
	String destination;

	if(((*(call.Q931_out))->connectionGetState())!=CONNECT){
           LOG((__FILE__,__FUNCTION__,8,"H323:Q931:LOCATOR:PDU:\n"))
	   for(int j=0;j<(destinationAddress->GetSize());++j){
		if((*destinationAddress)[j].GetTag()==H225_AliasAddress::e_dialedDigits){
			PASN_IA5String & dialedDigits = (*destinationAddress)[j];	
			destination = String((const char *)(dialedDigits.GetValue()));
        		LOG((__FILE__,__FUNCTION__,8,"H323:Q931:LOCATOR:PDU: destinationAddress contains dialedDigits = %s\n",destination.c_str()))
			for (int i=0;i<MAXLOCATORS;++i){
                        	locator = type + "_LOCATOR" + "_PDU" + "_DIGIT" + "_" + i;
				prefix = Global::getGlobal()->getConf()->getValAsString(locator);
                        	if (prefix!=String("")) {
					LOG((__FILE__,__FUNCTION__,8,"H323:Q931:LOCATOR:PDU: testing %s = %s for %s\n",locator.c_str(),prefix.c_str(),destination.c_str()))
                                	if(destination.leftequal(prefix)){
						requestIP   = locator + "_IP";
						requestPORT = locator + "_PORT";
        					if (((Global::getGlobal()->getConf()->getValAsString(requestIP))!=String("")) && 
            		    				((Global::getGlobal()->getConf()->getValAsString(requestPORT))!=String(""))) {
        						(*(call.Q931_out))->p_ip        = Global::getGlobal()->getConf()->getValAsIP(requestIP);
        						(*(call.Q931_out))->p_mask      = 0xffffffff;
        						(*(call.Q931_out))->p_port      = Global::getGlobal()->getConf()->getValAsInt(requestPORT);
							(*(call.Q931_out))->ipproto     = (*(call.Q931_in))->ipproto;
       							if(((*(call.Q931_out))->connectionConnect())<0){
               							WRN((__FILE__,__FUNCTION__,"H323:Q931:LOCATOR:PDU: connecting destination for control channel\n"))
								return(-1);
       							}
							LOG((__FILE__,__FUNCTION__,8,"H323:Q931:LOCATOR:PDU: using destination %s:%d for %s\n",
								(String::IPtoString((*(call.Q931_out))->p_ip)).c_str(),
								(*(call.Q931_out))->p_port,destination.c_str()))

							parent->data.dstIP=(*(call.Q931_out))->p_ip;
							return(1);
						}
						else {
							WRN((__FILE__,__FUNCTION__,"H323:Q931:LOCATOR:PDU: configuration file incomplete: KEY=%s VAL=%s KEY=%s VAL=%s\n",
                                                		requestIP.c_str()  ,Global::getGlobal()->getConf()->getValAsString(requestIP).c_str(),
                                                		requestPORT.c_str(),Global::getGlobal()->getConf()->getValAsString(requestPORT).c_str()))
						}
					}
                        	}
			}
                        /* use default routing entry if present */
                        locator = type + "_LOCATOR" + "_PDU" + "_DIGIT" + "_DEFAULT";
                        LOG((__FILE__,__FUNCTION__,8,"H323:Q931:LOCATOR:PDU: using %s for %s\n",locator.c_str(),destination.c_str()))
                        requestIP   = locator + "_IP";
                        requestPORT = locator + "_PORT";
                        if (((Global::getGlobal()->getConf()->getValAsString(requestIP))!=String("")) &&
                            ((Global::getGlobal()->getConf()->getValAsString(requestPORT))!=String(""))) {
        				(*(call.Q931_out))->p_ip        = Global::getGlobal()->getConf()->getValAsIP(requestIP);
        				(*(call.Q931_out))->p_mask      = 0xffffffff;
        				(*(call.Q931_out))->p_port      = Global::getGlobal()->getConf()->getValAsInt(requestPORT);
					(*(call.Q931_out))->ipproto     = (*(call.Q931_in))->ipproto;
       					if(((*(call.Q931_out))->connectionConnect())<0){
               					WRN((__FILE__,__FUNCTION__,"H323:Q931:LOCATOR:PDU: connecting destination for control channel\n"))
						return(-1);
       					}
					LOG((__FILE__,__FUNCTION__,8,"H323:Q931:LOCATOR:PDU: using destination %s:%d for %s\n",
						(String::IPtoString((*(call.Q931_out))->p_ip)).c_str(),
						(*(call.Q931_out))->p_port,destination.c_str()))

					parent->data.dstIP=(*(call.Q931_out))->p_ip;
					return(1);
                        }
                        else {
                                WRN((__FILE__,__FUNCTION__,"H323:Q931:LOCATOR:PDU: default routing entry not found in configuration file: KEY=%s VAL=%s KEY=%s VAL=%s\n",
                                            requestIP.c_str()  ,Global::getGlobal()->getConf()->getValAsString(requestIP).c_str(),
                                            requestPORT.c_str(),Global::getGlobal()->getConf()->getValAsString(requestPORT).c_str()))
                        }
		}
		if((*destinationAddress)[j].GetTag()==H225_AliasAddress::e_h323_ID){
        		LOG((__FILE__,__FUNCTION__,8,"H323:Q931:LOCATOR:PDU: destinationAddress contains h323_ID (locator not implemented yet)\n"))
		}
		if((*destinationAddress)[j].GetTag()==H225_AliasAddress::e_url_ID){
        		LOG((__FILE__,__FUNCTION__,8,"H323:Q931:LOCATOR:PDU: destinationAddress contains url_ID (locator not implemented yet)\n"))
		}
		if((*destinationAddress)[j].GetTag()==H225_AliasAddress::e_transportID){
        		LOG((__FILE__,__FUNCTION__,8,"H323:Q931:LOCATOR:PDU: destinationAddress contains transportID (locator not implemented yet)\n"))
		}
		if((*destinationAddress)[j].GetTag()==H225_AliasAddress::e_email_ID){
        		LOG((__FILE__,__FUNCTION__,8,"H323:Q931:LOCATOR:PDU: destinationAddress contains email_ID (locator not implemented yet)\n"))
		}
		if((*destinationAddress)[j].GetTag()==H225_AliasAddress::e_partyNumber){
        		LOG((__FILE__,__FUNCTION__,8,"H323:Q931:LOCATOR:PDU: destinationAddress contains partyNumber (locator not implemented yet)\n"))
		}
		if((*destinationAddress)[j].GetTag()==H225_AliasAddress::e_mobileUIM){
        		LOG((__FILE__,__FUNCTION__,8,"H323:Q931:LOCATOR:PDU: destinationAddress contains mobileUIM (locator not implemented yet)\n"))
		}
           }
           return(-1);
	}
	else {
	   return(1);
	}
}

int H323Parser::connectDestinationSPECIAL(String locator){
	//if (locator==String("PDU"))    return(connectDestinationPDU());
        return(-1);
}
int H323Parser::connectDestinationFAST(){
        connectDestination();
	return(1);
}

int H323Parser::setSetupDestCallSignalAddress(H225_TransportAddress* destCallSignalAddress){
	String logmessage="H323:Q931:SETUP:";
	if(call.Q931_out==NULL) return(-1);
	if(destCallSignalAddress->GetTag()==H225_TransportAddress::e_ipAddress){
		H225_TransportAddress_ipAddress & h225_transportaddress_ipaddress=(*destCallSignalAddress);
		LOG((__FILE__,__FUNCTION__,8,"%s destCallSignalAddress %d.%d.%d.%d:%d -> %s:%d\n",logmessage.c_str(),
			h225_transportaddress_ipaddress.m_ip[0],
			h225_transportaddress_ipaddress.m_ip[1],
			h225_transportaddress_ipaddress.m_ip[2],
			h225_transportaddress_ipaddress.m_ip[3],
			h225_transportaddress_ipaddress.m_port.GetValue(),
			String::IPtoString((*(call.Q931_out))->p_ip).c_str(),(*(call.Q931_out))->p_port))

		h225_transportaddress_ipaddress.m_port.SetValue((*(call.Q931_out))->p_port);
		h225_transportaddress_ipaddress.m_ip[0]=((((*(call.Q931_out))->p_ip)&0xff));
		h225_transportaddress_ipaddress.m_ip[1]=((((*(call.Q931_out))->p_ip)&0xff00) >>8);
		h225_transportaddress_ipaddress.m_ip[2]=((((*(call.Q931_out))->p_ip)&0xff0000) >>16);
		h225_transportaddress_ipaddress.m_ip[3]=((((*(call.Q931_out))->p_ip)&0xff000000)>>24);
	}
	else {
		WRN((__FILE__,__FUNCTION__,"%s H225_TransportAddress is not an ip address\n",logmessage.c_str()))
		return(-1);
	}
	return(0);
}
int H323Parser::setSetupSourceCallSignalAddress(H225_TransportAddress* sourceCallSignalAddress){
	String logmessage="H323:Q931:SETUP:";
	if(call.Q931_out==NULL) return(-1);
	if(sourceCallSignalAddress->GetTag()==H225_TransportAddress::e_ipAddress){
		H225_TransportAddress_ipAddress & h225_transportaddress_ipaddress=(*sourceCallSignalAddress);
		LOG((__FILE__,__FUNCTION__,8,"%s sourceCallSignalAddress %d.%d.%d.%d:%d -> %s:%d\n",logmessage.c_str(),
			h225_transportaddress_ipaddress.m_ip[0],
			h225_transportaddress_ipaddress.m_ip[1],
			h225_transportaddress_ipaddress.m_ip[2],
			h225_transportaddress_ipaddress.m_ip[3],
			h225_transportaddress_ipaddress.m_port.GetValue(),
			String::IPtoString((*(call.Q931_out))->l_ip).c_str(),(*(call.Q931_out))->l_port))

		h225_transportaddress_ipaddress.m_port.SetValue((*(call.Q931_out))->l_port);
		h225_transportaddress_ipaddress.m_ip[0]=((((*(call.Q931_out))->l_ip)&0xff));
		h225_transportaddress_ipaddress.m_ip[1]=((((*(call.Q931_out))->l_ip)&0xff00) >>8);
		h225_transportaddress_ipaddress.m_ip[2]=((((*(call.Q931_out))->l_ip)&0xff0000) >>16);
		h225_transportaddress_ipaddress.m_ip[3]=((((*(call.Q931_out))->l_ip)&0xff000000)>>24);
	}
	else {
		WRN((__FILE__,__FUNCTION__,"%s H225_TransportAddress is not an ip address\n",logmessage.c_str()))
		return(-1);
	}
}

int H323Parser::setProgressH245Address(H225_TransportAddress* h245Address){
	String logmessage="H323:Q931:PROGRESS:";
	return setH245Address(h245Address,logmessage);	
}
int H323Parser::setFacilityH245Address(H225_TransportAddress* h245Address){
	String logmessage="H323:Q931:FACILITY:";
	return setH245Address(h245Address,logmessage);	
}
int H323Parser::setAlertingH245Address(H225_TransportAddress* h245Address){
	String logmessage="H323:Q931:ALERTING:";
	return setH245Address(h245Address,logmessage);	
}
int H323Parser::setCallProceedingH245Address(H225_TransportAddress* h245Address){
	String logmessage="H323:Q931:CALLPROCEEDING:";
	return setH245Address(h245Address,logmessage);	
}
int H323Parser::setConnectH245Address(H225_TransportAddress* h245Address){
	String logmessage="H323:Q931:CONNECT:";
	return setH245Address(h245Address,logmessage);	
}
int H323Parser::setSetupH245Address(H225_TransportAddress* h245Address){
	String logmessage="H323:Q931:SETUP:";
	return setH245Address(h245Address,logmessage);	
}

int H323Parser::setH245Address(H225_TransportAddress* h245Address,String logmessage){
	if(h245Address->GetTag()!=H245_UnicastAddress::e_iPAddress){
		WRN((__FILE__,__FUNCTION__,"%s H225_TransportAddress is not an ip address\n",logmessage.c_str()))
		return(-1);
	}
	H225_TransportAddress_ipAddress & h225_transportaddress_ipaddress=(*h245Address);

	if(call.useH245Channel(parent)<0) {
		WRN((__FILE__,__FUNCTION__,"%s no ressources available for new H.245 channel\n",logmessage.c_str()))
		return(-1);
	}

	LOG((__FILE__,__FUNCTION__,8,"%s H.245 to %d.%d.%d.%d/%d\n",logmessage.c_str(),
			h225_transportaddress_ipaddress.m_ip[0],h225_transportaddress_ipaddress.m_ip[1],
			h225_transportaddress_ipaddress.m_ip[2],h225_transportaddress_ipaddress.m_ip[3],
			h225_transportaddress_ipaddress.m_port.GetValue()))
	
        (*(call.H245_out))->p_ip       = ((h225_transportaddress_ipaddress.m_ip[0])) |
					((h225_transportaddress_ipaddress.m_ip[1]) << 8) | 
					((h225_transportaddress_ipaddress.m_ip[2]) << 16) | 
					((h225_transportaddress_ipaddress.m_ip[3]) << 24); 
        (*(call.H245_out))->p_mask     = 0xffffffff;
        (*(call.H245_out))->p_port     = h225_transportaddress_ipaddress.m_port.GetValue();

	h225_transportaddress_ipaddress.m_port.SetValue((*(call.H245_in))->l_port);
	h225_transportaddress_ipaddress.m_ip[0]=((((*(call.H245_in))->l_ip)&0xff));
	h225_transportaddress_ipaddress.m_ip[1]=((((*(call.H245_in))->l_ip)&0xff00) >>8);
	h225_transportaddress_ipaddress.m_ip[2]=((((*(call.H245_in))->l_ip)&0xff0000) >>16);
	h225_transportaddress_ipaddress.m_ip[3]=((((*(call.H245_in))->l_ip)&0xff000000)>>24);
					
	LOG((__FILE__,__FUNCTION__,8,"%s H.245 %s:%d -> %s:%d\n",logmessage.c_str(),
		String::IPtoString((*(call.H245_out))->p_ip).c_str(),(*(call.H245_out))->p_port,
		String::IPtoString((*(call.H245_in))->l_ip).c_str(),(*(call.H245_in))->l_port))

	return(0);
}

int H323Parser::setCloseLogicalChannel(int id_a,int id_b){
	String logmessage="H323:H245:REQUEST:CLOSELOGICALCHANNEL";
	
	/* implementation for DATAFASTPATH is missing */

	if((call.delMediaChannel(id_a,id_b))<0){
			LOG((__FILE__,__FUNCTION__,8,"%s channel not found \n",logmessage.c_str()))
	}
	else{
			LOG((__FILE__,__FUNCTION__,8,"%s channel closed\n",logmessage.c_str()))
	}
	return(0);
}

int H323Parser::setOpenLogicalChannelAckMediaChannel(H245_UnicastAddress* RTPAddress,int id_a,int id_b){
	String logmessage="H323:H245:RESPONSE:OPENLOGICALCHANNELACK:";
	H323MediaChannel *mediaChannel;

	if(RTPAddress->GetTag()!=H245_UnicastAddress::e_iPAddress){
		WRN((__FILE__,__FUNCTION__,"%s H245_UnicastAddress is not an IP-address\n",logmessage.c_str()))
		return(-1);
	}
	H245_UnicastAddress_iPAddress & h245_unicastaddress_ipaddress=(*RTPAddress);

	if ((Global::getGlobal()->getConf()->getValAsInt(type+"_DATAFASTPATH"))==0){
		if(call.getMediaChannel(id_a,id_b)<0){
                	WRN((__FILE__,__FUNCTION__,"%s media channel not found\n",logmessage.c_str()))
                	return(-1);
        	}
		mediaChannel=call.mediaChannel[call.getMediaChannel(id_a,id_b)];
		LOG((__FILE__,__FUNCTION__,8,"%s RTP to %d.%d.%d.%d/%d\n",logmessage.c_str(),
			h245_unicastaddress_ipaddress.m_network[0],h245_unicastaddress_ipaddress.m_network[1],
			h245_unicastaddress_ipaddress.m_network[2],h245_unicastaddress_ipaddress.m_network[3],
			h245_unicastaddress_ipaddress.m_tsapIdentifier.GetValue()))
		(*(mediaChannel->RTP_in))->p_ip = ((h245_unicastaddress_ipaddress.m_network[0])) |
					((h245_unicastaddress_ipaddress.m_network[1]) << 8) |
					((h245_unicastaddress_ipaddress.m_network[2]) << 16) |
					((h245_unicastaddress_ipaddress.m_network[3]) << 24); 
		(*(mediaChannel->RTP_in))->p_mask     = 0xffffffff;
		(*(mediaChannel->RTP_in))->p_port     = (h245_unicastaddress_ipaddress.m_tsapIdentifier.GetValue());
		h245_unicastaddress_ipaddress.m_tsapIdentifier.SetValue((*(mediaChannel->RTP_out))->l_port);
		h245_unicastaddress_ipaddress.m_network[0]=((((*(mediaChannel->RTP_out))->l_ip)&0xff));
		h245_unicastaddress_ipaddress.m_network[1]=((((*(mediaChannel->RTP_out))->l_ip)&0xff00) >>8);
		h245_unicastaddress_ipaddress.m_network[2]=((((*(mediaChannel->RTP_out))->l_ip)&0xff0000) >>16);
		h245_unicastaddress_ipaddress.m_network[3]=((((*(mediaChannel->RTP_out))->l_ip)&0xff000000)>>24);
		LOG((__FILE__,__FUNCTION__,8,"%s RTP %s:%d -> %s:%d\n",logmessage.c_str(),
			String::IPtoString((*(mediaChannel->RTP_in))->p_ip).c_str(),(*(mediaChannel->RTP_in))->p_port,
			String::IPtoString((*(mediaChannel->RTP_out))->l_ip).c_str(),(*(mediaChannel->RTP_out))->l_port))
	}
	else {
		Fwfil *fwfilp;
                fwfilp=new Fwfil("RTP",17,0
                                         ,0
                                         ,FWFIL_PORTRANGE_GREATERT
                                         ,1023
                                         ,(((h245_unicastaddress_ipaddress.m_network[0])) |
					  ((h245_unicastaddress_ipaddress.m_network[1]) << 8) |
					  ((h245_unicastaddress_ipaddress.m_network[2]) << 16) |
					  ((h245_unicastaddress_ipaddress.m_network[3]) << 24) )
                                         ,0xffffffff
                                         ,FWFIL_PORTRANGE_EQUAL
                                         ,(h245_unicastaddress_ipaddress.m_tsapIdentifier.GetValue())
                 );
                 if(call.fwhndl.getChannel(fwfilp)<0){
                 	WRN((__FILE__,__FUNCTION__,"%s could not configure firewall for connection\n",logmessage.c_str()))
                 }
	}
	return(1);
}

int H323Parser::setOpenLogicalChannelAckMediaControlChannel(H245_UnicastAddress* RTPAddress,int id_a,int id_b){
	String logmessage="H323:H245:RESPONSE:OPENLOGICALCHANNELACK:";
	H323MediaChannel *mediaChannel;

	if(RTPAddress->GetTag()!=H245_UnicastAddress::e_iPAddress){
		WRN((__FILE__,__FUNCTION__,"%s H245_UnicastAddress is not an IP-address\n",logmessage.c_str()))
		return(-1);
	}
	H245_UnicastAddress_iPAddress & h245_unicastaddress_ipaddress=(*RTPAddress);

	if ((Global::getGlobal()->getConf()->getValAsInt(type+"_DATAFASTPATH"))==0){
		if(call.getMediaChannel(id_a,id_b)<0){
                	WRN((__FILE__,__FUNCTION__,"%s media channel not found\n",logmessage.c_str()))
                	return(-1);
        	}
		mediaChannel=call.mediaChannel[call.getMediaChannel(id_a,id_b)];
		LOG((__FILE__,__FUNCTION__,8,"%s RTCP to %d.%d.%d.%d/%d\n",logmessage.c_str(),
			h245_unicastaddress_ipaddress.m_network[0],h245_unicastaddress_ipaddress.m_network[1],
			h245_unicastaddress_ipaddress.m_network[2],h245_unicastaddress_ipaddress.m_network[3],
			h245_unicastaddress_ipaddress.m_tsapIdentifier.GetValue()))
		(*(mediaChannel->RTCP_in))->p_ip = ((h245_unicastaddress_ipaddress.m_network[0])) |
					((h245_unicastaddress_ipaddress.m_network[1]) << 8) |
					((h245_unicastaddress_ipaddress.m_network[2]) << 16) |
					((h245_unicastaddress_ipaddress.m_network[3]) << 24); 
		(*(mediaChannel->RTCP_in))->p_mask     = 0xffffffff;
		(*(mediaChannel->RTCP_in))->p_port     = (h245_unicastaddress_ipaddress.m_tsapIdentifier.GetValue());
		h245_unicastaddress_ipaddress.m_tsapIdentifier.SetValue((*(mediaChannel->RTCP_out))->l_port);
		h245_unicastaddress_ipaddress.m_network[0]=((((*(mediaChannel->RTCP_out))->l_ip)&0xff));
		h245_unicastaddress_ipaddress.m_network[1]=((((*(mediaChannel->RTCP_out))->l_ip)&0xff00) >>8);
		h245_unicastaddress_ipaddress.m_network[2]=((((*(mediaChannel->RTCP_out))->l_ip)&0xff0000) >>16);
		h245_unicastaddress_ipaddress.m_network[3]=((((*(mediaChannel->RTCP_out))->l_ip)&0xff000000)>>24);
		LOG((__FILE__,__FUNCTION__,8,"%s RTCP %s:%d -> %s:%d\n",logmessage.c_str(),
			String::IPtoString((*(mediaChannel->RTCP_in))->p_ip).c_str(),(*(mediaChannel->RTCP_in))->p_port,
			String::IPtoString((*(mediaChannel->RTCP_out))->l_ip).c_str(),(*(mediaChannel->RTCP_out))->l_port))

	}
	else {
		Fwfil *fwfilp;
                fwfilp=new Fwfil("RTCP",17,0
                                         ,0
                                         ,FWFIL_PORTRANGE_GREATERT
                                         ,1023
                                         ,(((h245_unicastaddress_ipaddress.m_network[0])) |
					  ((h245_unicastaddress_ipaddress.m_network[1]) << 8) |
					  ((h245_unicastaddress_ipaddress.m_network[2]) << 16) |
					  ((h245_unicastaddress_ipaddress.m_network[3]) << 24) )
                                         ,0xffffffff
                                         ,FWFIL_PORTRANGE_EQUAL
                                         ,(h245_unicastaddress_ipaddress.m_tsapIdentifier.GetValue())
                 );
                 if(call.fwhndl.getChannel(fwfilp)<0){
                 	WRN((__FILE__,__FUNCTION__,"%s could not configure firewall for connection\n",logmessage.c_str()))
                 }
	}
	return(1);
}

int H323Parser::setOpenLogicalChannelMediaChannel(H245_UnicastAddress* RTPAddress,int id_a,int id_b){
	String logmessage="H323:H245:REQUEST:OPENLOGICALCHANNEL:";
	ERR((__FILE__,__FUNCTION__,"%s RTP specification in OPENLOGICALCHANNEL, this is not h.323 conform!\n",logmessage.c_str()))
        return(-1);
}

int H323Parser::setOpenLogicalChannelMediaControlChannel(H245_UnicastAddress* RTPAddress,int id_a,int id_b){
	String logmessage="H323:H245:REQUEST:OPENLOGICALCHANNEL:";
	H323MediaChannel *mediaChannel;

	if(RTPAddress->GetTag()!=H245_UnicastAddress::e_iPAddress){
		WRN((__FILE__,__FUNCTION__,"%s H245_UnicastAddress is not an IP-address\n",logmessage.c_str()))
		return(-1);
	}
	H245_UnicastAddress_iPAddress & h245_unicastaddress_ipaddress=(*RTPAddress);

	if ((Global::getGlobal()->getConf()->getValAsInt(type+"_DATAFASTPATH"))==0){
		if(call.useMediaChannel(parent,id_a,id_b)<0){
                	WRN((__FILE__,__FUNCTION__,"%s no ressources available for new media channel\n",logmessage.c_str()))
                	return(-1);
        	}
		mediaChannel=call.mediaChannel[call.getMediaChannel(id_a,id_b)];
		LOG((__FILE__,__FUNCTION__,8,"%s RTCP to %d.%d.%d.%d/%d\n",logmessage.c_str(),
			h245_unicastaddress_ipaddress.m_network[0],h245_unicastaddress_ipaddress.m_network[1],
			h245_unicastaddress_ipaddress.m_network[2],h245_unicastaddress_ipaddress.m_network[3],
			h245_unicastaddress_ipaddress.m_tsapIdentifier.GetValue()))
		(*(mediaChannel->RTCP_out))->p_ip = ((h245_unicastaddress_ipaddress.m_network[0])) |
					((h245_unicastaddress_ipaddress.m_network[1]) << 8) |
					((h245_unicastaddress_ipaddress.m_network[2]) << 16) |
					((h245_unicastaddress_ipaddress.m_network[3]) << 24); 
		(*(mediaChannel->RTCP_out))->p_mask     = 0xffffffff;
		(*(mediaChannel->RTCP_out))->p_port     = (h245_unicastaddress_ipaddress.m_tsapIdentifier.GetValue());
		h245_unicastaddress_ipaddress.m_tsapIdentifier.SetValue((*(mediaChannel->RTCP_in))->l_port);
		h245_unicastaddress_ipaddress.m_network[0]=((((*(mediaChannel->RTCP_in))->l_ip)&0xff));
		h245_unicastaddress_ipaddress.m_network[1]=((((*(mediaChannel->RTCP_in))->l_ip)&0xff00) >>8);
		h245_unicastaddress_ipaddress.m_network[2]=((((*(mediaChannel->RTCP_in))->l_ip)&0xff0000) >>16);
		h245_unicastaddress_ipaddress.m_network[3]=((((*(mediaChannel->RTCP_in))->l_ip)&0xff000000)>>24);
		LOG((__FILE__,__FUNCTION__,8,"%s RTCP %s:%d -> %s:%d\n",logmessage.c_str(),
			String::IPtoString((*(mediaChannel->RTCP_out))->p_ip).c_str(),(*(mediaChannel->RTCP_out))->p_port,
			String::IPtoString((*(mediaChannel->RTCP_in))->l_ip).c_str(),(*(mediaChannel->RTCP_in))->l_port))
	}
	else {
		/* should be set together with OpenLogicalChannelMediaChannelACK to reduce rules */
		Fwfil *fwfilp;
                fwfilp=new Fwfil("RTCP",17,0
                                         ,0
                                         ,FWFIL_PORTRANGE_GREATERT
                                         ,1023
                                         ,(((h245_unicastaddress_ipaddress.m_network[0])) |
					  ((h245_unicastaddress_ipaddress.m_network[1]) << 8) |
					  ((h245_unicastaddress_ipaddress.m_network[2]) << 16) |
					  ((h245_unicastaddress_ipaddress.m_network[3]) << 24) )
                                         ,0xffffffff
                                         ,FWFIL_PORTRANGE_EQUAL
                                         ,(h245_unicastaddress_ipaddress.m_tsapIdentifier.GetValue())
                 );
                 if(call.fwhndl.getChannel(fwfilp)<0){
                 	WRN((__FILE__,__FUNCTION__,"%s could not configure firewall for connection\n",logmessage.c_str()))
                 }
	}
	return(1);
}

int H323Parser::write_0(Connection *dc, Connection *sc){
	LOG((__FILE__,__FUNCTION__,15,"H323:Q931: write_0 --> write_default\n"))
	return(write_default(dc,sc));
}
int H323Parser::write_1(Connection *dc, Connection *sc){
	LOG((__FILE__,__FUNCTION__,15,"H323:H245: write_1 --> write_default\n"))
	return(write_default(dc,sc));
}
int H323Parser::write_2(Connection *dc, Connection *sc){
	LOG((__FILE__,__FUNCTION__,15,"H323:RTP: write_2 --> write_default\n"))
	return(write_default(dc,sc));
}
int H323Parser::write_3(Connection *dc, Connection *sc){
	LOG((__FILE__,__FUNCTION__,15,"H323:RTCP: write_3 --> write_default\n"))
	return(write_default(dc,sc));
}
int H323Parser::write_default(Connection *dc, Connection *sc){
        LOG((__FILE__,__FUNCTION__,15,"H323: write_default\n"))
        if(dc->connectionWrite(sc)<0) {
                WRN((__FILE__,__FUNCTION__,"H323: write on socket\n"))
                return(-1);
        }
        return(sc->packet_len);
}

int H323Parser::read_0(Connection *c){
        int     len=0;
	LOG((__FILE__,__FUNCTION__,15,"H323:Q931: read_0\n"))
	c->zeroBuffer();

	/* read PDU up to length field */
	if ((len=c->connectionRead(TPKT_LENGTH_OFFSET+2))<(TPKT_LENGTH_OFFSET+2)) {
                        if (len==0) return(0);
                        return(-1);
        }
	len=(((unsigned char)(c->buffer[TPKT_LENGTH_OFFSET])) << 8)| ((unsigned char)(c->buffer[TPKT_LENGTH_OFFSET+1]));
	LOG((__FILE__,__FUNCTION__,15,"H323:Q931: PDU length: %d\n",len))

	if ((len=c->connectionRead(len-(TPKT_LENGTH_OFFSET+2)))<=0) return(len);
        LOG((__FILE__,__FUNCTION__,15,"H323:Q931: read_default, length: %d\n",len))
        return(c->packet_len);
}
int H323Parser::read_1(Connection *c){
        int     len=0;
	LOG((__FILE__,__FUNCTION__,15,"H323:H245: read_1\n"))
	c->zeroBuffer();

	/* read PDU up to length field */
	if ((len=c->connectionRead(TPKT_LENGTH_OFFSET+2))<(TPKT_LENGTH_OFFSET+2)) {
                        if (len==0) return(0);
                        return(-1);
        }
	len=(((unsigned char)(c->buffer[TPKT_LENGTH_OFFSET])) << 8)| ((unsigned char)(c->buffer[TPKT_LENGTH_OFFSET+1]));
	LOG((__FILE__,__FUNCTION__,15,"H323:H245: PDU length: %d\n",len))

	if ((len=c->connectionRead(len-(TPKT_LENGTH_OFFSET+2)))<=0) return(len);
        LOG((__FILE__,__FUNCTION__,15,"H323:H245: read_default, length: %d\n",len))
        return(c->packet_len);
}
int H323Parser::read_2(Connection *c){
	LOG((__FILE__,__FUNCTION__,15,"H323:RTP: read_2\n"))
	return(read_default(c));
}
int H323Parser::read_3(Connection *c){
	LOG((__FILE__,__FUNCTION__,15,"H323:RTCP: read_3\n"))
	return(read_default(c));
}
int H323Parser::read_default(Connection *c){
        int     len=0;
	LOG((__FILE__,__FUNCTION__,15,"H323: read_default\n"))
        c->zeroBuffer();
        if ((len=c->connectionRead(BUFFERSIZE-1))<0) return(-1);
        LOG((__FILE__,__FUNCTION__,15,"H323: read_default, length: %d\n",len))
        return(c->packet_len);
}

int H323Parser::handleQ931UserUserIE(H225_H323_UserInformation *uuInfo,bool direction){
#ifdef _H323DEBUG_
	printf("\n===BEGIN UserUserField DECODED =========================\n");
        uuInfo.PrintOn(cout);
        printf("\n===END UserUserField DECODED ===========================\n");
#endif
	if (uuInfo->m_h323_uu_pdu.HasOptionalField(H225_H323_UU_PDU::e_h245Tunneling)) {
		if(uuInfo->m_h323_uu_pdu.m_h245Tunneling==true){
               		WRN((__FILE__,__FUNCTION__,"H323:Q931: NOT IMPLEMENTED h245Tunneling=true -> h245Tunneling=false\n"))
			uuInfo->m_h323_uu_pdu.m_h245Tunneling=false;
		}
	}
        if (uuInfo->m_h323_uu_pdu.m_h323_message_body.GetTag()==H225_H323_UU_PDU_h323_message_body::e_setup) {
                LOG((__FILE__,__FUNCTION__,8,"H323:Q931:SETUP: PDU RECEIVED\n"))
		if (!checkPDU("Q931_setup")) return(0);
		H225_Setup_UUIE & h225_setup_uuie=uuInfo->m_h323_uu_pdu.m_h323_message_body;
		if(h225_setup_uuie.HasOptionalField(H225_Setup_UUIE::e_destinationAddress)) {
			connectDestinationDestinationAddressPDU(&h225_setup_uuie.m_destinationAddress);
		}
		if(h225_setup_uuie.HasOptionalField(H225_Setup_UUIE::e_destCallSignalAddress))   if(setSetupDestCallSignalAddress(&h225_setup_uuie.m_destCallSignalAddress)<0) return(-1);
		if(h225_setup_uuie.HasOptionalField(H225_Setup_UUIE::e_sourceCallSignalAddress)) if(setSetupSourceCallSignalAddress(&h225_setup_uuie.m_sourceCallSignalAddress)<0) return(-1);
		if(h225_setup_uuie.HasOptionalField(H225_Setup_UUIE::e_h245Address)) if(setSetupH245Address(&h225_setup_uuie.m_h245Address)<0) return(-1);
		if(h225_setup_uuie.HasOptionalField(H225_Setup_UUIE::e_fastStart)) {
                       	WRN((__FILE__,__FUNCTION__,"H323:Q931:SETUP: NOT IMPLEMENTED fast start detected -> removing faststart options\n"))
			h225_setup_uuie.m_fastStart.SetSize(0);
		}
			
	}
        else if (uuInfo->m_h323_uu_pdu.m_h323_message_body.GetTag()==H225_H323_UU_PDU_h323_message_body::e_callProceeding) {
        	LOG((__FILE__,__FUNCTION__,8,"H323:Q931:CALLPROCEEDING: PDU RECEIVED\n"))
		if (!checkPDU("Q931_callProceeding")) return(0);
		H225_CallProceeding_UUIE & h225_callproceeding_uuie=uuInfo->m_h323_uu_pdu.m_h323_message_body;
		if(h225_callproceeding_uuie.HasOptionalField(H225_CallProceeding_UUIE::e_h245Address)) 
			if(setCallProceedingH245Address(&h225_callproceeding_uuie.m_h245Address)<0) return(-1);
		if(h225_callproceeding_uuie.HasOptionalField(H225_CallProceeding_UUIE::e_fastStart)) {
                       	WRN((__FILE__,__FUNCTION__,"H323:Q931:CALLPROCEEDING: NOT IMPLEMENTED fast start detected -> removing faststart options\n"))
			h225_callproceeding_uuie.m_fastStart.SetSize(0);
		}
        }
        else if (uuInfo->m_h323_uu_pdu.m_h323_message_body.GetTag()==H225_H323_UU_PDU_h323_message_body::e_connect) {
                LOG((__FILE__,__FUNCTION__,8,"H323:Q931:CONNECT: PDU RECEIVED\n"))
		if (!checkPDU("Q931_connect")) return(0);
		H225_Connect_UUIE & h225_connect_uuie=uuInfo->m_h323_uu_pdu.m_h323_message_body;
		if(h225_connect_uuie.HasOptionalField(H225_Connect_UUIE::e_h245Address)) 
			if(setConnectH245Address(&h225_connect_uuie.m_h245Address)<0) return(-1);
		if(h225_connect_uuie.HasOptionalField(H225_Connect_UUIE::e_fastStart)) {
                       	WRN((__FILE__,__FUNCTION__,"H323:Q931:CONNECT: NOT IMPLEMENTED fast start detected -> removing faststart options\n"))
			h225_connect_uuie.m_fastStart.SetSize(0);
		}
        }
        else if (uuInfo->m_h323_uu_pdu.m_h323_message_body.GetTag()==H225_H323_UU_PDU_h323_message_body::e_alerting){ 
                LOG((__FILE__,__FUNCTION__,8,"H323:Q931:ALERTING: PDU RECEIVED\n"))
		if (!checkPDU("Q931_alerting")) return(0);
		H225_Alerting_UUIE & h225_alerting_uuie=uuInfo->m_h323_uu_pdu.m_h323_message_body;
		if(h225_alerting_uuie.HasOptionalField(H225_Alerting_UUIE::e_h245Address)) 
			if(setAlertingH245Address(&h225_alerting_uuie.m_h245Address)<0) return(-1);
		if(h225_alerting_uuie.HasOptionalField(H225_Alerting_UUIE::e_fastStart)) {
                       	WRN((__FILE__,__FUNCTION__,"H323:Q931:ALERTING: NOT IMPLEMENTED fast start detected -> removing faststart options\n"))
			h225_alerting_uuie.m_fastStart.SetSize(0);
		}
        }
        else if (uuInfo->m_h323_uu_pdu.m_h323_message_body.GetTag()==H225_H323_UU_PDU_h323_message_body::e_information){ 
                LOG((__FILE__,__FUNCTION__,8,"H323:Q931:INFORMATION: PDU RECEIVED\n"))
		if (!checkPDU("Q931_information")) return(0);
        }
        else if (uuInfo->m_h323_uu_pdu.m_h323_message_body.GetTag()==H225_H323_UU_PDU_h323_message_body::e_releaseComplete){ 
                LOG((__FILE__,__FUNCTION__,8,"H323:Q931:RELEASECOMPLETE: PDU RECEIVED\n"))
		if (!checkPDU("Q931_releaseComplete")) return(0);
        }
        else if (uuInfo->m_h323_uu_pdu.m_h323_message_body.GetTag()==H225_H323_UU_PDU_h323_message_body::e_facility){ 
                LOG((__FILE__,__FUNCTION__,8,"H323:Q931:FACILITY: PDU RECEIVED\n"))
		if (!checkPDU("Q931_facility")) return(0);
		H225_Facility_UUIE & h225_facility_uuie=uuInfo->m_h323_uu_pdu.m_h323_message_body;
		if(h225_facility_uuie.HasOptionalField(H225_Facility_UUIE::e_h245Address)) 
			if(setFacilityH245Address(&h225_facility_uuie.m_h245Address)<0) return(-1);
		if(h225_facility_uuie.HasOptionalField(H225_Facility_UUIE::e_fastStart)) {
                       	WRN((__FILE__,__FUNCTION__,"H323:Q931:FACILITY: NOT IMPLEMENTED fast start detected -> removing faststart options\n"))
			h225_facility_uuie.m_fastStart.SetSize(0);
		}
        }
        else if (uuInfo->m_h323_uu_pdu.m_h323_message_body.GetTag()==H225_H323_UU_PDU_h323_message_body::e_progress){ 
                LOG((__FILE__,__FUNCTION__,8,"H323:Q931:PROGRESS: PDU RECEIVED\n"))
		if (!checkPDU("Q931_progress")) return(0);
		H225_Progress_UUIE & h225_progress_uuie=uuInfo->m_h323_uu_pdu.m_h323_message_body;
		if(h225_progress_uuie.HasOptionalField(H225_Progress_UUIE::e_h245Address)) 
			if(setProgressH245Address(&h225_progress_uuie.m_h245Address)<0) return(-1);
		if(h225_progress_uuie.HasOptionalField(H225_Progress_UUIE::e_fastStart)) {
                       	WRN((__FILE__,__FUNCTION__,"H323:Q931:PROGRESS: NOT IMPLEMENTED fast start detected -> removing faststart options\n"))
			h225_progress_uuie.m_fastStart.SetSize(0);
		}
	}
        else if (uuInfo->m_h323_uu_pdu.m_h323_message_body.GetTag()==H225_H323_UU_PDU_h323_message_body::e_empty){ 
                LOG((__FILE__,__FUNCTION__,8,"H323:Q931:EMPTY: PDU RECEIVED\n"))
		if (!checkPDU("Q931_empty")) return(0);
        }
	else {
                WRN((__FILE__,__FUNCTION__,"H323:Q931:UNKNOWN: PDU RECEIVED\n"))
		return(0);
	}
#ifdef _H323DEBUG_
        printf("\n===BEGIN UserUserField MODIFIED ========================\n");
        uuInfo->PrintOn(cout);
        printf("\n===END UserUserField MODIFIED ==========================\n");
#endif
	return(1);
}

int H323Parser::handleH245Request(H245_RequestMessage *h245_requestmessage,bool direction){
	if(h245_requestmessage->GetTag()==H245_RequestMessage::e_nonStandard){
               	LOG((__FILE__,__FUNCTION__,8,"H323:H245:REQUEST:NONSTANDARD:\n"))
	}
	else if(h245_requestmessage->GetTag()==H245_RequestMessage::e_masterSlaveDetermination){
               	LOG((__FILE__,__FUNCTION__,8,"H323:H245:REQUEST:MASTERSLAVEDETERMINATION:\n"))
	}
        else if(h245_requestmessage->GetTag()==H245_RequestMessage::e_terminalCapabilitySet){
               	LOG((__FILE__,__FUNCTION__,8,"H323:H245:REQUEST:TERMINALCAPABILITYSET:\n"))
	}
        else if(h245_requestmessage->GetTag()==H245_RequestMessage::e_openLogicalChannel){
               	LOG((__FILE__,__FUNCTION__,8,"H323:H245:REQUEST:OPENLOGICALCHANNEL:\n"))
		H245_OpenLogicalChannel & h245_openlogicalchannel=(*h245_requestmessage);
		if (!checkOpenLogicalChannel(&h245_openlogicalchannel)){
			sendOpenLogicalChannelReject(direction,h245_openlogicalchannel.m_forwardLogicalChannelNumber.GetValue());
			return(0);
		}
		/* forwarLogicalChannelParameters */
		if(h245_openlogicalchannel.m_forwardLogicalChannelParameters.m_multiplexParameters.GetTag()==H245_OpenLogicalChannel_forwardLogicalChannelParameters_multiplexParameters::e_h2250LogicalChannelParameters) {
			H245_H2250LogicalChannelParameters & h245_h2250logicalchannelparameters=(h245_openlogicalchannel.m_forwardLogicalChannelParameters.m_multiplexParameters);
			if (h245_h2250logicalchannelparameters.HasOptionalField(H245_H2250LogicalChannelParameters::e_mediaChannel)) {
				if(h245_h2250logicalchannelparameters.m_mediaChannel.GetTag()==H245_TransportAddress::e_unicastAddress){
					H245_UnicastAddress & h245_unicastaddress=h245_h2250logicalchannelparameters.m_mediaChannel;
					if(setOpenLogicalChannelMediaChannel(&h245_unicastaddress,h245_openlogicalchannel.m_forwardLogicalChannelNumber.GetValue(),direction)<0){
						sendOpenLogicalChannelReject(direction,h245_openlogicalchannel.m_forwardLogicalChannelNumber.GetValue());
						return(0);
					}
				}
				else {
					WRN((__FILE__,__FUNCTION__,"H323:H245:REQUEST:OPENLOGICALCHANNEL: H245_TransportAddress is not a unicast address\n"))
					sendOpenLogicalChannelReject(direction,h245_openlogicalchannel.m_forwardLogicalChannelNumber.GetValue());
					return(0);
				}
			}
			if (h245_h2250logicalchannelparameters.HasOptionalField(H245_H2250LogicalChannelParameters::e_mediaControlChannel)) {
				if(h245_h2250logicalchannelparameters.m_mediaControlChannel.GetTag()==H245_TransportAddress::e_unicastAddress){
					H245_UnicastAddress & h245_unicastaddress=h245_h2250logicalchannelparameters.m_mediaControlChannel;
					if(setOpenLogicalChannelMediaControlChannel(&h245_unicastaddress,h245_openlogicalchannel.m_forwardLogicalChannelNumber.GetValue(),direction)<0){
						sendOpenLogicalChannelReject(direction,h245_openlogicalchannel.m_forwardLogicalChannelNumber.GetValue());
						return(0);
					}
				}
				else {
					WRN((__FILE__,__FUNCTION__,"H323:H245:REQUEST:OPENLOGICALCHANNEL: H245_TransportAddress is not a unicast address\n"))
					sendOpenLogicalChannelReject(direction,h245_openlogicalchannel.m_forwardLogicalChannelNumber.GetValue());
					return(0);
				}
			}
		}
		/* reverseLogicalChannelParameters */
		if(h245_openlogicalchannel.HasOptionalField(H245_OpenLogicalChannel::e_reverseLogicalChannelParameters)){
			WRN((__FILE__,__FUNCTION__,"H323:H245:REQUEST:OPENLOGICALCHANNEL: OpenLogicalChannel includes reverseLogicalChannelParameters\n"))
			sendOpenLogicalChannelReject(direction,h245_openlogicalchannel.m_forwardLogicalChannelNumber.GetValue());
			return(0);
		}
	}
	else if(h245_requestmessage->GetTag()==H245_RequestMessage::e_closeLogicalChannel){
		LOG((__FILE__,__FUNCTION__,8,"H323:H245:REQUEST:CLOSELOGICALCHANNEL:\n"))
		H245_CloseLogicalChannel & h245_closelogicalchannel=(*h245_requestmessage);
		setCloseLogicalChannel(h245_closelogicalchannel.m_forwardLogicalChannelNumber.GetValue(),direction);
	}
	else if(h245_requestmessage->GetTag()==H245_RequestMessage::e_requestChannelClose){
               	LOG((__FILE__,__FUNCTION__,8,"H323:H245:REQUEST:REQUESTCHANNELCLOSE:\n"))
	}
	else if(h245_requestmessage->GetTag()==H245_RequestMessage::e_multiplexEntrySend){
             	LOG((__FILE__,__FUNCTION__,8,"H323:H245:REQUEST:MULTIPLEXENTRYSEND:\n"))
	}
	else if(h245_requestmessage->GetTag()==H245_RequestMessage::e_requestMultiplexEntry){
               	LOG((__FILE__,__FUNCTION__,8,"H323:H245:REQUEST:REQUESTMULTIPLEXENTRY:\n"))
	}
	else if(h245_requestmessage->GetTag()==H245_RequestMessage::e_requestMode){
               	LOG((__FILE__,__FUNCTION__,8,"H323:H245:REQUEST:REQUESTMODE:\n"))
	}
	else if(h245_requestmessage->GetTag()==H245_RequestMessage::e_roundTripDelayRequest){
               	LOG((__FILE__,__FUNCTION__,8,"H323:H245:REQUEST:ROUNDTRIPDELAYREQUEST:\n"))
	}
	else if(h245_requestmessage->GetTag()==H245_RequestMessage::e_maintenanceLoopRequest){
               	LOG((__FILE__,__FUNCTION__,8,"H323:H245:REQUEST:MAINTENANCELOOPREQUEST:\n"))
	}
	else if(h245_requestmessage->GetTag()==H245_RequestMessage::e_communicationModeRequest){
               	LOG((__FILE__,__FUNCTION__,8,"H323:H245:REQUEST:COMMUNICATIONMODEREQUEST:\n"))
	}
	else if(h245_requestmessage->GetTag()==H245_RequestMessage::e_conferenceRequest){
               	LOG((__FILE__,__FUNCTION__,8,"H323:H245:REQUEST:CONFERENCEREQUEST:\n"))
	}
	else if(h245_requestmessage->GetTag()==H245_RequestMessage::e_multilinkRequest){
		LOG((__FILE__,__FUNCTION__,8,"H323:H245:REQUEST:MULTILINKREQUEST:\n"))
	}
	else if(h245_requestmessage->GetTag()==H245_RequestMessage::e_logicalChannelRateRequest){
		LOG((__FILE__,__FUNCTION__,8,"H323:H245:REQUEST:LOGICALCHANNELRATEREQUEST:\n"))
	}
	else {
		WRN((__FILE__,__FUNCTION__,"H323:H245:REQUEST:UNKNOWN: PDU RECEIVED\n"))
		return(0);
	}
	return(1);
}


int H323Parser::handleH245Response(H245_ResponseMessage *h245_responsemessage,bool direction){
	if(h245_responsemessage->GetTag()==H245_ResponseMessage::e_nonStandard){
               	LOG((__FILE__,__FUNCTION__,8,"H323:H245:RESPONSE:NONSTANDARD:\n"))
	}
	else if(h245_responsemessage->GetTag()==H245_ResponseMessage::e_masterSlaveDeterminationAck){
               	LOG((__FILE__,__FUNCTION__,8,"H323:H245:RESPONSE:MASTERSLAVEDETERMINATIONACK:\n"))
	}
	else if(h245_responsemessage->GetTag()==H245_ResponseMessage::e_masterSlaveDeterminationReject){
               	LOG((__FILE__,__FUNCTION__,8,"H323:H245:RESPONSE:MASTERSLAVEDETERMINATIONREJECT:\n"))
	}
	else if(h245_responsemessage->GetTag()==H245_ResponseMessage::e_terminalCapabilitySetAck){
               	LOG((__FILE__,__FUNCTION__,8,"H323:H245:RESPONSE:TERMINALCAPABILITYSETACK:\n"))
	}
	else if(h245_responsemessage->GetTag()==H245_ResponseMessage::e_terminalCapabilitySetReject){
               	LOG((__FILE__,__FUNCTION__,8,"H323:H245:RESPONSE:TERMINALCAPABILITYSETREJECT:\n"))
	}
	else if(h245_responsemessage->GetTag()==H245_ResponseMessage::e_openLogicalChannelAck){
               	LOG((__FILE__,__FUNCTION__,8,"H323:H245:RESPONSE:OPENLOGICALCHANNELACK:\n"))
		H245_OpenLogicalChannelAck & h245_openlogicalchannelack=(*h245_responsemessage);
		LOG((__FILE__,__FUNCTION__,8,"H323:H245:RESPONSE:OPENLOGICALCHANNELACK: forwardLogicalChannelNumber = %d\n",h245_openlogicalchannelack.m_forwardLogicalChannelNumber.GetValue()))
		if(h245_openlogicalchannelack.HasOptionalField(H245_OpenLogicalChannelAck::e_forwardMultiplexAckParameters)){
			if(h245_openlogicalchannelack.m_forwardMultiplexAckParameters.GetTag()==H245_OpenLogicalChannelAck_forwardMultiplexAckParameters::e_h2250LogicalChannelAckParameters){
				H245_H2250LogicalChannelAckParameters & h245_h2250logicalchannelackparameters=(h245_openlogicalchannelack.m_forwardMultiplexAckParameters);
                               	if (h245_h2250logicalchannelackparameters.HasOptionalField(H245_H2250LogicalChannelAckParameters::e_mediaChannel)){
                                       	if(h245_h2250logicalchannelackparameters.m_mediaChannel.GetTag()==H245_TransportAddress::e_unicastAddress){
                                               	H245_UnicastAddress & h245_unicastaddress=h245_h2250logicalchannelackparameters.m_mediaChannel;
						if(setOpenLogicalChannelAckMediaChannel(&h245_unicastaddress,h245_openlogicalchannelack.m_forwardLogicalChannelNumber.GetValue(),!direction)<0) return(-1);
                                       	}
					else {
						WRN((__FILE__,__FUNCTION__,"H323:H245:RESPONSE:OPENLOGICALCHANNELACK: H245_TransportAddress is not a unicast address\n"))
						return(-1);
					}
                                }
                                if (h245_h2250logicalchannelackparameters.HasOptionalField(H245_H2250LogicalChannelAckParameters::e_mediaControlChannel)){
                                       	if(h245_h2250logicalchannelackparameters.m_mediaChannel.GetTag()==H245_TransportAddress::e_unicastAddress){
                                               	H245_UnicastAddress & h245_unicastaddress=h245_h2250logicalchannelackparameters.m_mediaControlChannel;
						if(setOpenLogicalChannelAckMediaControlChannel(&h245_unicastaddress,h245_openlogicalchannelack.m_forwardLogicalChannelNumber.GetValue(),!direction)<0) return(-1); 
                                       	}
					else {
						WRN((__FILE__,__FUNCTION__,"H323:H245:RESPONSE:OPENLOGICALCHANNELACK: H245_TransportAddress is not a unicast address\n"))
						return(-1);
					}
                                }
			}
		}
		if(h245_openlogicalchannelack.HasOptionalField(H245_OpenLogicalChannel::e_reverseLogicalChannelParameters)){
			WRN((__FILE__,__FUNCTION__,"H323:H245:RESPONSE:OPENLOGICALCHANNELACK: OpenLogicalChannelAck includes reverseLogicalChannelParameters\n"))
		}
	}
	else if(h245_responsemessage->GetTag()==H245_ResponseMessage::e_openLogicalChannelReject){
               	LOG((__FILE__,__FUNCTION__,8,"H323:H245:RESPONSE:OPENLOGICALCHANNELREJECT:\n"))
	}
	else if(h245_responsemessage->GetTag()==H245_ResponseMessage::e_closeLogicalChannelAck){
               	LOG((__FILE__,__FUNCTION__,8,"H323:H245:RESPONSE:CLOSELOGICALCHANNELACK:\n"))
	}
	else if(h245_responsemessage->GetTag()==H245_ResponseMessage::e_requestChannelCloseAck){
               	LOG((__FILE__,__FUNCTION__,8,"H323:H245:RESPONSE:REQUESTCHANNELCLOSEACK:\n"))
	}
	else if(h245_responsemessage->GetTag()==H245_ResponseMessage::e_requestChannelCloseReject){
               	LOG((__FILE__,__FUNCTION__,8,"H323:H245:RESPONSE:REQUESTCHANNELCLOSEREJECT:\n"))
	}
	else if(h245_responsemessage->GetTag()==H245_ResponseMessage::e_multiplexEntrySendAck){
               	LOG((__FILE__,__FUNCTION__,8,"H323:H245:RESPONSE:MULTIPLEXENTRYSENDACK:\n"))
	}
	else if(h245_responsemessage->GetTag()==H245_ResponseMessage::e_multiplexEntrySendReject){
               	LOG((__FILE__,__FUNCTION__,8,"H323:H245:RESPONSE:MULTIPLEXENTRYSENDREJECT:\n"))
	}
	else if(h245_responsemessage->GetTag()==H245_ResponseMessage::e_requestMultiplexEntryAck){
               	LOG((__FILE__,__FUNCTION__,8,"H323:H245:RESPONSE:REQUESTMULTIPLEXENTRYACK:\n"))
	}
	else if(h245_responsemessage->GetTag()==H245_ResponseMessage::e_requestMultiplexEntryReject){
               	LOG((__FILE__,__FUNCTION__,8,"H323:H245:RESPONSE:REQUESTMULTIPLEXENTRYREJECT:\n"))
	}
	else if(h245_responsemessage->GetTag()==H245_ResponseMessage::e_requestModeAck){
               	LOG((__FILE__,__FUNCTION__,8,"H323:H245:RESPONSE:REQUESTMODEACK:\n"))
	}
	else if(h245_responsemessage->GetTag()==H245_ResponseMessage::e_requestModeReject){
               	LOG((__FILE__,__FUNCTION__,8,"H323:H245:RESPONSE:REQUESTMODEREJECT:\n"))
	}
	else if(h245_responsemessage->GetTag()==H245_ResponseMessage::e_roundTripDelayResponse){
               	LOG((__FILE__,__FUNCTION__,8,"H323:H245:RESPONSE:ROUNDTRIPDELAYRESPONSE:\n"))
	}
	else if(h245_responsemessage->GetTag()==H245_ResponseMessage::e_maintenanceLoopAck){
               	LOG((__FILE__,__FUNCTION__,8,"H323:H245:RESPONSE:MAINTENANCELOOPACK:\n"))
	}
	else if(h245_responsemessage->GetTag()==H245_ResponseMessage::e_maintenanceLoopReject){
               	LOG((__FILE__,__FUNCTION__,8,"H323:H245:RESPONSE:MAINTENANCELOOPREJECT:\n"))
	}
	else if(h245_responsemessage->GetTag()==H245_ResponseMessage::e_communicationModeResponse){
               	LOG((__FILE__,__FUNCTION__,8,"H323:H245:RESPONSE:COMMUNICATIONMODERESPONSE:\n"))
	}
	else if(h245_responsemessage->GetTag()==H245_ResponseMessage::e_conferenceResponse){
               	LOG((__FILE__,__FUNCTION__,8,"H323:H245:RESPONSE:CONFERENCERESPONSE:\n"))
	}
	else if(h245_responsemessage->GetTag()==H245_ResponseMessage::e_multilinkResponse){
               	LOG((__FILE__,__FUNCTION__,8,"H323:H245:RESPONSE:MULTILINKRESPONSE:\n"))
	}
	else if(h245_responsemessage->GetTag()==H245_ResponseMessage::e_logicalChannelRateAcknowledge){
               	LOG((__FILE__,__FUNCTION__,8,"H323:H245:RESPONSE:LOGICALCHANNELRATEACKNOWLEDGE:\n"))
	}
	else if(h245_responsemessage->GetTag()==H245_ResponseMessage::e_logicalChannelRateReject){
               	LOG((__FILE__,__FUNCTION__,8,"H323:H245:RESPONSE:LOGICALCHANNELRATEREJECT:\n"))
	}
	else {
		WRN((__FILE__,__FUNCTION__,"H323:H245:RESPONSE:UNKNOWN: PDU RECEIVED\n"))
		return(0);
	}
	return(1);
}



int H323Parser::handleH245(H245_MultimediaSystemControlMessage *mscMsg,bool direction){
	int ret;
#ifdef _H323DEBUG_
        printf("\n===BEGIN H245 PDU DECODED ==============================\n");
	mscMsg.PrintOn(cout);
        printf("\n===END H245 PDU DECODED ================================\n");
#endif
        if (mscMsg->GetTag()==H245_MultimediaSystemControlMessage::e_request) {
                LOG((__FILE__,__FUNCTION__,8,"H323:H245:REQUEST: PDU RECEIVED\n"))
		if (!checkPDU("H245_request")) return(0);
		H245_RequestMessage & h245_requestmessage=(*mscMsg);
		if((ret=handleH245Request(&h245_requestmessage,direction))<=0) {
			WRN((__FILE__,__FUNCTION__,"H323:H245:REQUEST: handling h245_requestmessage\n"))
			return(ret);
		}
        }
        else if (mscMsg->GetTag()==H245_MultimediaSystemControlMessage::e_response) {
                LOG((__FILE__,__FUNCTION__,8,"H323:H245:RESPONSE: PDU RECEIVED\n"))
		if (!checkPDU("H245_response")) return(0);
		H245_ResponseMessage & h245_responsemessage=(*mscMsg);
		if((ret=handleH245Response(&h245_responsemessage,direction))<=0) {
			WRN((__FILE__,__FUNCTION__,"H323:H245:RESPONSE: handling h245_requestmessage\n"))
			return(ret);
		}
	}
	else if (mscMsg->GetTag()==H245_MultimediaSystemControlMessage::e_command) {
                LOG((__FILE__,__FUNCTION__,8,"H323:H245:COMMAND: PDU RECEIVED\n"))
		if (!checkPDU("H245_command")) return(0);
        }
        else if (mscMsg->GetTag()==H245_MultimediaSystemControlMessage::e_indication) {
                LOG((__FILE__,__FUNCTION__,8,"H323:H245:INDICATION: PDU RECEIVED\n"))
		if (!checkPDU("H245_indication")) return(0);
        }
	else {
		WRN((__FILE__,__FUNCTION__,"H323:H245:UNKNOWN: PDU RECEIVED\n"))
		return(0);
	}
#ifdef _H323DEBUG_
        printf("\n===BEGIN H245 PDU MODIFIED =============================\n");
        mscMsg.PrintOn(cout);
        printf("\n===END H245 PDU MODIFIED ===============================\n");
#endif
	return(1);
}

int H323Parser::parse_00(Connection *c){
        Q931              		q931pdu;
	PBYTEArray        		q931arrayIN;
        PPER_Stream                     q931streamIN;
        PPER_Stream                     q931streamOUT;
        H225_H323_UserInformation       uuInfo;
        PPER_Stream                     uuInfostreamIN;
        PPER_Stream                     uuInfostreamOUT;
	PString				calledPartyNumber;
	String				displayName;
	int				ret;

	LOG((__FILE__,__FUNCTION__,15,"H323:Q931: parse_00\n"))

        /*read and decode/modify Q931*/
        for(int i=0;i<(c->packet_len-(TPKT_LENGTH_OFFSET+2));i++){
                q931arrayIN[i]=(unsigned char) c->buffer[i+TPKT_LENGTH_OFFSET+2];
        }

        q931streamIN = PPER_Stream(q931arrayIN);
        q931pdu.Decode(q931streamIN);

#ifdef _H323DEBUG_
        printf("\n===BEGIN Q931 PDU DECODED ==============================\n");
        q931pdu.PrintOn(cout);
        printf("\n===END Q931 PDU DECODED ================================\n");
#endif
	parent->data.srcUser=q931pdu.GetDisplayName();
	displayName = displayName + "(" + parent->data.srcUser + ")@KOMproxy";
	q931pdu.SetDisplayName(PString(displayName.c_str()));
	if (q931pdu.GetCalledPartyNumber(calledPartyNumber,(unsigned int*) NULL,(unsigned int*) NULL)){
		connectDestinationCalledPartyNumberPDU(String(calledPartyNumber));
	}
	if(call.id<0) {
		call.id=q931pdu.GetCallReference();
		LOG((__FILE__,__FUNCTION__,8,"H323:Q931: callReference = %d\n",call.id))
	}
        if(q931pdu.HasIE(Q931::UserUserIE)) {
                uuInfostreamIN = q931pdu.GetIE(Q931::UserUserIE);
                uuInfo.Decode(uuInfostreamIN);
		if((ret=handleQ931UserUserIE(&uuInfo,0))<=0) {
			WRN((__FILE__,__FUNCTION__,"H323:Q931: handling UserUserIE\n"))
			return(ret);
		}
                uuInfo.Encode(uuInfostreamOUT);
		uuInfostreamOUT.CompleteEncoding();
                q931pdu.SetIE(Q931::UserUserIE, uuInfostreamOUT);
        }
	else {
		WRN((__FILE__,__FUNCTION__,"H323:Q931:UNKNOWN: PDU RECEIVED\n"))
		return(0);
	}
#ifdef _H323DEBUG_
        printf("\n===BEGIN Q931 PDU MODIFIED =============================\n");
        q931pdu.PrintOn(cout);
        printf("\n===END Q931 PDU MODIFIED ===============================\n");
#endif
        q931pdu.Encode(q931streamOUT);

        c->packet_len=q931streamOUT.GetSize()+TPKT_LENGTH_OFFSET+2;
        c->buffer[TPKT_LENGTH_OFFSET]=  (unsigned char) (((c->packet_len) >> 8) & 0xFFFF) ;
        c->buffer[TPKT_LENGTH_OFFSET+1]=(unsigned char) (((c->packet_len)     ) & 0x00FF) ;
        for(u_int32_t i=0;i<q931streamOUT.GetSize();i++){
                c->buffer[TPKT_LENGTH_OFFSET+2+i]=q931streamOUT[i];
        }

	if(((*(call.Q931_out))->connectionGetState())!=CONNECT){
                        WRN((__FILE__,__FUNCTION__,"H323:Q931: the call destination was not connected!\n"))
			return(-1);
	}
	return(1);
}


int H323Parser::parse_10(Connection *c){
        H245_MultimediaSystemControlMessage        mscMsg;
        PBYTEArray                                 h245arrayIN;
        PPER_Stream                                h245streamIN;
        PPER_Stream                                h245streamOUT;
	int ret;

	LOG((__FILE__,__FUNCTION__,15,"H323:H245: parse_10\n"))

        /*read and decode/modify H245*/
        for(int i=0;i<(c->packet_len-(TPKT_LENGTH_OFFSET+2));i++){
                h245arrayIN[i]=(unsigned char) c->buffer[i+TPKT_LENGTH_OFFSET+2];
        }

        h245streamIN = PPER_Stream(h245arrayIN);
        mscMsg.Decode(h245streamIN);
	if((ret=handleH245(&mscMsg,0))<=0) {
		WRN((__FILE__,__FUNCTION__,"H323:H245: handling H245\n"))
		return(ret);
	}
        mscMsg.Encode(h245streamOUT);
	h245streamOUT.CompleteEncoding();

        c->packet_len=h245streamOUT.GetSize()+TPKT_LENGTH_OFFSET+2;

        c->buffer[TPKT_LENGTH_OFFSET]=  (unsigned char) (((c->packet_len) >> 8) & 0xFFFF) ;
        c->buffer[TPKT_LENGTH_OFFSET+1]=(unsigned char) (((c->packet_len)     ) & 0x00FF) ;
        for(u_int32_t i=0;i<h245streamOUT.GetSize();i++){
                c->buffer[TPKT_LENGTH_OFFSET+2+i]=h245streamOUT[i];
        }
	return(1);
}
int H323Parser::parse_20(Connection *c){
	LOG((__FILE__,__FUNCTION__,15,"H323:RTP: parse_20\n"))
	return(1);
}
int H323Parser::parse_30(Connection *c){
	LOG((__FILE__,__FUNCTION__,15,"H323:RTCP: parse_30\n"))
	return(1);
}
int H323Parser::parse_default0(Connection *c){
	LOG((__FILE__,__FUNCTION__,15,"H323: parse_default0\n"))
	return(1);
}

int H323Parser::parse_01(Connection *c){
        Q931              		q931pdu;
	PBYTEArray        		q931arrayIN;
        PPER_Stream                     q931streamIN;
        PPER_Stream                     q931streamOUT;

        H225_H323_UserInformation       uuInfo;
        PPER_Stream                     uuInfostreamIN;
        PPER_Stream                     uuInfostreamOUT;
	String				displayName;
	int				ret;

	LOG((__FILE__,__FUNCTION__,15,"H323:Q931: parse_01\n"))


        /*read and decode/modify Q931*/
        for(int i=0;i<(c->packet_len-(TPKT_LENGTH_OFFSET+2));i++){
                q931arrayIN[i]=(unsigned char) c->buffer[i+TPKT_LENGTH_OFFSET+2];
        }

        q931streamIN = PPER_Stream(q931arrayIN);
        q931pdu.Decode(q931streamIN);

#ifdef _H323DEBUG_
        printf("\n===BEGIN Q931 PDU DECODED ==============================\n");
        q931pdu.PrintOn(cout);
        printf("\n===END Q931 PDU DECODED ================================\n");
#endif
	parent->data.dstUser=q931pdu.GetDisplayName();
	displayName=displayName + "(" + parent->data.dstUser + ")@KOMproxy";
	q931pdu.SetDisplayName(PString(displayName.c_str()));
        if(q931pdu.HasIE(Q931::UserUserIE)) {
                uuInfostreamIN = q931pdu.GetIE(Q931::UserUserIE);
                uuInfo.Decode(uuInfostreamIN);
		if((ret=handleQ931UserUserIE(&uuInfo,0))<=0) {
			WRN((__FILE__,__FUNCTION__,"H323:Q931: handling UserUserIE\n"))
			return(ret);
		}
                uuInfo.Encode(uuInfostreamOUT);
		uuInfostreamOUT.CompleteEncoding();
                q931pdu.SetIE(Q931::UserUserIE, uuInfostreamOUT);
        }
	else {
		WRN((__FILE__,__FUNCTION__,"H323:Q931:UNKNOWN: PDU RECEIVED\n"))
		return(0);
	}
#ifdef _H323DEBUG_
        printf("\n===BEGIN Q931 PDU MODIFIED =============================\n");
        q931pdu.PrintOn(cout);
        printf("\n===END Q931 PDU MODIFIED ===============================\n");
#endif
        q931pdu.Encode(q931streamOUT);

        c->packet_len=q931streamOUT.GetSize()+TPKT_LENGTH_OFFSET+2;

        c->buffer[TPKT_LENGTH_OFFSET]=  (unsigned char) (((c->packet_len) >> 8) & 0xFFFF) ;
        c->buffer[TPKT_LENGTH_OFFSET+1]=(unsigned char) (((c->packet_len)     ) & 0x00FF) ;
        for(u_int32_t i=0;i<q931streamOUT.GetSize();i++){
                c->buffer[TPKT_LENGTH_OFFSET+2+i]=q931streamOUT[i];
        }
	return(1);
}

int H323Parser::parse_11(Connection *c){
        H245_MultimediaSystemControlMessage        mscMsg;
        PBYTEArray                                 h245arrayIN;
        PPER_Stream                                h245streamIN;
        PPER_Stream                                h245streamOUT;
	int ret;

	LOG((__FILE__,__FUNCTION__,15,"H323:H245: parse_11\n"))

        /*read and decode/modify H245*/
        for(int i=0;i<(c->packet_len-(TPKT_LENGTH_OFFSET+2));i++){
                h245arrayIN[i]=(unsigned char) c->buffer[i+TPKT_LENGTH_OFFSET+2];
        }

        h245streamIN = PPER_Stream(h245arrayIN);
        mscMsg.Decode(h245streamIN);
	if((ret=handleH245(&mscMsg,1))<=0) {
		WRN((__FILE__,__FUNCTION__,"H323:H245: handling H245\n"))
		return(ret);
	}
        mscMsg.Encode(h245streamOUT);
	h245streamOUT.CompleteEncoding();

        c->packet_len=h245streamOUT.GetSize()+TPKT_LENGTH_OFFSET+2;

        c->buffer[TPKT_LENGTH_OFFSET]=  (unsigned char) (((c->packet_len) >> 8) & 0xFFFF) ;
        c->buffer[TPKT_LENGTH_OFFSET+1]=(unsigned char) (((c->packet_len)     ) & 0x00FF) ;
        for(u_int32_t i=0;i<h245streamOUT.GetSize();i++){
                c->buffer[TPKT_LENGTH_OFFSET+2+i]=h245streamOUT[i];
        }
	return(1);
}
int H323Parser::parse_21(Connection *c){
	LOG((__FILE__,__FUNCTION__,15,"H323:RTP: parse_21\n"))
	return(1);
}
int H323Parser::parse_31(Connection *c){
	LOG((__FILE__,__FUNCTION__,15,"H323:RTCP: parse_31\n"))
	return(1);
}
int H323Parser::parse_default1(Connection *c){
	LOG((__FILE__,__FUNCTION__,15,"H323: parse_default1\n"))
	return(1);
}

