/* Copyright (C) 2000 KOM/Darmstadt University of Technology
 *
 * You are allowed to use all other parts of the code under the following terms:
 *
 * For non-commercial use, code may be used in unmodified form provided
 * that this copyright notice and this permission notice appear in
 * supporting documentation.
 *
 * This software is provided "as is" and without any express or implied
 * warranties, including, without limitation, the implied warranty of
 * fitness for a particular purpose.
 *
 * The code may be subjected to the GNU General Public License, Version 2,
 * and re-distributed under the terms of this license.
 *
 * Commercial use other than under the terms of the GNU General Public
 * License is allowed only after express negotiation of conditions
 * with the authors.
 */
#ifndef _H323Parser_h_
#define _H323Parser_h_ 1

#include "../Global.h"
#include "../Parser.h"
#include "../Connection.h"
#include "../TimeValue.h"
#include "../String.h"
#include "../Session.h"
#include "../Conf.h"
#include "../Log.h"
#include "H323Call.h"
#include "H323MediaChannel.h"

#include <ptlib.h>
#include <q931.h>
#include <h225.h>
#include <h245.h>

/*#define _H323DEBUG_ 1*/

#define TPKT_LENGTH_OFFSET      2

class H323Parser : public Parser {
	Fwhndl  	fwhndl;
	H323Call	call;

	int setSetupDestCallSignalAddress(H225_TransportAddress* sourceCallSignalAddress);
	int setSetupSourceCallSignalAddress(H225_TransportAddress* sourceCallSignalAddress);
	int setSetupH245Address(H225_TransportAddress* h245Address);
	int setConnectH245Address(H225_TransportAddress* h245Address);
	int setCallProceedingH245Address(H225_TransportAddress* h245Address);
	int setAlertingH245Address(H225_TransportAddress* h245Address);
	int setFacilityH245Address(H225_TransportAddress* h245Address);
	int setProgressH245Address(H225_TransportAddress* h245Address);
	int setH245Address(H225_TransportAddress* h245Address,String logmessage);

	int setCloseLogicalChannel(int id_a,int id_b);
	int setOpenLogicalChannelAckMediaControlChannel(H245_UnicastAddress* RTPAddress,int id_a,int id_b);
	int setOpenLogicalChannelAckMediaChannel(H245_UnicastAddress* RTPAddress,int id_a,int id_b);
	int setOpenLogicalChannelMediaControlChannel(H245_UnicastAddress* RTPAddress,int id_a,int id_b);
	int setOpenLogicalChannelMediaChannel(H245_UnicastAddress* RTPAddress,int id_a,int id_b);


	int connectDestinationDestinationAddressPDU(H225_ArrayOf_AliasAddress* destinationAddress);
	int connectDestinationCalledPartyNumberPDU(String destination);


	int handleQ931UserUserIE(H225_H323_UserInformation *uuInfo,bool direction);
	int handleH245Request(H245_RequestMessage *h245_requestmessage,bool direction);
	int handleH245Response(H245_ResponseMessage *h245_responsemessage,bool direction);
	int handleH245(H245_MultimediaSystemControlMessage *mscMsg,bool direction);


	bool checkPDU(String pdutype);
	bool checkOpenLogicalChannel(H245_OpenLogicalChannel *h245_openlogicalchannel);
	int sendOpenLogicalChannelReject(bool direction,int num);
	int sendReleaseComplete(bool direction);
public:
	H323Parser(Session* parent_t, String type_t);
	~H323Parser();
	
        int connectDestinationSPECIAL(String locator);
        int connectDestinationFAST();

	int read_0(Connection *c);
	int read_1(Connection *c);
	int read_2(Connection *c);
	int read_3(Connection *c);
	int read_default(Connection *c);

	int write_0(Connection *dc, Connection *sc);
	int write_1(Connection *dc, Connection *sc);
	int write_2(Connection *dc, Connection *sc);
	int write_3(Connection *dc, Connection *sc);
	int write_default(Connection *dc, Connection *sc);
	
	int parse_00(Connection *c);
	int parse_10(Connection *c);
	int parse_20(Connection *c);
	int parse_30(Connection *c);
	int parse_default0(Connection *c);
	int parse_01(Connection *c);
	int parse_11(Connection *c);
	int parse_21(Connection *c);
	int parse_31(Connection *c);
	int parse_default1(Connection *c);

};

#ifndef STATIC_PROXY
extern "C" {
Parser *maker(Session* parent_t,String type_t){
   return new H323Parser(parent_t,type_t);
}
}
#endif


#endif
