/* Copyright (C) 2000 KOM/Darmstadt University of Technology
 *
 * You are allowed to use all other parts of the code under the following terms:
 *
 * For non-commercial use, code may be used in unmodified form provided
 * that this copyright notice and this permission notice appear in
 * supporting documentation.
 *
 * This software is provided "as is" and without any express or implied
 * warranties, including, without limitation, the implied warranty of
 * fitness for a particular purpose.
 *
 * The code may be subjected to the GNU General Public License, Version 2,
 * and re-distributed under the terms of this license.
 *
 * Commercial use other than under the terms of the GNU General Public
 * License is allowed only after express negotiation of conditions
 * with the authors.
 */





#include "RASParser.h"
#include "../Log.h"
#include "../Conf.h"
#include "../Session.h"
#include "../Fwhndl.h"

RASParser::RASParser(Session* parent_t,String type_t){
	LOG((__FILE__,__FUNCTION__,15,"RAS: creating parser\n"))
	type	= type_t;
	parent	= parent_t;
	lrqring=0;
        for(int i=0;i<MAXLRQ;++i){
		lrqRequestSeqNum[i]=-1;
        }
	parent->con[0][0]->l_ip        = Global::getGlobal()->getConf()->getValAsIP("LOCALIP");
	parent->con[0][0]->l_mask      = 0xffffffff;
	parent->con[0][0]->l_port      = Global::getGlobal()->getConf()->getValAsInt(type+"_PORT");
	parent->con[0][0]->ipproto     = 17; 
	parent->con[0][0]->p_ip        = 0;
	parent->con[0][0]->p_mask      = 0;
	parent->con[0][0]->p_port      = 0; 
	parent->con[0][0]->direction   = 1;
	parent->con[0][0]->hardUDP     = false;
	if((parent->con[0][0]->connectionConnect())<0){
		WRN((__FILE__,__FUNCTION__,"RAS: connecting source\n"))
	}
	//parent->con[0][1]=parent->con[0][0];
}

RASParser::~RASParser() {
	LOG((__FILE__,__FUNCTION__,15,"RAS: deleting parser\n"))
}

int RASParser::connectDestinationReplyAddressPDU(H225_TransportAddress* replyAddress){
	if((parent->con[0][0]->p_ip==0) || (parent->con[0][0]->p_port==0) ){
        	LOG((__FILE__,__FUNCTION__,8,"RAS:LOCATOR:PDU:\n"))
        	if(replyAddress->GetTag()==H225_TransportAddress::e_ipAddress){
                	H225_TransportAddress_ipAddress & h225_transportaddress_ipaddress=(*replyAddress);
                	LOG((__FILE__,__FUNCTION__,8,"RAS:LOCATOR:PDU: using destination %d.%d.%d.%d:%d\n",
                        	h225_transportaddress_ipaddress.m_ip[0],
                        	h225_transportaddress_ipaddress.m_ip[1],
                        	h225_transportaddress_ipaddress.m_ip[2],
                        	h225_transportaddress_ipaddress.m_ip[3],
                        	h225_transportaddress_ipaddress.m_port.GetValue()))

			parent->con[0][0]->p_ip       = ((h225_transportaddress_ipaddress.m_ip[0])) |
                        	                        ((h225_transportaddress_ipaddress.m_ip[1]) << 8) |
                                	                ((h225_transportaddress_ipaddress.m_ip[2]) << 16) |
                                        	        ((h225_transportaddress_ipaddress.m_ip[3]) << 24);
			parent->con[0][0]->p_mask     = 0xffffffff;
			parent->con[0][0]->p_port     = (h225_transportaddress_ipaddress.m_port.GetValue());
                        return(1);

        	}
        	else {
                	WRN((__FILE__,__FUNCTION__,"RAS:LOCATOR:PDU: H225_TransportAddress is not an ip address\n"))
                	return(-1);
        	}
        return(-1);
        }
        else {
           return(1);
        }
}
int RASParser::connectDestinationDestinationAddressPDU(H225_ArrayOf_AliasAddress* destinationAddress){
        String locator;
        String prefix;
        String requestIP;
        String requestPORT;
        String destination;

	if((parent->con[0][0]->p_ip==0) || (parent->con[0][0]->p_port==0) ){
           LOG((__FILE__,__FUNCTION__,8,"RAS:LOCATOR:PDU:\n"))
           for(int j=0;j<(destinationAddress->GetSize());++j){
                if((*destinationAddress)[j].GetTag()==H225_AliasAddress::e_dialedDigits){
                        PASN_IA5String & dialedDigits = (*destinationAddress)[j];
                        destination = String((const char *)(dialedDigits.GetValue()));
                        LOG((__FILE__,__FUNCTION__,8,"RAS:LOCATOR:PDU: destinationAddress contains dialedDigits = %s\n",destination.c_str()))
                        for (int i=0;i<MAXLOCATORS;++i){
                                locator = type + "_LOCATOR" + "_PDU" + "_DIGIT" + "_" + i;
                                prefix = Global::getGlobal()->getConf()->getValAsString(locator);
                                if (prefix!=String("")) {
                                        LOG((__FILE__,__FUNCTION__,8,"RAS:LOCATOR:PDU: testing %s = %s for %s\n",locator.c_str(),prefix.c_str(),destination.c_str()))
                                        if(destination.leftequal(prefix)){
                                                requestIP   = locator + "_IP";
                                                requestPORT = locator + "_PORT";
                                                if (((Global::getGlobal()->getConf()->getValAsString(requestIP))!=String("")) &&
                                                        ((Global::getGlobal()->getConf()->getValAsString(requestPORT))!=String(""))) {
                                                        parent->con[0][0]->p_ip        = Global::getGlobal()->getConf()->getValAsIP(requestIP);
                                                        parent->con[0][0]->p_mask      = 0xffffffff;
                                                        parent->con[0][0]->p_port      = Global::getGlobal()->getConf()->getValAsInt(requestPORT);
                                                        LOG((__FILE__,__FUNCTION__,8,"RAS:LOCATOR:PDU: using destination %s:%d for %s\n",
                                                                (String::IPtoString(parent->con[0][0]->p_ip)).c_str(),
                                                                parent->con[0][0]->p_port,destination.c_str()))
                                                        return(1);
                                                }
                                                else {
                                                        WRN((__FILE__,__FUNCTION__,"RAS:LOCATOR:PDU: configuration file incomplete: KEY=%s VAL=%s KEY=%s VAL=%s\n",
                                                                requestIP.c_str()  ,Global::getGlobal()->getConf()->getValAsString(requestIP).c_str(),
                                                                requestPORT.c_str(),Global::getGlobal()->getConf()->getValAsString(requestPORT).c_str()))
                                                }
                                        }
                                }
                        }
			/* use default routing entry if present */
                        locator = type + "_LOCATOR" + "_PDU" + "_DIGIT" + "_DEFAULT";
			LOG((__FILE__,__FUNCTION__,8,"RAS:LOCATOR:PDU: using %s for %s\n",locator.c_str(),destination.c_str()))
                        requestIP   = locator + "_IP";
                        requestPORT = locator + "_PORT";
			if (((Global::getGlobal()->getConf()->getValAsString(requestIP))!=String("")) &&
                            ((Global::getGlobal()->getConf()->getValAsString(requestPORT))!=String(""))) {
                        	parent->con[0][0]->p_ip        = Global::getGlobal()->getConf()->getValAsIP(requestIP);
                        	parent->con[0][0]->p_mask      = 0xffffffff;
                        	parent->con[0][0]->p_port      = Global::getGlobal()->getConf()->getValAsInt(requestPORT);
                        	LOG((__FILE__,__FUNCTION__,8,"RAS:LOCATOR:PDU: using destination %s:%d for %s\n",
                        	   (String::IPtoString(parent->con[0][0]->p_ip)).c_str(),
                        	   parent->con[0][0]->p_port,destination.c_str()))
                        	return(1);
			}
                        else {
                        	WRN((__FILE__,__FUNCTION__,"RAS:LOCATOR:PDU: default routing entry not found in configuration file: KEY=%s VAL=%s KEY=%s VAL=%s\n",
                                            requestIP.c_str()  ,Global::getGlobal()->getConf()->getValAsString(requestIP).c_str(),
                                            requestPORT.c_str(),Global::getGlobal()->getConf()->getValAsString(requestPORT).c_str()))
			}
                }
                if((*destinationAddress)[j].GetTag()==H225_AliasAddress::e_h323_ID){   
                        LOG((__FILE__,__FUNCTION__,8,"RAS:LOCATOR:PDU: destinationAddress contains h323_ID (locator not implemented yet)\n"))
                }
                if((*destinationAddress)[j].GetTag()==H225_AliasAddress::e_url_ID){
                        LOG((__FILE__,__FUNCTION__,8,"RAS:LOCATOR:PDU: destinationAddress contains url_ID (locator not implemented yet)\n"))
                }
                if((*destinationAddress)[j].GetTag()==H225_AliasAddress::e_transportID){
                        LOG((__FILE__,__FUNCTION__,8,"RAS:LOCATOR:PDU: destinationAddress contains transportID (locator not implemented yet)\n"))
                }
                if((*destinationAddress)[j].GetTag()==H225_AliasAddress::e_email_ID){
                        LOG((__FILE__,__FUNCTION__,8,"RAS:LOCATOR:PDU: destinationAddress contains email_ID (locator not implemented yet)\n"))
                }
                if((*destinationAddress)[j].GetTag()==H225_AliasAddress::e_partyNumber){
                        LOG((__FILE__,__FUNCTION__,8,"RAS:LOCATOR:PDU: destinationAddress contains partyNumber (locator not implemented yet)\n"))
                }
                if((*destinationAddress)[j].GetTag()==H225_AliasAddress::e_mobileUIM){
                        LOG((__FILE__,__FUNCTION__,8,"RAS:LOCATOR:PDU: destinationAddress contains mobileUIM (locator not implemented yet)\n"))
                }
           }
           return(-1);
        }
        else {
           return(1);
        }
}

int RASParser::connectDestinationSPECIAL(String locator){
        return(-1);
}
int RASParser::connectDestinationFAST(){
        //return(connectDestination());
}

int RASParser::setCallRoutingEntry(H225_TransportAddress* callSignalAddress,H225_TransportAddress* rasAddress,H225_ArrayOf_AliasAddress* destinationInfo){
	String locator;
	String prefix;
        String requestIP;
	String IP;
        String requestPORT;
	String PORT;
        String requestTIMEOUT;
	String TIMEOUT;
        String destination;
	TimeValue currTime;

	TimeValue::getCurrentSystemTime(currTime);

	LOG((__FILE__,__FUNCTION__,8,"RAS:LOCATOR:ADD:\n"))
        for(int j=0;j<(destinationInfo->GetSize());++j){
                if((*destinationInfo)[j].GetTag()==H225_AliasAddress::e_dialedDigits){
                        PASN_IA5String & dialedDigits = (*destinationInfo)[j];  
                        destination = String((const char *)(dialedDigits.GetValue())); 
                        LOG((__FILE__,__FUNCTION__,8,"RAS:LOCATOR:ADD: destinationInfo contains dialedDigits = %s\n",destination.c_str()))
			for (int i=0;i<MAXLOCATORS;++i){
                                locator = Global::getGlobal()->getConf()->getValAsString(type+"_Q931_LOCATOR") + "_LOCATOR" + "_PDU" + "_DIGIT" + "_" + i;
                                prefix  = Global::getGlobal()->getConf()->getValAsString(locator);
				requestIP      = locator + "_IP";
				requestPORT    = locator + "_PORT";
				requestTIMEOUT = locator + "_TIMEOUT";
                                if (prefix!=String("")) {
                                        LOG((__FILE__,__FUNCTION__,8,"RAS:LOCATOR:ADD: testing %s = %s for %s\n",locator.c_str(),prefix.c_str(),destination.c_str()))
                                        if(destination==prefix){
                                        	LOG((__FILE__,__FUNCTION__,8,"RAS:LOCATOR:ADD: %s = %s for %s exists, updating!\n",locator.c_str(),prefix.c_str(),destination.c_str()))
						break;
					}
					if ((Global::getGlobal()->getConf()->getValAsString(requestTIMEOUT))!=String("")) {
						if(currTime.tv_sec>(Global::getGlobal()->getConf()->getValAsInt(requestTIMEOUT))) {
                                        		LOG((__FILE__,__FUNCTION__,8,"RAS:LOCATOR:ADD: %s = %s timed out\n",locator.c_str(),prefix.c_str()))
							break; 
						}
					}
				}
				else {
					break;
				}	
				
			}
			if((Global::getGlobal()->getConf()->setKeyPrm(locator,destination))<0){
             			WRN((__FILE__,__FUNCTION__,"RAS:LOCATOR:ADD: could not set locator in database\n"))
                		return(-1);
			}
                        LOG((__FILE__,__FUNCTION__,8,"RAS:LOCATOR:ADD: %s = %s set\n",locator.c_str(),destination.c_str()))
			if(callSignalAddress->GetTag()==H225_TransportAddress::e_ipAddress){
                		H225_TransportAddress_ipAddress & h225_transportaddress_ipaddress=(*callSignalAddress);
				IP = "";
				IP = IP + h225_transportaddress_ipaddress.m_ip[0]
					+ "." + h225_transportaddress_ipaddress.m_ip[1]
					+ "." + h225_transportaddress_ipaddress.m_ip[2]
					+ "." + h225_transportaddress_ipaddress.m_ip[3];
				if((Global::getGlobal()->getConf()->setKeyPrm(requestIP,IP))<0){
                			WRN((__FILE__,__FUNCTION__,"RAS:LOCATOR:ADD: could not add locator in database\n"))
                			return(-1);
				}
                               	LOG((__FILE__,__FUNCTION__,8,"RAS:LOCATOR:ADD: %s = %s set\n",requestIP.c_str(),IP.c_str()))
				PORT = "";
				PORT = PORT + h225_transportaddress_ipaddress.m_port.GetValue();
				if((Global::getGlobal()->getConf()->setKeyPrm(requestPORT,PORT))<0){
                			WRN((__FILE__,__FUNCTION__,"RAS:LOCATOR:ADD: could not add locator in database\n"))
               				return(-1);
				}
                                LOG((__FILE__,__FUNCTION__,8,"RAS:LOCATOR:ADD: %s = %s set\n",requestPORT.c_str(),PORT.c_str()))
				if ((Global::getGlobal()->getConf()->getValAsString(type+"_Q931_LOCATOR_TIMEOUT"))!=String("")) {
					TIMEOUT = "";
					TIMEOUT = TIMEOUT + (currTime.tv_sec + (Global::getGlobal()->getConf()->getValAsInt(type+"_Q931_LOCATOR_TIMEOUT")));
					if((Global::getGlobal()->getConf()->setKeyPrm(requestTIMEOUT,TIMEOUT))<0){
                				WRN((__FILE__,__FUNCTION__,"RAS:LOCATOR:ADD: could not add locator in database\n"))
                				return(-1);
					}
                                	LOG((__FILE__,__FUNCTION__,8,"RAS:LOCATOR:ADD: %s = %s set\n",requestTIMEOUT.c_str(),TIMEOUT.c_str()))
				}
				
			}
        		else {
                		WRN((__FILE__,__FUNCTION__,"RAS:LOCATOR:ADD: H225_TransportAddress is not an ip address\n"))
                		return(-1);
        		}
		}
	        if(destinationInfo[j].GetTag()==H225_AliasAddress::e_h323_ID){
                        LOG((__FILE__,__FUNCTION__,8,"RAS:LOCATOR:ADD: destinationInfo contains h323_ID (locator not implemented yet)\n"))
                }
                if(destinationInfo[j].GetTag()==H225_AliasAddress::e_url_ID){
                        LOG((__FILE__,__FUNCTION__,8,"RAS:LOCATOR:ADD: destinationInfo contains url_ID (locator not implemented yet)\n"))
                }
                if(destinationInfo[j].GetTag()==H225_AliasAddress::e_transportID){
                        LOG((__FILE__,__FUNCTION__,8,"RAS:LOCATOR:ADD: destinationInfo contains transportID (locator not implemented yet)\n"))
                }
                if(destinationInfo[j].GetTag()==H225_AliasAddress::e_email_ID){
                        LOG((__FILE__,__FUNCTION__,8,"RAS:LOCATOR:ADD: destinationInfo contains email_ID (locator not implemented yet)\n"))
                }
                if(destinationInfo[j].GetTag()==H225_AliasAddress::e_partyNumber){
                        LOG((__FILE__,__FUNCTION__,8,"RAS:LOCATOR:ADD: destinationInfo contains partyNumber (locator not implemented yet)\n"))
                }
                if(destinationInfo[j].GetTag()==H225_AliasAddress::e_mobileUIM){
                        LOG((__FILE__,__FUNCTION__,8,"RAS:LOCATOR:ADD: destinationInfo contains mobileUIM (locator not implemented yet)\n"))
                }
	}
	return(-1);
}
int RASParser::setReplyAddress(H225_TransportAddress* replyAddress){
        if(replyAddress->GetTag()==H225_TransportAddress::e_ipAddress){
                H225_TransportAddress_ipAddress & h225_transportaddress_ipaddress=(*replyAddress);
                LOG((__FILE__,__FUNCTION__,8,"RAS: replyAddress %d.%d.%d.%d:%d -> %s:%d\n",
                        h225_transportaddress_ipaddress.m_ip[0],
                        h225_transportaddress_ipaddress.m_ip[1],
                        h225_transportaddress_ipaddress.m_ip[2],
                        h225_transportaddress_ipaddress.m_ip[3],
                        h225_transportaddress_ipaddress.m_port.GetValue(),
                        String::IPtoString(parent->con[0][0]->l_ip).c_str(),parent->con[0][0]->l_port))

                h225_transportaddress_ipaddress.m_port.SetValue(parent->con[0][0]->l_port);
                h225_transportaddress_ipaddress.m_ip[0]=(((parent->con[0][0]->l_ip)&0xff));
                h225_transportaddress_ipaddress.m_ip[1]=(((parent->con[0][0]->l_ip)&0xff00) >>8);
                h225_transportaddress_ipaddress.m_ip[2]=(((parent->con[0][0]->l_ip)&0xff0000) >>16);
                h225_transportaddress_ipaddress.m_ip[3]=(((parent->con[0][0]->l_ip)&0xff000000)>>24);
        }
        else {
                WRN((__FILE__,__FUNCTION__,"RAS: H225_TransportAddress is not an ip address\n"))
                return(-1);
        }
        return(0);
}
int RASParser::setCallSignalAddress(H225_TransportAddress* callSignalAddress){
	u_int32_t ip   = Global::getGlobal()->getConf()->getValAsIP("LOCALIP");
	int port = Global::getGlobal()->getConf()->getValAsInt(type+"_Q931_PORT");
        if(callSignalAddress->GetTag()==H225_TransportAddress::e_ipAddress){
                H225_TransportAddress_ipAddress & h225_transportaddress_ipaddress=(*callSignalAddress);
                LOG((__FILE__,__FUNCTION__,8,"RAS: callSignalAddress %d.%d.%d.%d:%d -> %s:%d\n",
                        h225_transportaddress_ipaddress.m_ip[0],
                        h225_transportaddress_ipaddress.m_ip[1],
                        h225_transportaddress_ipaddress.m_ip[2],
                        h225_transportaddress_ipaddress.m_ip[3],
                        h225_transportaddress_ipaddress.m_port.GetValue(),
                        String::IPtoString(ip).c_str(),port))

                h225_transportaddress_ipaddress.m_port.SetValue(port);
                h225_transportaddress_ipaddress.m_ip[0]=(((ip)&0xff));
                h225_transportaddress_ipaddress.m_ip[1]=(((ip)&0xff00) >>8);
                h225_transportaddress_ipaddress.m_ip[2]=(((ip)&0xff0000) >>16);
                h225_transportaddress_ipaddress.m_ip[3]=(((ip)&0xff000000)>>24);
        }
        else {
                WRN((__FILE__,__FUNCTION__,"RAS: H225_TransportAddress is not an ip address\n"))
                return(-1);
        }
        return(0);
}
int RASParser::setRasAddress(H225_TransportAddress* rasAddress){
	u_int32_t ip   = Global::getGlobal()->getConf()->getValAsIP("LOCALIP");
	int port = Global::getGlobal()->getConf()->getValAsInt(type+"_PORT");
        if(rasAddress->GetTag()==H225_TransportAddress::e_ipAddress){
                H225_TransportAddress_ipAddress & h225_transportaddress_ipaddress=(*rasAddress);
                LOG((__FILE__,__FUNCTION__,8,"RAS: rasAddress %d.%d.%d.%d:%d -> %s:%d\n",
                        h225_transportaddress_ipaddress.m_ip[0],
                        h225_transportaddress_ipaddress.m_ip[1],
                        h225_transportaddress_ipaddress.m_ip[2],
                        h225_transportaddress_ipaddress.m_ip[3],
                        h225_transportaddress_ipaddress.m_port.GetValue(),
                        String::IPtoString(ip).c_str(),port))

                h225_transportaddress_ipaddress.m_port.SetValue(port);
                h225_transportaddress_ipaddress.m_ip[0]=(((ip)&0xff));
                h225_transportaddress_ipaddress.m_ip[1]=(((ip)&0xff00) >>8);
                h225_transportaddress_ipaddress.m_ip[2]=(((ip)&0xff0000) >>16);
                h225_transportaddress_ipaddress.m_ip[3]=(((ip)&0xff000000)>>24);
        }
        else {
                WRN((__FILE__,__FUNCTION__,"RAS: H225_TransportAddress is not an ip address\n"))
                return(-1);
        }
        return(0);
}



int RASParser::write_0(Connection *dc, Connection *sc){
	LOG((__FILE__,__FUNCTION__,15,"RAS: write_0\n"))
	dc=sc;
        if(dc->connectionWrite(sc)<0) {
                WRN((__FILE__,__FUNCTION__,"RAS: write on socket\n"))
                return(0);
        }
	dc->p_ip=0;
	dc->p_mask=0;
	dc->p_port=0;
        return(sc->packet_len);
}
int RASParser::write_1(Connection *dc, Connection *sc){
	LOG((__FILE__,__FUNCTION__,15,"RAS: write_1\n"))
	return(-1);
}
int RASParser::write_2(Connection *dc, Connection *sc){
	LOG((__FILE__,__FUNCTION__,15,"RAS: write_2\n"))
	return(-1);
}
int RASParser::write_3(Connection *dc, Connection *sc){
	LOG((__FILE__,__FUNCTION__,15,"RAS: write_3\n"))
	return(-1);
}
int RASParser::write_default(Connection *dc, Connection *sc){
        LOG((__FILE__,__FUNCTION__,15,"RAS: write_default\n"))
	return(-1);
}

int RASParser::read_0(Connection *c){
        int     len=0;
	LOG((__FILE__,__FUNCTION__,15,"RAS: read_0\n"))
        c->zeroBuffer();
        if ((len=c->connectionRead(BUFFERSIZE-1))<0) return(-1);
        LOG((__FILE__,__FUNCTION__,15,"RAS: read_default, length: %d\n",len))
        return(c->packet_len);
}
int RASParser::read_1(Connection *c){
	LOG((__FILE__,__FUNCTION__,15,"RAS: read_1\n"))
	return(-1);
}
int RASParser::read_2(Connection *c){
	LOG((__FILE__,__FUNCTION__,15,"RAS: read_2\n"))
	return(-1);
}
int RASParser::read_3(Connection *c){
	LOG((__FILE__,__FUNCTION__,15,"RAS: read_3\n"))
	return(-1);
}
int RASParser::read_default(Connection *c){
	LOG((__FILE__,__FUNCTION__,15,"RAS: read_default\n"))
	return(-1);
}

bool RASParser::checkPDU(String pdutype){
	String check = type + "_ALLOW_" + pdutype;
	if ((Global::getGlobal()->getConf()->getValAsString(check))==String("YES")) return true; 
	WRN((__FILE__,__FUNCTION__,"RAS: drop pdu %s\n",pdutype.c_str()))
	return false;
}

int RASParser::parse_00(Connection *c){
	H225_RasMessage 		rasMsg;
	PBYTEArray        		rasarrayIN;
        PPER_Stream                     rasstreamIN;
        PPER_Stream                     rasstreamOUT;

	LOG((__FILE__,__FUNCTION__,15,"RAS: parse_00\n"))

        /*read and decode/modify ras*/
        for(int i=0;i<(c->packet_len);i++){
                rasarrayIN[i]=(unsigned char) c->buffer[i];
        }

        rasstreamIN = PPER_Stream(rasarrayIN);
        rasMsg.Decode(rasstreamIN);

#ifdef _H323DEBUG_
        printf("\n===BEGIN RAS PDU DECODED ==============================\n");
        rasMsg.PrintOn(cout);
        printf("\n===END RAS PDU DECODED ================================\n");
#endif
	if (rasMsg.GetTag()==H225_RasMessage::e_gatekeeperRequest) {
                LOG((__FILE__,__FUNCTION__,8,"RAS:GATEKEEPERREQUEST: PDU RECEIVED\n"))
		if (!checkPDU("gatekeeperRequest")) return(0);
	}
	else if (rasMsg.GetTag()==H225_RasMessage::e_gatekeeperConfirm) {
                LOG((__FILE__,__FUNCTION__,8,"RAS:GATEKEEPERCONFIRM: PDU RECEIVED\n"))
		if (!checkPDU("gatekeeperConfirm")) return(0);
	}
	else if (rasMsg.GetTag()==H225_RasMessage::e_gatekeeperReject) {
                LOG((__FILE__,__FUNCTION__,8,"RAS:GATEKEEPERREJECT: PDU RECEIVED\n"))
		if (!checkPDU("gatekeeperReject")) return(0);
	}
	else if (rasMsg.GetTag()==H225_RasMessage::e_registrationRequest) {
                LOG((__FILE__,__FUNCTION__,8,"RAS:REGISTRATIONREQUEST: PDU RECEIVED\n"))
		if (!checkPDU("registrationRequest")) return(0);
	}
	else if (rasMsg.GetTag()==H225_RasMessage::e_registrationConfirm) {
                LOG((__FILE__,__FUNCTION__,8,"RAS:REGISTRATIONCONFIRM: PDU RECEIVED\n"))
		if (!checkPDU("registrationConfirm")) return(0);
	}
	else if (rasMsg.GetTag()==H225_RasMessage::e_registrationReject) {
                LOG((__FILE__,__FUNCTION__,8,"RAS:REGISTRATIONREJECT: PDU RECEIVED\n"))
		if (!checkPDU("registrationReject")) return(0);
	}
	else if (rasMsg.GetTag()==H225_RasMessage::e_unregistrationRequest) {
                LOG((__FILE__,__FUNCTION__,8,"RAS:UNREGISTRATIONREQUEST: PDU RECEIVED\n"))
		if (!checkPDU("unregistrationRequest")) return(0);
	}
	else if (rasMsg.GetTag()==H225_RasMessage::e_unregistrationConfirm) {
                LOG((__FILE__,__FUNCTION__,8,"RAS:UNREGISTRATIONCONFIRM: PDU RECEIVED\n"))
		if (!checkPDU("unregistrationConfirm")) return(0);
	}
	else if (rasMsg.GetTag()==H225_RasMessage::e_unregistrationReject) {
                LOG((__FILE__,__FUNCTION__,8,"RAS:UNREGISTRATIONREJECT: PDU RECEIVED\n"))
		if (!checkPDU("unregistrationReject")) return(0);
	}
	else if (rasMsg.GetTag()==H225_RasMessage::e_admissionRequest) {
                LOG((__FILE__,__FUNCTION__,8,"RAS:ADMISSIONREQUEST: PDU RECEIVED\n"))
		if (!checkPDU("admissionRequest")) return(0);
	}
	else if (rasMsg.GetTag()==H225_RasMessage::e_admissionConfirm) {
                LOG((__FILE__,__FUNCTION__,8,"RAS:ADMISSIONCONFIRM: PDU RECEIVED\n"))
		if (!checkPDU("admissionConfirm")) return(0);
	}
	else if (rasMsg.GetTag()==H225_RasMessage::e_admissionReject) {
                LOG((__FILE__,__FUNCTION__,8,"RAS:ADMISSIONREJECT: PDU RECEIVED\n"))
		if (!checkPDU("admissionReject")) return(0);
	}
	else if (rasMsg.GetTag()==H225_RasMessage::e_bandwidthRequest) {
                LOG((__FILE__,__FUNCTION__,8,"RAS:BANDWITHREQUEST: PDU RECEIVED\n"))
		if (!checkPDU("bandwidthRequest")) return(0);
	}
	else if (rasMsg.GetTag()==H225_RasMessage::e_bandwidthConfirm) {
                LOG((__FILE__,__FUNCTION__,8,"RAS:BANDWITHCONFIRM: PDU RECEIVED\n"))
		if (!checkPDU("bandwidthConfirm")) return(0);
	}
	else if (rasMsg.GetTag()==H225_RasMessage::e_bandwidthReject) {
                LOG((__FILE__,__FUNCTION__,8,"RAS:BANDWITHREJECT: PDU RECEIVED\n"))
		if (!checkPDU("bandwidthReject")) return(0);
	}
	else if (rasMsg.GetTag()==H225_RasMessage::e_disengageRequest) {
                LOG((__FILE__,__FUNCTION__,8,"RAS:DISENGAGEREQUEST: PDU RECEIVED\n"))
		if (!checkPDU("disengageRequest")) return(0);
	}
	else if (rasMsg.GetTag()==H225_RasMessage::e_disengageConfirm) {
                LOG((__FILE__,__FUNCTION__,8,"RAS:DISENGAGECONFIRM: PDU RECEIVED\n"))
		if (!checkPDU("disengageConfirm")) return(0);
	}
	else if (rasMsg.GetTag()==H225_RasMessage::e_disengageReject) {
                LOG((__FILE__,__FUNCTION__,8,"RAS:DISENGAGEREJECT: PDU RECEIVED\n"))
		if (!checkPDU("disengageReject")) return(0);
	}
	else if (rasMsg.GetTag()==H225_RasMessage::e_locationRequest) {
                LOG((__FILE__,__FUNCTION__,8,"RAS:LOCATIONREQUEST: PDU RECEIVED\n"))
		if (!checkPDU("locationRequest")) return(0);
		H225_LocationRequest & lrq = rasMsg;
		connectDestinationDestinationAddressPDU(&lrq.m_destinationInfo);
		lrqRequestSeqNum[lrqring]   	= lrq.m_requestSeqNum.GetValue();
		lrqRequestReplyAddress[lrqring] = lrq.m_replyAddress;
		lrqDestinationInfo[lrqring] 	= lrq.m_destinationInfo;
		lrqring=lrqring+1;
		if (lrqring>=MAXLRQ) lrqring=0;
		if(setReplyAddress(&(lrq.m_replyAddress))<0) return(0);
	}
	else if (rasMsg.GetTag()==H225_RasMessage::e_locationConfirm) {
                LOG((__FILE__,__FUNCTION__,8,"RAS:LOCATIONCONFIRM: PDU RECEIVED\n"))
		if (!checkPDU("locationConfirm")) return(0);
		H225_LocationConfirm & lcf = rasMsg;
        	for(int i=0;i<MAXLRQ;++i){
			if (lrqRequestSeqNum[i]==lcf.m_requestSeqNum.GetValue()){
				setCallRoutingEntry(&(lcf.m_callSignalAddress),&(lcf.m_rasAddress),&(lrqDestinationInfo[i]));
				connectDestinationReplyAddressPDU(&(lrqRequestReplyAddress[i]));
				lrqRequestSeqNum[i]=-1;
				break;
			}
        	}
		if(setCallSignalAddress(&(lcf.m_callSignalAddress))<0) return(0);
		if(setRasAddress(&(lcf.m_rasAddress))<0) return(0);
	}
	else if (rasMsg.GetTag()==H225_RasMessage::e_locationReject) {
                LOG((__FILE__,__FUNCTION__,8,"RAS:LOCATIONREJECT: PDU RECEIVED\n"))
		if (!checkPDU("locationReject")) return(0);
		H225_LocationReject & lrj = rasMsg;
        	for(int i=0;i<MAXLRQ;++i){
			if (lrqRequestSeqNum[i]==lrj.m_requestSeqNum.GetValue()){
				connectDestinationReplyAddressPDU(&(lrqRequestReplyAddress[i]));
				lrqRequestSeqNum[i]=-1;
				break;
			}
        	}
	}
	else if (rasMsg.GetTag()==H225_RasMessage::e_infoRequest) {
                LOG((__FILE__,__FUNCTION__,8,"RAS:INFOREQUEST: PDU RECEIVED\n"))
		if (!checkPDU("infoRequest")) return(0);
	}
	else if (rasMsg.GetTag()==H225_RasMessage::e_infoRequestResponse) {
                LOG((__FILE__,__FUNCTION__,8,"RAS:INFOREQUESTRESPONSE: PDU RECEIVED\n"))
		if (!checkPDU("infoRequestResponse")) return(0);
	}
	else if (rasMsg.GetTag()==H225_RasMessage::e_nonStandardMessage) {
                LOG((__FILE__,__FUNCTION__,8,"RAS:NONSTANDARDMESSAGE: PDU RECEIVED\n"))
		if (!checkPDU("nonStandardMessage")) return(0);
	}
	else if (rasMsg.GetTag()==H225_RasMessage::e_unknownMessageResponse) {
                LOG((__FILE__,__FUNCTION__,8,"RAS:UNKNOWNMESSAGERESPONSE: PDU RECEIVED\n"))
		if (!checkPDU("unknownMessageResponse")) return(0);
	}
	else if (rasMsg.GetTag()==H225_RasMessage::e_requestInProgress) {
                LOG((__FILE__,__FUNCTION__,8,"RAS:REQUESTINPROGRESS: PDU RECEIVED\n"))
		if (!checkPDU("requestInProgress")) return(0);
	}
	else if (rasMsg.GetTag()==H225_RasMessage::e_resourcesAvailableIndicate) {
                LOG((__FILE__,__FUNCTION__,8,"RAS:RESSOURCEAVAILABLEINDICATION: PDU RECEIVED\n"))
		if (!checkPDU("resourcesAvailableIndicate")) return(0);
	}
	else if (rasMsg.GetTag()==H225_RasMessage::e_resourcesAvailableConfirm) {
                LOG((__FILE__,__FUNCTION__,8,"RAS:RESSOURCEAVAILABLECONFIRM: PDU RECEIVED\n"))
		if (!checkPDU("resourcesAvailableConfirm")) return(0);
	}
	else if (rasMsg.GetTag()==H225_RasMessage::e_infoRequestAck) {
                LOG((__FILE__,__FUNCTION__,8,"RAS:INFOREQUESTACK: PDU RECEIVED\n"))
		if (!checkPDU("infoRequestAck")) return(0);
	}
	else if (rasMsg.GetTag()==H225_RasMessage::e_infoRequestNak) {
                LOG((__FILE__,__FUNCTION__,8,"RAS:INFOREQUESTNACK: PDU RECEIVED\n"))
		if (!checkPDU("infoRequestNak")) return(0);
	}
	else if (rasMsg.GetTag()==H225_RasMessage::e_serviceControlIndication) {
                LOG((__FILE__,__FUNCTION__,8,"RAS:SERVICECONTROLINDICATION: PDU RECEIVED\n"))
		if (!checkPDU("serviceControlIndication")) return(0);
	}
	else if (rasMsg.GetTag()==H225_RasMessage::e_serviceControlResponse) {
                LOG((__FILE__,__FUNCTION__,8,"RAS:SERVICECONTROLRESPONSE: PDU RECEIVED\n"))
		if (!checkPDU("serviceControlResponse")) return(0);
	}
	else{
		WRN((__FILE__,__FUNCTION__,"RAS:UNKNOWN: RAS PDU RECEIVED\n"))
		return(0);
	}

#ifdef _H323DEBUG_
        printf("\n===BEGIN RAS PDU MODIFIED =============================\n");
        rasMsg.PrintOn(cout);
        printf("\n===END RAS PDU MODIFIED ===============================\n");
#endif
        rasMsg.Encode(rasstreamOUT);
	rasstreamOUT.CompleteEncoding();

        c->packet_len=rasstreamOUT.GetSize();

        for(u_int32_t i=0;i<rasstreamOUT.GetSize();i++){
                c->buffer[i]=rasstreamOUT[i];
        }

	if((parent->con[0][0]->p_ip==0) || (parent->con[0][0]->p_port==0) ){
                        WRN((__FILE__,__FUNCTION__,"RAS: the destination was not specified!\n"))
                        return(0);
        }

	return(1);
}

int RASParser::parse_10(Connection *c){
	LOG((__FILE__,__FUNCTION__,15,"RAS: parse_10\n"))
	return(-1);
}
int RASParser::parse_20(Connection *c){
	LOG((__FILE__,__FUNCTION__,15,"RAS: parse_20\n"))
	return(-1);
}
int RASParser::parse_30(Connection *c){
	LOG((__FILE__,__FUNCTION__,15,"RAS: parse_30\n"))
	return(-1);
}
int RASParser::parse_default0(Connection *c){
	LOG((__FILE__,__FUNCTION__,15,"RAS: parse_default0\n"))
	return(-1);
}

int RASParser::parse_01(Connection *c){
	LOG((__FILE__,__FUNCTION__,15,"RAS: parse_01\n"))
	return(-1);
}

int RASParser::parse_11(Connection *c){
	LOG((__FILE__,__FUNCTION__,15,"RAS: parse_11\n"))
	return(-1);
}
int RASParser::parse_21(Connection *c){
	LOG((__FILE__,__FUNCTION__,15,"RAS: parse_21\n"))
	return(-1);
}
int RASParser::parse_31(Connection *c){
	LOG((__FILE__,__FUNCTION__,15,"RAS: parse_31\n"))
	return(-1);
}
int RASParser::parse_default1(Connection *c){
	LOG((__FILE__,__FUNCTION__,15,"RAS: parse_default1\n"))
	return(-1);
}

