/* Copyright (C) 2000 KOM/Darmstadt University of Technology
 *
 * You are allowed to use all other parts of the code under the following terms:
 *
 * For non-commercial use, code may be used in unmodified form provided
 * that this copyright notice and this permission notice appear in
 * supporting documentation.
 *
 * This software is provided "as is" and without any express or implied
 * warranties, including, without limitation, the implied warranty of
 * fitness for a particular purpose.
 *
 * The code may be subjected to the GNU General Public License, Version 2,
 * and re-distributed under the terms of this license.
 *
 * Commercial use other than under the terms of the GNU General Public
 * License is allowed only after express negotiation of conditions
 * with the authors.
 */

#include "SIPCall.h"
#include "../Connection.h"
#include "../Session.h"
#include "../Conf.h"
#include "SIPMediaChannel.h"

#include <stdlib.h> /* malloc, free*/
#include <string.h> /* strncpy */


int SIPCall::sip=-1;
Connection **SIPCall::SIP_in=NULL;
Connection **SIPCall::SIP_out=NULL;


SIPCall::SIPCall(){
	msg_init(&msg_in);
	msg_init(&msg_out);

	s_ip=0;
	s_port=0;
	d_ip=0;
	d_port=0;
	id=0;
	
	callEnd=false;

	for (int i=0;i<MAXMEDIACHANNELS;++i){
		mediaChannel[i]=NULL;
	}
}

SIPCall::~SIPCall(){
	for (int i=0;i<MAXMEDIACHANNELS;++i){
		if (mediaChannel[i]!=NULL) delete mediaChannel[i];
	}
	msg_free(msg_in);
	free(msg_in);
	msg_free(msg_out);
	free(msg_out);
}

String SIPCall::getCallId(sip_t  *msg){
	char    *tmp;

	tmp=(char*)malloc(120*sizeof(char));

        if (call_id_2char(msg->call_id, &tmp)<0){
                free(tmp);
                return("");
        }

        return String(tmp);
}

int SIPCall::getMediaChannel(int id_a,int id_b){
	for (int i=0;i<MAXMEDIACHANNELS;++i){
               	if (mediaChannel[i]!=NULL){
                       	if(((mediaChannel[i]->id_a)==id_a) && ((mediaChannel[i]->id_b==id_b))) return(i);
               	}
       	}
	return(-1);
}

int SIPCall::delMediaChannel(int id_a,int id_b){
	for (int i=0;i<MAXMEDIACHANNELS;++i){
        	if (mediaChannel[i]!=NULL){
			if(((mediaChannel[i]->id_a)==id_a) && ((mediaChannel[i]->id_b==id_b))) {
				delete mediaChannel[i];
				mediaChannel[i]=NULL;
				return(0);
			}
               	}
        }
	return(-1);
}

int SIPCall::setMediaChannel(Session* parent){
	for (int i=0;i<MAXMEDIACHANNELS;++i){
               	if (mediaChannel[i]==NULL){
			mediaChannel[i]=new SIPMediaChannel();
			if((mediaChannel[i]->setMediaChannel(parent))<0){
				delete mediaChannel[i];
				mediaChannel[i]=NULL;
				return(-1);
			}
			return(i);
               	}
       	}
	return(-1);
}

int SIPCall::useMediaChannel(Session* parent,int id_a,int id_b){
	int ret=-1;
	if ((ret=getMediaChannel(id_a,id_b))>=0) return(ret);
	for (int i=0;i<MAXMEDIACHANNELS;++i){
               	if (mediaChannel[i]==NULL){
			mediaChannel[i]=new SIPMediaChannel();
			if((mediaChannel[i]->useMediaChannel(parent,id_a,id_b))<0){
				delete mediaChannel[i];
				mediaChannel[i]=NULL;
				return(-1);
			}
			return(i);
               	}
       	}
	return(-1);
}

int SIPCall::delSIPChannel(){
        if (SIP_in!=NULL)   {
                delete (*SIP_in);
                (*SIP_in)=NULL;
        }
        if (SIP_out!=NULL)  {
                delete (*SIP_out);
                (*SIP_out)=NULL;
        }
	sip=-1;
	return(0);
}

int SIPCall::setSIPChannel(Session* parent){
	SIP_in=&(parent->con[0][0]);
	SIP_out=SIP_in;
	sip=0;
	return(sip);
}

int SIPCall::useSIPChannel(Session* parent,int port){
	if (setSIPChannel(parent)<0) return(-1);

	(*SIP_in)->l_ip        = Global::getGlobal()->getConf()->getValAsIP("LOCALIP");
        (*SIP_in)->l_mask      = 0xffffffff;
        (*SIP_in)->l_port      = port;
        (*SIP_in)->ipproto     = 17;
        (*SIP_in)->p_ip        = 0;
        (*SIP_in)->p_mask      = 0;
        (*SIP_in)->p_port      = 0;
        (*SIP_in)->direction   = 1;
        (*SIP_in)->hardUDP     = false;
        if(((*SIP_in)->connectionConnect())<0) return(-1);

	return(sip);
}

