/* Copyright (C) 2000 KOM/Darmstadt University of Technology
 *
 * You are allowed to use all other parts of the code under the following terms:
 *
 * For non-commercial use, code may be used in unmodified form provided
 * that this copyright notice and this permission notice appear in
 * supporting documentation.
 *
 * This software is provided "as is" and without any express or implied
 * warranties, including, without limitation, the implied warranty of
 * fitness for a particular purpose.
 *
 * The code may be subjected to the GNU General Public License, Version 2,
 * and re-distributed under the terms of this license.
 *
 * Commercial use other than under the terms of the GNU General Public
 * License is allowed only after express negotiation of conditions
 * with the authors.
 */

#include "SIPMediaChannel.h"
#include "../Connection.h"
#include "../Session.h"
#include "../Conf.h"


SIPMediaChannel::SIPMediaChannel(){
	id_a	= -1;
	id_b	= -1;
	rtp	= -1;
	rtcp	= -1;
	RTP_in	= NULL;
	RTCP_in	= NULL;
	RTP_out	= NULL;
	RTCP_out= NULL;
}

SIPMediaChannel::~SIPMediaChannel(){
	delMediaChannel();
}

int SIPMediaChannel::getMediaChannelRTCP(){
	return (rtcp);
}

int SIPMediaChannel::getMediaChannelRTP(){
	return (rtp);
}

int SIPMediaChannel::delMediaChannel(){
	if (RTP_in!=NULL)   {
		delete (*RTP_in); 
		(*RTP_in)=NULL;
	}
	if (RTP_out!=NULL)  {
		delete (*RTP_out); 
		(*RTP_out)=NULL;
	}
	if (RTCP_in!=NULL)  {
		delete (*RTCP_in); 
		(*RTCP_in)=NULL;
	}
	if (RTCP_out!=NULL) {
		delete (*RTCP_out);
		(*RTCP_out)=NULL;
	}
	id_a	= -1;
	id_b	= -1;
	rtp	= -1;
	rtcp	= -1;
	RTP_in	= NULL;
	RTCP_in	= NULL;
	RTP_out	= NULL;
	RTCP_out= NULL;
	return(0);
}

int SIPMediaChannel::setMediaChannel(Session* parent){
	if ((RTP_in==NULL) && (RTP_out==NULL) &&  (RTCP_in==NULL) && (RTCP_out==NULL)){
		if ((rtcp=parent->addConnection())<0) {
			delMediaChannel();
			return(-1);
		}
                RTCP_in  = &(parent->con[rtcp][0]);
                RTCP_out = &(parent->con[rtcp][1]);
		if ((rtp=parent->addConnection())<0) {
			delMediaChannel();
			return(-1);
		}
                RTP_in  = &(parent->con[rtp][0]);
                RTP_out = &(parent->con[rtp][1]);
	}
	return(rtcp);
}
int SIPMediaChannel::useMediaChannel(Session* parent,int id_a_t,int id_b_t){
	int tryChannel=0;
	if (setMediaChannel(parent)<0) return(-1);
	id_a=id_a_t;
	id_b=id_b_t;
        while ((tryChannel<10) && ((((*(RTCP_in))->connectionGetState())!=CONNECT) || (((*(RTP_in))->connectionGetState())!=CONNECT))) {
		(*(RTCP_in))->connectionClose();
                (*(RTCP_in))->d_name     = String("RTCP-SRC");
                (*(RTCP_in))->d_number   = 3;
                (*(RTCP_in))->RTCP       = true;
                (*(RTCP_in))->direction  = 1;
                (*(RTCP_in))->ipproto    = 17;
                (*(RTCP_in))->l_ip       = Global::getGlobal()->getConf()->getValAsIP("LOCALIP");
                (*(RTCP_in))->l_mask     = 0xffffffff;
                (*(RTCP_in))->l_port     = -1;
                (*(RTCP_in))->p_ip       = 0;
                (*(RTCP_in))->p_mask     = 0;
                (*(RTCP_in))->p_port     = -1;
                (*(RTCP_in))->hardUDP    = false;
                if((*(RTCP_in))->connectionConnect()<0) return(-1);

                (*(RTP_in))->connectionClose();
                (*(RTP_in))->d_name     = String("RTP-SRC");
                (*(RTP_in))->d_number   = 2;
                (*(RTP_in))->direction  = 1;
                (*(RTP_in))->ipproto    = 17;
                (*(RTP_in))->l_ip       = Global::getGlobal()->getConf()->getValAsIP("LOCALIP");
                (*(RTP_in))->l_mask     = 0xffffffff;
                (*(RTP_in))->l_port     = ((*(RTCP_in))->l_port)-1;
                (*(RTP_in))->p_ip       = 0;
                (*(RTP_in))->p_mask     = 0;
                (*(RTP_in))->p_port     = -1;
                (*(RTP_in))->hardUDP    = false;
                (*(RTP_in))->connectionConnect();

		tryChannel++;
        }
	tryChannel=0;
        while ((tryChannel<10) && ((((*(RTCP_out))->connectionGetState())!=CONNECT) || (((*(RTP_out))->connectionGetState())!=CONNECT))) {
		(*(RTCP_out))->connectionClose();
                (*(RTCP_out))->d_name     = String("RTCP-SRC");
                (*(RTCP_out))->d_number   = 3;
                (*(RTCP_out))->RTCP       = true;
                (*(RTCP_out))->direction  = 1;
                (*(RTCP_out))->ipproto    = 17;
                (*(RTCP_out))->l_ip       = Global::getGlobal()->getConf()->getValAsIP("LOCALIP");
                (*(RTCP_out))->l_mask     = 0xffffffff;
                (*(RTCP_out))->l_port     = -1;
                (*(RTCP_out))->p_ip       = 0;
                (*(RTCP_out))->p_mask     = 0;
                (*(RTCP_out))->p_port     = -1;
                (*(RTCP_out))->hardUDP    = false;
                if((*(RTCP_out))->connectionConnect()<0) return(-1);

                (*(RTP_out))->connectionClose();
                (*(RTP_out))->d_name     = String("RTP-SRC");
                (*(RTP_out))->d_number   = 2;
                (*(RTP_out))->direction  = 1;
                (*(RTP_out))->ipproto    = 17;
                (*(RTP_out))->l_ip       = Global::getGlobal()->getConf()->getValAsIP("LOCALIP");
                (*(RTP_out))->l_mask     = 0xffffffff;
                (*(RTP_out))->l_port     = ((*(RTCP_out))->l_port)-1;
                (*(RTP_out))->p_ip       = 0;
                (*(RTP_out))->p_mask     = 0;
                (*(RTP_out))->p_port     = -1;
                (*(RTP_out))->hardUDP    = false;
                (*(RTP_out))->connectionConnect();

		tryChannel++;
        }
	return rtcp;
}
