/* Copyright (C) 2000 KOM/Darmstadt University of Technology
 *
 * You are allowed to use all other parts of the code under the following terms:
 *
 * For non-commercial use, code may be used in unmodified form provided
 * that this copyright notice and this permission notice appear in
 * supporting documentation.
 *
 * This software is provided "as is" and without any express or implied
 * warranties, including, without limitation, the implied warranty of
 * fitness for a particular purpose.
 *
 * The code may be subjected to the GNU General Public License, Version 2,
 * and re-distributed under the terms of this license.
 *
 * Commercial use other than under the terms of the GNU General Public
 * License is allowed only after express negotiation of conditions
 * with the authors.
 */





#include "SIPParser.h"
#include "../Log.h"
#include "../Conf.h"
#include "../Session.h"
#include "../Fwfil.h"

#include <stdlib.h> /* malloc, free*/
#include <string.h> /* strncmp, strncmpy*/



int SIPParser::KOMproxydCallId=0;

SIPParser::SIPParser(Session* parent_t,String type_t){
	LOG((__FILE__,__FUNCTION__,15,"SIP: creating parser\n"))
	type	= type_t;
	parent	= parent_t;
	size=0;

	for(int i=0;i<MAXCALLS;++i){
                callList[i]=NULL;
        }

	parser_init();
	SIPCall::useSIPChannel(parent,Global::getGlobal()->getConf()->getValAsInt(type+"_PORT"));
}

SIPParser::~SIPParser() {
	LOG((__FILE__,__FUNCTION__,15,"SIP: deleting parser\n"))
	delCall();
}


int SIPParser::getDirection(sip_t  *msg, SIPCall *c){
	if (msg->call_id==NULL) return(-1);

	/* if in and out are not set, the message is the first one of a call */
	if ((c->msg_in->call_id==NULL) && (c->msg_out->call_id==NULL)) return(0);

	/* from this point on, both callids have to be set */
	if ((c->msg_in->call_id==NULL) || (c->msg_out->call_id==NULL)) return(-1);

	if(SIPCall::getCallId(msg)==SIPCall::getCallId(c->msg_in)){
		return(0);
        }
        else if(SIPCall::getCallId(msg)==SIPCall::getCallId(c->msg_out)){
		return(1);
        }
	return(-1);
}

int SIPParser::storeMethod(sip_t  *msg, SIPCall *c, bool direction){
	char *tmp;

        if ((c->SIP_in==NULL) || (c->SIP_out==NULL)) return(-1);
        tmp=(char*)malloc(120*sizeof(char));

	if (c->msg_in->strtline->rquri==NULL) {
        	if(url_clone(msg->strtline->rquri, &(c->msg_in->strtline->rquri))<0) {
                	free(tmp);
                	return(-1);
        	}
	}
	if (c->msg_out->strtline->rquri==NULL) {
		url_init(&(c->msg_out->strtline->rquri));
        	sprintf(tmp,"sip:%s@%s:%d",msg->strtline->rquri->username,(String::IPtoString((*SIPCall::SIP_out)->p_ip)).c_str(),(*SIPCall::SIP_out)->p_port);
        	if (url_parse(c->msg_out->strtline->rquri, tmp)<0){
                	free(tmp);
                	return(-1);
        	}
	}
	free(tmp);
	return(0);
}
int SIPParser::setMethod(sip_t  *msg, SIPCall *c, bool direction){
	sip_t   *msg_modifier;

        if ((c->SIP_in==NULL) || (c->SIP_out==NULL)) return(-1);

	if (!direction) msg_modifier=c->msg_out;
	else msg_modifier=c->msg_in;

        url_free(msg->strtline->rquri);
        free(msg->strtline->rquri);
        if(url_clone(msg_modifier->strtline->rquri, &(msg->strtline->rquri))<0) {
                return(-1);
        }

	return(0);
}

int SIPParser::storeCallid(sip_t  *msg, SIPCall *c, bool direction){
	char *tmp;

        if ((c->SIP_in==NULL) || (c->SIP_out==NULL)) return(-1);
        tmp=(char*)malloc(120*sizeof(char));
	
	if (c->msg_in->call_id==NULL) {
        	/* set incomming callid */
        	if(call_id_clone(msg->call_id, &(c->msg_in->call_id))<0) {
                	free(tmp);
                	return(-1);
        	}
	}
	if (c->msg_out->call_id==NULL) {
        	/* set outgoing callid */
        	KOMproxydCallId++;
        	sprintf(tmp,"KOMproxyd-call-%d-%d@%s",KOMproxydCallId,c->id,(String::IPtoString((*SIPCall::SIP_out)->l_ip)).c_str());
        	if (msg_setcall_id(c->msg_out, tmp)<0){
                	free(tmp);
                	return(-1);
        	}
	}
	free(tmp);
	return(0);
}

int SIPParser::setCallid(sip_t  *msg, SIPCall *c, bool direction){
	sip_t   *msg_modifier;

        if ((c->SIP_in==NULL) || (c->SIP_out==NULL)) return(-1);

	if (!direction) msg_modifier=c->msg_out;
	else msg_modifier=c->msg_in;

        /* modify callid */
        call_id_free(msg->call_id);
        free(msg->call_id);
        if(call_id_clone(msg_modifier->call_id, &(msg->call_id))<0) {
                return(-1);
        }

	return(0);
}

int SIPParser::storeVia(sip_t  *msg, SIPCall *c, bool direction){
	char *tmp;
	int pos = 0;

        if ((c->SIP_in==NULL) || (c->SIP_out==NULL)) return(-1);
        tmp=(char*)malloc(120*sizeof(char));
	
	if (list_size(c->msg_in->vias)==0) {
        	/* set vias in */
        	pos = 0;
        	while (!list_eol(msg->vias,pos)) {
                	via_t *via;
                	via = (via_t *)list_get(msg->vias,pos);
                	if (via_2char(via, &tmp)<0){
                        	free(tmp);
                        	return(-1);
                	}
                	if (msg_setvia(c->msg_in, tmp)<0){
                        	free(tmp);
                        	return(-1);
                	}
                	pos++;
        	}
	}
	if (list_size(c->msg_out->vias)==0) {
        	/* set vias out */
        	tmp=(char*)malloc(120*sizeof(char));
        	sprintf(tmp,"SIP/2.0/UDP %s:%d",(String::IPtoString((*SIPCall::SIP_out)->l_ip)).c_str(),((*SIPCall::SIP_out)->l_port));
        	if (msg_appendtopvia(c->msg_out, tmp)<0){
                	free(tmp);
                	return(-1);
        	}
	}
        free(tmp);
	return(0);
}

int SIPParser::setVia(sip_t  *msg, SIPCall *c, bool direction){
	char *tmp;
	sip_t   *msg_modifier;
	int pos=0;

        if ((c->SIP_in==NULL) || (c->SIP_out==NULL)) return(-1);
        tmp=(char*)malloc(120*sizeof(char));

	if (!direction) msg_modifier=c->msg_out;
	else msg_modifier=c->msg_in;

	/* modify vias */
        /* remove current vias */
        pos = 0;
        while (!list_eol(msg->vias,pos)) {
                via_t *via;
                via = (via_t *)list_get(msg->vias,pos);
                list_remove(msg->vias,pos);
                via_free(via);
                free(via);
        }
        /* add vias */
        pos = 0;
        while (!list_eol(msg_modifier->vias,pos)) {
                via_t *via;
                via = (via_t *)list_get(msg_modifier->vias,pos);
                if (via_2char(via, &tmp)<0){
                        free(tmp);
                        return(-1);
                }
                if (msg_setvia(msg, tmp)<0){
                        free(tmp);
                        return(-1);
                }
                pos++;
        }
        free(tmp);
	return(0);
}

int SIPParser::storeFrom(sip_t  *msg, SIPCall *c, bool direction){
	char *tmp;

        if ((c->SIP_in==NULL) || (c->SIP_out==NULL)) return(-1);
        tmp=(char*)malloc(120*sizeof(char));

	if ((c->msg_in->from)==NULL) {
		/* set from in */
        	if(from_clone(msg->from, &(c->msg_in->from))<0) {
                	free(tmp);
                	return(-1);
        	}
	}
	if ((c->msg_out->from)==NULL) {
        	/* set from out */
        	sprintf(tmp,"<sip:%s@%s:%d>",msg->from->url->username,(String::IPtoString((*SIPCall::SIP_out)->l_ip)).c_str(),(*SIPCall::SIP_out)->l_port);
        	if (msg_setfrom(c->msg_out, tmp)<0){
                	free(tmp);
                	return(-1);
        	}
	}

	free(tmp);
	return(0);
}

int SIPParser::setFrom(sip_t  *msg, SIPCall *c, bool direction){
	sip_t   *msg_modifier;

        if ((c->SIP_in==NULL) || (c->SIP_out==NULL)) return(-1);

	if (!direction) msg_modifier=c->msg_out;
	else msg_modifier=c->msg_in;

	/* modify from */
        from_free(msg->from);
        free(msg->from);
        if(from_clone(msg_modifier->from, &(msg->from))<0) {
                return(-1);
        }
	
	return(0);
}

int SIPParser::storeTo(sip_t  *msg, SIPCall *c, bool direction){
	char *tmp;
	sip_t   *msg_modifier;

        if ((c->SIP_in==NULL) || (c->SIP_out==NULL)) return(-1);
        tmp=(char*)malloc(120*sizeof(char));

	if ((c->msg_in->to)==NULL) {
		/* set to in */
        	if(to_clone(msg->to, &(c->msg_in->to))<0) {
                	free(tmp);
                	return(-1);
        	}
	}
	if ((c->msg_out->to)==NULL) {
        	/* set to out */
        	if(to_clone(msg->to, &(c->msg_out->to))<0) {
                	free(tmp);
                	return(-1);
        	}
	}
	free(tmp);
	return(0);
}

int SIPParser::setTo(sip_t  *msg, SIPCall *c, bool direction){
	sip_t   *msg_modifier;

	if (!direction) msg_modifier=c->msg_out;
	else msg_modifier=c->msg_in;

	/* modify from */
        to_free(msg->to);
        free(msg->to);
        if(to_clone(msg_modifier->to, &(msg->to))<0) {
                return(-1);
        }
	
	return(0);
}

int SIPParser::setContact(sip_t  *msg, SIPCall *c, bool direction){
	char *tmp;
        int pos = 0;
	sip_t   *msg_modifier;

        if ((c->SIP_in==NULL) || (c->SIP_out==NULL)) return(-1);
        tmp=(char*)malloc(120*sizeof(char));

	if (!direction) msg_modifier=c->msg_out;
	else msg_modifier=c->msg_in;

	/* modify contacts */
        /* remove current contacts */
        pos = 0;
        while (!list_eol(msg->contacts,pos)) {
                contact_t *contact;
                contact = (contact_t *)list_get(msg->contacts,pos);
                list_remove(msg->contacts,pos);
                contact_free(contact);
                free(contact);
        }

	free(tmp);
	return(0);
}

bool SIPParser::checkMedia(char *mediatype,String logmessage){
        String check = type + "_ALLOW_CHANNEL_";
        if (String(mediatype)==String("audio")){
                check = check + "audioData";
                if ((Global::getGlobal()->getConf()->getValAsString(check))==String("YES")) {
                        LOG((__FILE__,__FUNCTION__,8,"%s request to open channel for audioData\n",logmessage.c_str()))
                        return true;
                }
                else LOG((__FILE__,__FUNCTION__,8,"%s request to open channel for audioData not allowed by config!\n",logmessage.c_str()))
        }
        else if (String(mediatype)==String("video")){ 
                check = check + "videoData";
                if ((Global::getGlobal()->getConf()->getValAsString(check))==String("YES")) {
                        LOG((__FILE__,__FUNCTION__,8,"%s request to open channel for audioData\n",logmessage.c_str()))
                        return true;
                }
                else LOG((__FILE__,__FUNCTION__,8,"%s request to open channel for videoData not allowed by config!\n",logmessage.c_str()))
        }
        else {
                WRN((__FILE__,__FUNCTION__,"%s unknown DataType requested: %s\n",logmessage.c_str(),mediatype))
        }
        return false;
}

int SIPParser::setMedia(sip_t  *msg, SIPCall *c, bool direction, String logmessage){
	sdp_t *sdp;
	body_t *body;
	int pos;
	SIPMediaChannel *mediaChannel;

	/* WE ASSUME IT IS A SDP BODY AND THAT    */
        /* IT IS THE ONLY ONE, OF COURSE, THIS IS */
        /* NOT TRUE */

	sdp_init(&sdp);
        body = (body_t *)list_get(msg->bodies,0);
	if (body==NULL) {
	        LOG((__FILE__,__FUNCTION__,8,"%sSDP: message does not contain an sdp body!\n",logmessage.c_str()))
		sdp_free(sdp);free(sdp);return(0);
	}
	if(sdp_parse(sdp,body->body)<0){
	        LOG((__FILE__,__FUNCTION__,8,"%sSDP: parse sdp body\n",logmessage.c_str()))
		sdp_free(sdp);free(sdp);return(-1);
	}

	LOG((__FILE__,__FUNCTION__,8,"%sSDP:o=: %s -> %s\n",logmessage.c_str(),sdp->o_addr,String::IPtoString((*(c->SIP_in))->l_ip).c_str()));
	free(sdp->o_addr);
	(sdp->o_addr)=(char*)malloc(20*sizeof(char));
	sprintf(sdp->o_addr,"%s",String::IPtoString((*(c->SIP_in))->l_ip).c_str());

	LOG((__FILE__,__FUNCTION__,8,"%sSDP:s=: %s -> %s\n",logmessage.c_str(),sdp->s_name,"call via KOMproxyd"))
	free(sdp->s_name);
	(sdp->s_name)=(char*)malloc(20*sizeof(char));
	sprintf(sdp->s_name,"%s","call via KOMproxyd");

        pos = 0;
        while (!list_eol(sdp->m_medias,pos)) {
		sdp_media_t *media;
		media = (sdp_media_t *)list_get(sdp->m_medias,pos);
		if (checkMedia(media->m_media,logmessage)){
		 if ((Global::getGlobal()->getConf()->getValAsInt(type+"_DATAFASTPATH"))==0){
			if(c->useMediaChannel(parent,(String(sdp_m_port_get(sdp,pos))).c_int(),direction)<0){
                        	WRN((__FILE__,__FUNCTION__,"%sSDP:m=: media channel not found\n",logmessage.c_str()))
                        	return(-1);
                	}
			mediaChannel=c->mediaChannel[c->getMediaChannel((String(sdp_m_port_get(sdp,pos))).c_int(),direction)];
			LOG((__FILE__,__FUNCTION__,8,"%sSDP:m=: RTP to %s:%s\n",logmessage.c_str(),sdp->c_connection->c_addr,sdp_m_port_get(sdp,pos)))
			(*(mediaChannel->RTP_out))->p_ip	= Connection::gethostip(sdp->c_connection->c_addr);
			(*(mediaChannel->RTP_out))->p_mask     = 0xffffffff;
			(*(mediaChannel->RTP_out))->p_port	= (String(sdp_m_port_get(sdp,pos))).c_int(); 
			(*(mediaChannel->RTCP_out))->p_ip	= Connection::gethostip(sdp->c_connection->c_addr);
			(*(mediaChannel->RTCP_out))->p_mask     = 0xffffffff;
			(*(mediaChannel->RTCP_out))->p_port	= ((String(sdp_m_port_get(sdp,pos))).c_int())+1; 

			free(media->m_port);
        		(media->m_port)=(char*)malloc(10*sizeof(char));
			sprintf(media->m_port,"%d",(*(mediaChannel->RTP_in))->l_port);

			free(sdp->c_connection->c_addr);
			(sdp->c_connection->c_addr)=(char*)malloc(20*sizeof(char));
			sprintf(sdp->c_connection->c_addr,"%s",String::IPtoString((*(mediaChannel->RTP_in))->l_ip).c_str());

			LOG((__FILE__,__FUNCTION__,8,"%sSDP:m=: RTP %s:%d -> %s:%d\n",logmessage.c_str(),
                        	String::IPtoString((*(mediaChannel->RTP_out))->p_ip).c_str(),(*(mediaChannel->RTP_out))->p_port,
                        	String::IPtoString((*(mediaChannel->RTP_in))->l_ip).c_str(),(*(mediaChannel->RTP_in))->l_port))

		 }
		 else {
			Fwfil *fwfilp;
			fwfilp=new Fwfil("RTP",17,0
						,0
						,FWFIL_PORTRANGE_GREATERT
						,1023
						,Connection::gethostip(sdp->c_connection->c_addr)
						,0xffffffff
						,FWFIL_PORTRANGE_EQUAL
						,(String(sdp_m_port_get(sdp,pos))).c_int()
			);
			if(c->fwhndl.getChannel(fwfilp)<0){
                        	WRN((__FILE__,__FUNCTION__,"%s could not configure firewall for connection\n",logmessage.c_str()))
                        }
			fwfilp=new Fwfil("RTCP",17,0
						,0
						,FWFIL_PORTRANGE_GREATERT
						,1023
						,Connection::gethostip(sdp->c_connection->c_addr)
						,0xffffffff
						,FWFIL_PORTRANGE_EQUAL
						,((String(sdp_m_port_get(sdp,pos))).c_int())+1
			);
			if(c->fwhndl.getChannel(fwfilp)<0){
                        	WRN((__FILE__,__FUNCTION__,"%s could not configure firewall for connection\n",logmessage.c_str()))
                        }
        	 }
		}
		else { /* media not allowed, remove it ! */
			free(media->m_port);
        		(media->m_port)=(char*)malloc(10*sizeof(char));
			sprintf(media->m_port,"0");

			free(sdp->c_connection->c_addr);
			(sdp->c_connection->c_addr)=(char*)malloc(20*sizeof(char));
			sprintf(sdp->c_connection->c_addr,"%s",String::IPtoString((*(c->SIP_in))->l_ip).c_str());
		}
                pos++;
	}
	
        pos = 0;
        while (!list_eol(msg->bodies,pos)) {
                body = (body_t *)list_get(msg->bodies,pos); 
                list_remove(msg->bodies,pos);
		body_free(body);
		free(body);
        }
	char *tmp=(char*)malloc(5000*sizeof(char));
	sdp_2char(sdp,&tmp);
	msg_setbody(msg,tmp);
		
	free(tmp);sdp_free(sdp);free(sdp);return(0);
}


int SIPParser::connectDestinationReplyAddressPDU(SIPCall *c){
	char *tmp;
        if ((c->SIP_in==NULL) || (c->SIP_out==NULL)) return(-1);
        tmp=(char*)malloc(120*sizeof(char));
	if((c->s_ip==0) && (c->s_port==0)){
		/* check via for reply address */
        	via_t *via;
        	if ((msg_getvia(c->msg_in,0,&via))<0){
                	free(tmp);
                	return(-1);
        	}
        	if (via->port!=NULL) c->s_port=(String((via->port))).c_int();
        	else c->s_port=5060;
        	if ((c->s_ip=Connection::gethostip(via->host))==0){
                	free(tmp);
                	return(-1);
        	}
		via_2char(via, &tmp);
                LOG((__FILE__,__FUNCTION__,8,"SIP: using %s:%d for response for via: %s\n",
                	(String::IPtoString(c->s_ip)).c_str(),c->s_port,tmp))
	}
	(*SIPCall::SIP_out)->p_ip 	= c->s_ip; 
	(*SIPCall::SIP_out)->p_mask     = 0xffffffff;
	(*SIPCall::SIP_out)->p_port     = c->s_port; 
	free(tmp);
	return(1);
}

int SIPParser::connectDestinationDestinationAddressPDU(SIPCall *c){
        String locator;
        String prefix;
        String requestIP;
        String requestPORT;
        String destination;

	if((c->d_ip==0) && (c->d_port==0)){
		LOG((__FILE__,__FUNCTION__,8,"SIP:LOCATOR:PDU:\n"))
		destination=String(c->msg_in->to->url->host);
		LOG((__FILE__,__FUNCTION__,8,"SIP:LOCATOR:PDU: to address = %s\n",destination.c_str()))
		for (int i=0;i<MAXLOCATORS;++i){
			locator = type + "_LOCATOR" + "_PDU" + "_DOMAIN" + "_" + i;
			prefix = Global::getGlobal()->getConf()->getValAsString(locator);
			if (prefix!=String("")) {
                        	LOG((__FILE__,__FUNCTION__,8,"SIP:LOCATOR:PDU: testing %s = %s for %s\n",locator.c_str(),prefix.c_str(),destination.c_str()))
                                if(prefix==destination){
					requestIP   = locator + "_IP";
                                	requestPORT = locator + "_PORT";
                                        if (((Global::getGlobal()->getConf()->getValAsString(requestIP))!=String("")) &&
                                                        ((Global::getGlobal()->getConf()->getValAsString(requestPORT))!=String(""))) {
							c->d_ip   = Global::getGlobal()->getConf()->getValAsIP(requestIP);
							c->d_port = Global::getGlobal()->getConf()->getValAsInt(requestPORT);
                                                        LOG((__FILE__,__FUNCTION__,8,"SIP:LOCATOR:PDU: using destination address %s:%d for to: %s\n",
                                                                (String::IPtoString(c->d_ip)).c_str(),
                                                                c->d_port,destination.c_str()))
							(*SIPCall::SIP_out)->p_ip        = c->d_ip; 
							(*SIPCall::SIP_out)->p_mask      = 0xffffffff;
							(*SIPCall::SIP_out)->p_port      = c->d_port; 
                                                        return(1);
                                        }
                                        else {
                                                        WRN((__FILE__,__FUNCTION__,"SIP:LOCATOR:PDU: configuration file incomplete: KEY=%s VAL=%s KEY=%s VAL=%s\n",
                                                                requestIP.c_str()  ,Global::getGlobal()->getConf()->getValAsString(requestIP).c_str(),
                                                                requestPORT.c_str(),Global::getGlobal()->getConf()->getValAsString(requestPORT).c_str()))
                                        }
                                }
			}
		}
                /* use default routing entry if present */
                locator = type + "_LOCATOR" + "_PDU" + "_DOMAIN" + "_DEFAULT";
                LOG((__FILE__,__FUNCTION__,8,"SIP:LOCATOR:PDU: using %s for %s\n",locator.c_str(),destination.c_str()))
                requestIP   = locator + "_IP";
                requestPORT = locator + "_PORT";
                if (((Global::getGlobal()->getConf()->getValAsString(requestIP))!=String("")) &&
                            ((Global::getGlobal()->getConf()->getValAsString(requestPORT))!=String(""))) {
					c->d_ip   = Global::getGlobal()->getConf()->getValAsIP(requestIP);
					c->d_port = Global::getGlobal()->getConf()->getValAsInt(requestPORT);
                                        LOG((__FILE__,__FUNCTION__,8,"SIP:LOCATOR:PDU: using destination address %s:%d for to: %s\n",
                                                        (String::IPtoString(c->d_ip)).c_str(),
                                                        c->d_port,destination.c_str()))
					(*SIPCall::SIP_out)->p_ip        = c->d_ip; 
					(*SIPCall::SIP_out)->p_mask      = 0xffffffff;
					(*SIPCall::SIP_out)->p_port      = c->d_port; 
                                        return(1);
                }
                else {
                	WRN((__FILE__,__FUNCTION__,"SIP:LOCATOR:PDU: default routing entry not found in configuration file: KEY=%s VAL=%s KEY=%s VAL=%s\n",
                            requestIP.c_str()  ,Global::getGlobal()->getConf()->getValAsString(requestIP).c_str(),
                            requestPORT.c_str(),Global::getGlobal()->getConf()->getValAsString(requestPORT).c_str()))
                }

	}
	else {
		(*SIPCall::SIP_out)->p_ip        = c->d_ip; 
		(*SIPCall::SIP_out)->p_mask      = 0xffffffff;
		(*SIPCall::SIP_out)->p_port      = c->d_port; 
		return(1);
	}
	return(-1);
}


SIPCall *SIPParser::addCall(sip_t  *msg){
        for(int i=0;i<MAXCALLS;++i){
                if (callList[i]!=NULL){
			if (SIPCall::getCallId(msg)==SIPCall::getCallId(callList[i]->msg_in)){
                		return(callList[i]);
			}
		}
        }
        for(int i=0;i<MAXCALLS;++i){
                if (callList[i]!=NULL){
			if (SIPCall::getCallId(msg)==SIPCall::getCallId(callList[i]->msg_out)){
                		return(callList[i]);
			}
		}
        }
	if (MSG_IS_INVITE(msg)) {
        	for(int i=0;i<MAXCALLS;++i){
                	if (callList[i]==NULL){
				callList[i]=new SIPCall();
				callList[i]->id=i;
                		LOG((__FILE__,__FUNCTION__,8,"SIP:CALL:ADD: intern_id:%d extern_id:%s\n",i,(SIPCall::getCallId(msg)).c_str()))
                        	++size;
                        	return(callList[i]);
                	}
        	}
	}
        WRN((__FILE__,__FUNCTION__,"SIP:CALL:ADD: could not add call to list: extern_id:%s\n",(SIPCall::getCallId(msg)).c_str()))
        return(NULL);
}

int SIPParser::delCall(SIPCall *c){
        for(int i=0;i<MAXCALLS;++i){
                if (callList[i]!=NULL){
                	if (callList[i]==c){
               			LOG((__FILE__,__FUNCTION__,8,"SIP:CALL:DEL: intern_id:%d extern_id:%s\n",i,(SIPCall::getCallId(callList[i]->msg_in)).c_str()))
                		delete (callList[i]);
                		callList[i]=NULL;
                		--size;
                		return(0);
			}
		}
	}
        WRN((__FILE__,__FUNCTION__,"SIP:CALL:DEL: could not find call in list: intern_id:%d\n",c->id));
	return(-1);
}

int SIPParser::delCall(sip_t  *msg){
        for(int i=0;i<MAXCALLS;++i){
                if (callList[i]!=NULL){
			if (SIPCall::getCallId(msg)==SIPCall::getCallId(callList[i]->msg_in)){
                		LOG((__FILE__,__FUNCTION__,8,"SIP:CALL:DEL: intern_id:%d extern_id:%s\n",i,(SIPCall::getCallId(msg)).c_str()))
                		delete (callList[i]);
                		callList[i]=NULL;
                		--size;
                		return(0);
			}
		}
        }
        for(int i=0;i<MAXCALLS;++i){
                if (callList[i]!=NULL){
			if (SIPCall::getCallId(msg)==SIPCall::getCallId(callList[i]->msg_out)){
                		LOG((__FILE__,__FUNCTION__,8,"SIP:CALL:DEL: intern_id:%d extern_id:%s\n",i,(SIPCall::getCallId(msg)).c_str()))
                		delete (callList[i]);
                		callList[i]=NULL;
                		--size;
                		return(0);
			}
		}
        }
        WRN((__FILE__,__FUNCTION__,"SIP:CALL:DEL: could not find call in list: %s\n",(SIPCall::getCallId(msg)).c_str()))
	return(-1);
}

int SIPParser::delCall(){
        for(int i=0;i<MAXCALLS;++i){
                if (callList[i]!=NULL) delCall(callList[i]);
	}
        return(0);
}


int SIPParser::connectDestinationSPECIAL(String locator){
        return(-1);
}
int SIPParser::connectDestinationFAST(){
        //return(connectDestination());
}

int SIPParser::write_0(Connection *dc, Connection *sc){
	LOG((__FILE__,__FUNCTION__,15,"SIP: write_0\n"))
	dc=sc;
        if(dc->connectionWrite(sc)<0) {
                WRN((__FILE__,__FUNCTION__,"SIP: write on socket\n"))
                return(0);
        }
	dc->p_ip=0;
	dc->p_mask=0;
	dc->p_port=0;
        return(sc->packet_len);
}
int SIPParser::write_1(Connection *dc, Connection *sc){
	LOG((__FILE__,__FUNCTION__,15,"SIP: write_1\n"))
	return(-1);
}
int SIPParser::write_2(Connection *dc, Connection *sc){
	LOG((__FILE__,__FUNCTION__,15,"SIP:RTP: write_2 --> write_default\n"))
        return(write_default(dc,sc));
}
int SIPParser::write_3(Connection *dc, Connection *sc){
	LOG((__FILE__,__FUNCTION__,15,"SIP:RTCP: write_3 --> write_default\n"))
        return(write_default(dc,sc));
}
int SIPParser::write_default(Connection *dc, Connection *sc){
	LOG((__FILE__,__FUNCTION__,15,"SIP: write_default\n"))
        if(dc->connectionWrite(sc)<0) {
                WRN((__FILE__,__FUNCTION__,"SIP: write on socket\n"))
                return(-1);
        }
        return(sc->packet_len);
}

int SIPParser::read_0(Connection *c){
        int     len=0;
	LOG((__FILE__,__FUNCTION__,15,"SIP: read_0\n"))
        c->zeroBuffer();
        if ((len=c->connectionRead(BUFFERSIZE-1))<0) return(-1);
        LOG((__FILE__,__FUNCTION__,15,"SIP: read_default, length: %d\n",len))
        return(c->packet_len);
}
int SIPParser::read_1(Connection *c){
	LOG((__FILE__,__FUNCTION__,15,"SIP: read_1\n"))
	return(-1);
}
int SIPParser::read_2(Connection *c){
	LOG((__FILE__,__FUNCTION__,15,"SIP:RTP: read_2\n"))
        return(read_default(c));
}
int SIPParser::read_3(Connection *c){
	LOG((__FILE__,__FUNCTION__,15,"SIP:RTCP: read_3\n"))
        return(read_default(c));
}
int SIPParser::read_default(Connection *c){
	int     len=0;
        LOG((__FILE__,__FUNCTION__,15,"SIP: read_default\n"))
        c->zeroBuffer();
        if ((len=c->connectionRead(BUFFERSIZE-1))<0) return(-1);
        LOG((__FILE__,__FUNCTION__,15,"SIP: read_default, length: %d\n",len))
        return(c->packet_len);
}

bool SIPParser::checkPDU(String pdutype){
	String check = type + "_ALLOW_" + pdutype;
	if ((Global::getGlobal()->getConf()->getValAsString(check))==String("YES")) return true; 
	WRN((__FILE__,__FUNCTION__,"SIP: drop pdu %s\n",pdutype.c_str()))
	return false;
}

int SIPParser::parse_00(Connection *c){
	LOG((__FILE__,__FUNCTION__,15,"SIP: parse_00\n"))

	LOG((__FILE__,__FUNCTION__,8,"SIP: PDU RECEIVED\n"))
	
	SIPCall   *call;

	sip_t  *msg;
	msg_init(&msg);
	char *sipstreamOUT;
	int direction;

	if((msg_parse(msg,c->buffer))!=0){
		WRN((__FILE__,__FUNCTION__,"SIP: error decoding sip message!\n"))
		msg_free(msg);
		free(msg);
		return(0);
	}

#ifdef _SIPDEBUG_
	char *sipdebug1;
        printf("\n===BEGIN SIP PDU DECODED ==============================\n");
	msg_force_update(msg);
	msg_2char(msg,&sipdebug1);
	printf("\n%s\n",sipdebug1);
	free(sipdebug1);
        printf("\n===END SIP PDU DECODED ================================\n");
#endif

	/* add call in list or get call from list if exists */
	if ((call=addCall(msg))==NULL){
               	WRN((__FILE__,__FUNCTION__,"SIP: could not handle request\n"))
		msg_free(msg);free(msg);return(0);
	}

	/* identify the direction of the message */
	if ((direction=getDirection(msg,call))<0){
               	WRN((__FILE__,__FUNCTION__,"SIP: could not identify direction\n"))
		msg_free(msg);free(msg);return(0);
	}



	/* set the destination of the packet */
	if (MSG_IS_REQUEST(msg))  {
		if ((storeCallid(msg,call,direction))<0){
               	WRN((__FILE__,__FUNCTION__,"SIP: store Call-ID\n"))
			delCall(call);
			msg_free(msg);free(msg);return(0);
		}
		if ((storeVia(msg,call,direction))<0){
               		WRN((__FILE__,__FUNCTION__,"SIP: store Via\n"))
			delCall(call);
			msg_free(msg);free(msg);return(0);
		}
		if ((storeFrom(msg,call,direction))<0){
               		WRN((__FILE__,__FUNCTION__,"SIP: store From\n"))
			delCall(call);
			msg_free(msg);free(msg);return(0);
		}
		if ((storeTo(msg,call,direction))<0){
               		WRN((__FILE__,__FUNCTION__,"SIP: store To\n"))
			delCall(call);
			msg_free(msg);free(msg);return(0);
		}
		connectDestinationDestinationAddressPDU(call);
		if ((storeMethod(msg,call,direction))<0){
               		WRN((__FILE__,__FUNCTION__,"SIP: store method\n"))
			delCall(call);
			msg_free(msg);free(msg);return(0);
		}
		if (MSG_IS_INVITE(msg)) {
	                LOG((__FILE__,__FUNCTION__,8,"SIP:INVITE PDU RECEIVED\n"))
	                if (!checkPDU("INVITE")){msg_free(msg);free(msg);return(0);}
			if ((setMedia(msg,call,direction,"SIP:INVITE:"))<0){
               			WRN((__FILE__,__FUNCTION__,"SIP:INVITE: set method\n"))
				delCall(call);
				msg_free(msg);free(msg);return(0);
			}
      	 	}
		else if (MSG_IS_ACK(msg)) {
        	        LOG((__FILE__,__FUNCTION__,8,"SIP:ACK PDU RECEIVED\n"))
               		 if (!checkPDU("ACK")){msg_free(msg);free(msg);return(0);}
        	}
		else if (MSG_IS_BYE(msg)) {
                	LOG((__FILE__,__FUNCTION__,8,"SIP:BYE PDU RECEIVED\n"))
                	if (!checkPDU("BYE")){msg_free(msg);free(msg);return(0);}
        	}
		else if (MSG_IS_REGISTER(msg)) {
                	LOG((__FILE__,__FUNCTION__,8,"SIP:REGISTER PDU RECEIVED\n"))
                	if (!checkPDU("REGISTER")){msg_free(msg);free(msg);return(0);}
        	}
		else if (MSG_IS_CANCEL(msg)) {
                	LOG((__FILE__,__FUNCTION__,8,"SIP:CANCEL PDU RECEIVED\n"))
                	if (!checkPDU("CANCEL")){msg_free(msg);free(msg);return(0);}
        	}
		else if (MSG_IS_OPTIONS(msg)) {
               	 	LOG((__FILE__,__FUNCTION__,8,"SIP:OPTIONS PDU RECEIVED\n"))
                	if (!checkPDU("OPTIONS")){msg_free(msg);free(msg);return(0);}
        	}
		else if (MSG_IS_INFO(msg)) {
                	LOG((__FILE__,__FUNCTION__,8,"SIP:INFO PDU RECEIVED\n"))
                	if (!checkPDU("INFO")){msg_free(msg);free(msg);return(0);}
        	}
		else if (MSG_IS_PRACK(msg)) {
                	LOG((__FILE__,__FUNCTION__,8,"SIP:PRACK PDU RECEIVED\n"))
                	if (!checkPDU("PRACK")){msg_free(msg);free(msg);return(0);}
        	}
		else{
                	WRN((__FILE__,__FUNCTION__,"SIP:UNKNOWN PDU RECEIVED\n"))
                	msg_free(msg);free(msg);return(0);
        	}
		if ((setMethod(msg,call,direction))<0){
               		WRN((__FILE__,__FUNCTION__,"SIP: set method\n"))
			delCall(call);
			msg_free(msg);free(msg);return(0);
		}
	}
	if (MSG_IS_RESPONSE(msg)) {
		connectDestinationReplyAddressPDU(call);

		if (MSG_IS_STATUS_1XX(msg)) {
	                LOG((__FILE__,__FUNCTION__,8,"SIP:STATUS_1XX PDU RECEIVED\n"))
	                if (!checkPDU("STATUS_1XX")){msg_free(msg);free(msg);return(0);}
      	 	}
		else if (MSG_IS_STATUS_2XX(msg)) {
                	LOG((__FILE__,__FUNCTION__,8,"SIP:STATUS_2XX PDU RECEIVED\n"))
                	if (!checkPDU("STATUS_2XX")){msg_free(msg);free(msg);return(0);}
			if(MSG_IS_RESPONSEFOR(msg,"INVITE")){
				if ((setMedia(msg,call,direction,"SIP:STATUS_2XX:"))<0){
               				WRN((__FILE__,__FUNCTION__,"SIP:STATUS_2XX: set method\n"))
					delCall(call);
					msg_free(msg);free(msg);return(0);
				}
			}
			else if(MSG_IS_RESPONSEFOR(msg,"BYE")){
               			LOG((__FILE__,__FUNCTION__,8,"SIP:STATUS_2XX: ack for BYE received, marking call for termination\n"))
				call->callEnd=true;
			}
        	}
		else if (MSG_IS_STATUS_3XX(msg)) {
                	LOG((__FILE__,__FUNCTION__,8,"SIP:STATUS_3XX PDU RECEIVED\n"))
                	if (!checkPDU("STATUS_3XX")){msg_free(msg);free(msg);return(0);}
        	}
		else if (MSG_IS_STATUS_4XX(msg)) {
                	LOG((__FILE__,__FUNCTION__,8,"SIP:STATUS_4XX PDU RECEIVED\n"))
                	if (!checkPDU("STATUS_4XX")){msg_free(msg);free(msg);return(0);}
        	}
		else if (MSG_IS_STATUS_5XX(msg)) {
                	LOG((__FILE__,__FUNCTION__,8,"SIP:STATUS_5XX PDU RECEIVED\n"))
                	if (!checkPDU("STATUS_5XX")){msg_free(msg);free(msg);return(0);}
        	}
		else if (MSG_IS_STATUS_6XX(msg)) {
                	LOG((__FILE__,__FUNCTION__,8,"SIP:STATUS_6XX PDU RECEIVED\n"))
                	if (!checkPDU("STATUS_6XX")){msg_free(msg);free(msg);return(0);}
        	}
		else{
                	WRN((__FILE__,__FUNCTION__,"SIP:UNKNOWN PDU RECEIVED\n"))
                	msg_free(msg);free(msg);return(0);
        	}
	}

	if ((setCallid(msg,call,direction))<0){
               	WRN((__FILE__,__FUNCTION__,"SIP: set Call-ID\n"))
		delCall(call);
		msg_free(msg);free(msg);return(0);
	}
	if ((setVia(msg,call,direction))<0){
               	WRN((__FILE__,__FUNCTION__,"SIP: set Via\n"))
		delCall(call);
		msg_free(msg);free(msg);return(0);
	}
	if ((setFrom(msg,call,direction))<0){
               	WRN((__FILE__,__FUNCTION__,"SIP: set From\n"))
		delCall(call);
		msg_free(msg);free(msg);return(0);
	}
	if ((setTo(msg,call,direction))<0){
               	WRN((__FILE__,__FUNCTION__,"SIP: set To\n"))
		delCall(call);
		msg_free(msg);free(msg);return(0);
	}
	if ((setContact(msg,call,direction))<0){
               	WRN((__FILE__,__FUNCTION__,"SIP: set Contact\n"))
		delCall(call);
		msg_free(msg);free(msg);return(0);
	}

#ifdef _SIPDEBUG_
	char *sipdebug2;
        printf("\n===BEGIN SIP PDU MODIFIED =============================\n");
	msg_force_update(msg);
	msg_2char(msg,&sipdebug2);
	printf("\n%s\n",sipdebug2);
	free(sipdebug2);
        printf("\n===END SIP PDU MODIFIED ===============================\n");
#endif
	msg_force_update(msg);
	msg_2char(msg,&sipstreamOUT);
        c->packet_len=strlen(sipstreamOUT);
	if (c->packet_len>=BUFFERSIZE){
		c->packet_len=0;
                WRN((__FILE__,__FUNCTION__,"SIP: packet too long\n"))
		msg_free(msg); free(msg); free(sipstreamOUT);return(0);
	}
	strncpy( c->buffer, sipstreamOUT,c->packet_len);

        if(((*SIPCall::SIP_out)->p_ip==0) || ((*SIPCall::SIP_out)->p_port==0) ){
                WRN((__FILE__,__FUNCTION__,"SIP: the destination was not specified!\n"))
		free(sipstreamOUT); msg_free(msg); free(msg); return(0);
        }

	free(sipstreamOUT); msg_free(msg); free(msg); 

	if(call->callEnd){
		LOG((__FILE__,__FUNCTION__,8,"SIP: terminating call\n"))
		delCall(call);
	}
	return(1);
}

int SIPParser::parse_10(Connection *c){
	LOG((__FILE__,__FUNCTION__,15,"SIP: parse_10\n"))
	return(-1);
}
int SIPParser::parse_20(Connection *c){
	LOG((__FILE__,__FUNCTION__,15,"SIP:RTP: parse_20\n"))
        return(1);
}
int SIPParser::parse_30(Connection *c){
	LOG((__FILE__,__FUNCTION__,15,"SIP:RTCP: parse_30\n"))
        return(1);
}
int SIPParser::parse_default0(Connection *c){
	LOG((__FILE__,__FUNCTION__,15,"SIP: parse_default0\n"))
	return(-1);
}

int SIPParser::parse_01(Connection *c){
	LOG((__FILE__,__FUNCTION__,15,"SIP: parse_01\n"))
	return(-1);
}

int SIPParser::parse_11(Connection *c){
	LOG((__FILE__,__FUNCTION__,15,"SIP: parse_11\n"))
	return(-1);
}
int SIPParser::parse_21(Connection *c){
	LOG((__FILE__,__FUNCTION__,15,"SIP:RTP: parse_21\n"))
        return(1);
}
int SIPParser::parse_31(Connection *c){
	LOG((__FILE__,__FUNCTION__,15,"SIP:RTCP: parse_31\n"))
        return(1);
}
int SIPParser::parse_default1(Connection *c){
	LOG((__FILE__,__FUNCTION__,15,"SIP: parse_default1\n"))
	return(-1);
}


