/* Copyright (C) 2000 KOM/Darmstadt University of Technology
 *
 * You are allowed to use all other parts of the code under the following terms:
 *
 * For non-commercial use, code may be used in unmodified form provided
 * that this copyright notice and this permission notice appear in
 * supporting documentation.
 *
 * This software is provided "as is" and without any express or implied
 * warranties, including, without limitation, the implied warranty of
 * fitness for a particular purpose.
 *
 * The code may be subjected to the GNU General Public License, Version 2,
 * and re-distributed under the terms of this license.
 *
 * Commercial use other than under the terms of the GNU General Public
 * License is allowed only after express negotiation of conditions
 * with the authors.
 */
#ifndef _SIPParser_h_
#define _SIPParser_h_ 1

#include "../Global.h"
#include "../Parser.h"
#include "../Connection.h"
#include "../String.h"

#include "SIPCall.h"
#include "SIPMediaChannel.h"

#define _SIPDEBUG_ 1

#define MAXCALLS	MAXCONNECTIONS/4

class SIPParser : public Parser {
	static int KOMproxydCallId;

	SIPCall *callList[MAXCALLS];	
	int 	size;

	SIPCall *addCall(sip_t  *msg);
	int 	delCall(sip_t  *msg);
	int 	delCall(SIPCall *c);
	int 	delCall();

	bool checkPDU(String pdutype);
	bool checkMedia(char *mediatype,String logmessage);


	int storeMethod(sip_t  *msg, SIPCall *c,bool direction);
	int storeCallid(sip_t  *msg, SIPCall *c,bool direction);
	int storeVia(sip_t  *msg, SIPCall *c,bool direction);
	int storeFrom(sip_t  *msg, SIPCall *c,bool direction);
	int storeTo(sip_t  *msg, SIPCall *c,bool direction);

	int setMedia(sip_t  *msg, SIPCall *c, bool direction, String logmessage);
	int setMethod(sip_t  *msg, SIPCall *c,bool direction);
	int setCallid(sip_t  *msg, SIPCall *c,bool direction);
	int setVia(sip_t  *msg, SIPCall *c,bool direction);
	int setFrom(sip_t  *msg, SIPCall *c,bool direction);
	int setTo(sip_t  *msg, SIPCall *c,bool direction);
	int setContact(sip_t  *msg, SIPCall *c,bool direction);

	int connectDestinationReplyAddressPDU(SIPCall *call);
	int connectDestinationDestinationAddressPDU(SIPCall *call);

	int getDirection(sip_t  *msg, SIPCall *c);

public:
	SIPParser(Session* parent_t,String type_t);
	~SIPParser();
	
        int connectDestinationSPECIAL(String locator);
        int connectDestinationFAST();

	int read_0(Connection *c);
	int read_1(Connection *c);
	int read_2(Connection *c);
	int read_3(Connection *c);
	int read_default(Connection *c);

	int write_0(Connection *dc, Connection *sc);
	int write_1(Connection *dc, Connection *sc);
	int write_2(Connection *dc, Connection *sc);
	int write_3(Connection *dc, Connection *sc);
	int write_default(Connection *dc, Connection *sc);
	
	int parse_00(Connection *c);
	int parse_10(Connection *c);
	int parse_20(Connection *c);
	int parse_30(Connection *c);
	int parse_default0(Connection *c);
	int parse_01(Connection *c);
	int parse_11(Connection *c);
	int parse_21(Connection *c);
	int parse_31(Connection *c);
	int parse_default1(Connection *c);

};

#ifndef STATIC_PROXY
extern "C" {
Parser *maker(Session* parent_t,String type_t){
   return new SIPParser(parent_t,type_t);
}
}
#endif


#endif
