/* Copyright (C) 2000 KOM/Darmstadt University of Technology
 *
 * You are allowed to use all other parts of the code under the following terms:
 *
 * For non-commercial use, code may be used in unmodified form provided
 * that this copyright notice and this permission notice appear in
 * supporting documentation.
 *
 * This software is provided "as is" and without any express or implied
 * warranties, including, without limitation, the implied warranty of
 * fitness for a particular purpose.
 *
 * The code may be subjected to the GNU General Public License, Version 2,
 * and re-distributed under the terms of this license.
 *
 * Commercial use other than under the terms of the GNU General Public
 * License is allowed only after express negotiation of conditions
 * with the authors.
 */





#include "TestParser.h"
#include "../Session.h"
#include "../Log.h"
#include "../Conf.h"
#include "../FirewallControl.h"
#include "../Fwhndl.h"

#include <netinet/in.h> // htonl,  htons,  ntohl, ntohs
#include <sys/socket.h> // inet_ntoa
#include <arpa/inet.h>  // inet_ntoa


TestParser::TestParser(Session* parent_t,String type_t):play1(false),play2(false) {
	LOG((__FILE__,__FUNCTION__,15,"creating parser\n"))
	type=type_t;
	parent=parent_t;
}

TestParser::~TestParser() {
	LOG((__FILE__,__FUNCTION__,15,"deleting parser\n"))
}
int TestParser::connectDestinationSPECIAL(String locator){
        return(-1);
}
int TestParser::connectDestinationFAST(){
        return(connectDestination());
}


int TestParser::write_0(Connection *dc, Connection *sc){
        LOG((__FILE__,__FUNCTION__,15,"write_0 --> write_default\n"))
        return(write_default(dc,sc));
}
int TestParser::write_1(Connection *dc, Connection *sc){
        LOG((__FILE__,__FUNCTION__,15,"write_1 --> write_default\n"))
        return(write_default(dc,sc));
}
int TestParser::write_2(Connection *dc, Connection *sc){
        LOG((__FILE__,__FUNCTION__,15,"write_2\n"))
	return(-1);
}
int TestParser::write_3(Connection *dc, Connection *sc){
        LOG((__FILE__,__FUNCTION__,15,"write_3\n"))
	return(-1);
}
int TestParser::write_default(Connection *dc, Connection *sc){
        LOG((__FILE__,__FUNCTION__,15,"write_default\n"))
	if(dc->connectionWrite(sc)<0) {
                WRN((__FILE__,__FUNCTION__,"write on socket\n"))
                return(-1);
        }
        return(sc->packet_len);
}

int TestParser::read_0(Connection *c){
	LOG((__FILE__,__FUNCTION__,15,"read_0 --> read_default\n"))
        return read_default(c);
}
int TestParser::read_1(Connection *c){
	LOG((__FILE__,__FUNCTION__,15,"read_1 --> read_default\n"))
        return read_default(c);
}
int TestParser::read_2(Connection *c){
        LOG((__FILE__,__FUNCTION__,15,"read_2\n"))
	return(-1);
}
int TestParser::read_3(Connection *c){
        LOG((__FILE__,__FUNCTION__,15,"read_3\n"))
	return(-1);
}

int TestParser::read_default(Connection *c){
        int     len=0;
        LOG((__FILE__,__FUNCTION__,15,"read_default\n"))
        c->zeroBuffer();
        if ((len=c->connectionRead(4))<4) {
			if (len==0) return(0);
			return(-1);
	}
        len=(ntohl(*(((int *)(c->buffer))))-1)*4;
        if ((len=c->connectionRead(len))<=0) return(len);
        LOG((__FILE__,__FUNCTION__,15,"read_default, length: %d\n",len))
        return(c->packet_len);
}

int TestParser::parse_00(Connection *c){
	int plen,ptyp,pack,pseq;
	Fwfil *fwfilp;
	int       srchost, srcmask, srcport, dsthost, dstmask, dstport;

	double a=1214235;
	double b=1232342;
	double x=1;

        LOG((__FILE__,__FUNCTION__,15,"parse_00\n"))

	plen=ntohl(*(((int *)(c->buffer))+0));
        ptyp=ntohl(*(((int *)(c->buffer))+1));
        pseq=ntohl(*(((int *)(c->buffer))+2));
        pack=ntohl(*(((int *)(c->buffer))+3));
	
	switch (ptyp) {
		case 101:
			LOG((__FILE__,__FUNCTION__,8,"HELO received len=%d typ=%d seq=%d ack=%d\n",plen,ptyp,pseq,pack))
			for(int i=1;i<0;i++){
				x=x*a*b;
			}
        		return(1);
			break;
		case 102:
			LOG((__FILE__,__FUNCTION__,8,"PLAY received len=%d typ=%d seq=%d ack=%d\n",plen,ptyp,pseq,pack))

			if ((Global::getGlobal()->getConf()->getValAsInt(type+"_DATAFASTPATH"))==0){
				if (play1) LOG((__FILE__,__FUNCTION__,8,"SECOND PLAY\n"))
				else{
					parent->con[2][1]->d_name     = String("TEST-MEDIA-2-DST");
					parent->con[2][1]->l_ip       = Global::getGlobal()->getConf()->getValAsIP("LOCALIP"); 
					parent->con[2][1]->l_mask     = 0xffffffff;
					parent->con[2][1]->l_port     = -1; 
					parent->con[2][1]->p_ip       = ntohl(*(((int *)(c->buffer))+4)); 
					parent->con[2][1]->p_mask     = 0xffffffff;
					parent->con[2][1]->p_port     = ntohl(*(((int *)(c->buffer))+5));
					parent->con[2][1]->ipproto    = 17; 
					parent->con[2][1]->direction  = 0;
					parent->con[2][1]->d_number   = 1;

					parent->con[2][0]->d_name     = String("TEST-MEDIA-2-SRC");
					parent->con[2][0]->l_ip       = Global::getGlobal()->getConf()->getValAsIP("LOCALIP"); 
					parent->con[2][0]->l_mask     = 0xffffffff;
					parent->con[2][0]->l_port     = -1; 
					parent->con[2][0]->p_ip       = 0; 
					parent->con[2][0]->p_mask     = 0; 
					parent->con[2][0]->p_port     = -1;
					parent->con[2][0]->ipproto    = 17; 
					parent->con[2][0]->direction  = 1;
					parent->con[2][0]->d_number   = 1;

					if(parent->con[2][0]->connectionListen()<0) {
						ERR((__FILE__,__FUNCTION__,"listen on MEDIA-2\n"))
                                		return(-1);
                        		}

					LOG((__FILE__,__FUNCTION__,8,"MODIFY PLAY: %s:%d -> %s:%d\n",String::IPtoString(parent->con[2][1]->p_ip).c_str(),parent->con[2][1]->p_port,String::IPtoString(parent->con[2][0]->l_ip).c_str(),parent->con[2][0]->l_port))
					*(((int *)(c->buffer))+4) = htonl(parent->con[2][0]->l_ip); // IPDESTADDRESS
			        	*(((int *)(c->buffer))+5) = htonl(parent->con[2][0]->l_port); // IPDESTPORT
				}
			}
			else {
				if (play1) LOG((__FILE__,__FUNCTION__,8,"SECOND PLAY\n"))
				else{
					srcport = 1023;
                			srchost = parent->con[0][1]->p_ip;
                			srcmask = parent->con[0][1]->p_mask;

               				dstport = ntohl(*(((int *)(c->buffer))+5));
                			dsthost = ntohl(*(((int *)(c->buffer))+4));
					dstmask = 0xFFFFFFFF;

                			fwfilp = new Fwfil("TEST-MEDIA-2",17,srchost,srcmask,FWFIL_PORTRANGE_GREATERT,srcport,dsthost,dstmask,FWFIL_PORTRANGE_EQUAL,dstport);
                			if(fwhndl.getChannel(fwfilp)<0){
                        			ERR((__FILE__,__FUNCTION__,"could not configure firewall for incomming connection\n"))
                        			return(-1);
                			}
                			fwfilp = new Fwfil("TEST-MEDIA-2",17,dsthost,dstmask,FWFIL_PORTRANGE_EQUAL,dstport,srchost,srcmask,FWFIL_PORTRANGE_GREATERT,srcport);
                			if(fwhndl.getChannel(fwfilp)<0){
                        			ERR((__FILE__,__FUNCTION__,"could not configure firewall for incomming connection\n"))
                        			return(-1);
                			}
				}
			}
			play1=true;
        		return(1);
			break;
		case 103:
			LOG((__FILE__,__FUNCTION__,8,"ACK  received len=%d typ=%d seq=%d ack=%d\n",plen,ptyp,pseq,pack))
        		return(1);
			break;
		case 104:
			LOG((__FILE__,__FUNCTION__,8,"STOP received len=%d typ=%d seq=%d ack=%d\n",plen,ptyp,pseq,pack))
        		return(1);
			break;
		default:
			LOG((__FILE__,__FUNCTION__,8,"UNKNOWN packet received\n"))
			break;
	}
        return(-1);
}

int TestParser::parse_10(Connection *c){
	int plen,ptyp,pack,pseq;
        LOG((__FILE__,__FUNCTION__,15,"parse_10\n"))

	plen=ntohl(*(((int *)(c->buffer))+0));
        ptyp=ntohl(*(((int *)(c->buffer))+1));
        pseq=ntohl(*(((int *)(c->buffer))+2));
        pack=ntohl(*(((int *)(c->buffer))+3));
	
	switch (ptyp) {
		case 501:
			LOG((__FILE__,__FUNCTION__,8,"TEST received len=%d typ=%d seq=%d ack=%d\n",plen,ptyp,pseq,pack))
        		return(1);
			break;
		case 502:
			LOG((__FILE__,__FUNCTION__,8,"DATA received len=%d typ=%d seq=%d ack=%d\n",plen,ptyp,pseq,pack))
        		return(1);
			break;
		default:
			LOG((__FILE__,__FUNCTION__,8,"UNKNOWN packet received\n"))
			break;
	}
        return(-1);
}
int TestParser::parse_20(Connection *c){
        LOG((__FILE__,__FUNCTION__,15,"parse_20\n"))
	return(1);
}
int TestParser::parse_30(Connection *c){
        LOG((__FILE__,__FUNCTION__,15,"parse_30\n"))
	return(1);
}
int TestParser::parse_default0(Connection *c){
        LOG((__FILE__,__FUNCTION__,15,"parse_default0\n"))
	return(1);
}
int TestParser::parse_01(Connection *c){
	int plen,ptyp,pack,pseq;
	Fwfil *fwfilp;
	int       srchost, srcmask, srcport, dsthost, dstmask, dstport;
	double a=1214235;
	double b=1232342;
	double x=1;
        LOG((__FILE__,__FUNCTION__,15,"parse_01\n"))

	plen=ntohl(*(((int *)(c->buffer))+0));
        ptyp=ntohl(*(((int *)(c->buffer))+1));
        pseq=ntohl(*(((int *)(c->buffer))+2));
        pack=ntohl(*(((int *)(c->buffer))+3));
	
	switch (ptyp) {
		case 101:
			LOG((__FILE__,__FUNCTION__,8,"HELO received len=%d typ=%d seq=%d ack=%d\n",plen,ptyp,pseq,pack))
			for(int i=1;i<0;i++){
				x=x*a*b;
			}
        		return(1);
			break;
		case 102:
			LOG((__FILE__,__FUNCTION__,8,"PLAY received len=%d typ=%d seq=%d ack=%d\n",plen,ptyp,pseq,pack))
			if ((Global::getGlobal()->getConf()->getValAsInt(type+"_DATAFASTPATH"))==0){
				if (play2) LOG((__FILE__,__FUNCTION__,8,"SECOND PLAY\n"))
				else{
					parent->con[1][1]->d_name     = String("TEST-MEDIA-1-DST");
					parent->con[1][1]->l_ip       = Global::getGlobal()->getConf()->getValAsIP("LOCALIP"); 
					parent->con[1][1]->l_mask     = 0xffffffff;
					parent->con[1][1]->l_port     = -1; 
					parent->con[1][1]->p_ip       = ntohl(*(((int *)(c->buffer))+4)); 
					parent->con[1][1]->p_mask     = 0xffffffff;
					parent->con[1][1]->p_port     = ntohl(*(((int *)(c->buffer))+5));
					parent->con[1][1]->ipproto    = 17;
					parent->con[1][1]->direction  = 0;
					parent->con[1][1]->d_number   = 1;

					parent->con[1][0]->d_name     = String("TEST-MEDIA-1-SRC");
					parent->con[1][0]->l_ip       = Global::getGlobal()->getConf()->getValAsIP("LOCALIP"); 
					parent->con[1][0]->l_mask     = 0xffffffff;
					parent->con[1][0]->l_port     = -1; 
					parent->con[1][0]->p_ip       = 0; 
					parent->con[1][0]->p_mask     = 0; 
					parent->con[1][0]->p_port     = -1;
					parent->con[1][0]->ipproto    = 17; 
					parent->con[1][0]->direction  = 1;
					parent->con[1][0]->d_number   = 1;

					if(parent->con[1][0]->connectionListen()<0) {
						ERR((__FILE__,__FUNCTION__,"listen on MEDIA-1\n"))
                                		return(-1);
                        		}
					LOG((__FILE__,__FUNCTION__,8,"MODIFY PLAY: %s:%d -> %s:%d\n",String::IPtoString(parent->con[1][1]->p_ip).c_str(),parent->con[1][1]->p_port,String::IPtoString(parent->con[1][0]->l_ip).c_str(),parent->con[1][0]->l_port))
					*(((int *)(c->buffer))+4) = htonl(parent->con[1][0]->l_ip); // IPDESTADDRESS
			        	*(((int *)(c->buffer))+5) = htonl(parent->con[1][0]->l_port); // IPDESTPORT
				}
			}
			else {
				if (play2) LOG((__FILE__,__FUNCTION__,8,"SECOND PLAY\n"))
				else{
					srcport = 1023;
                			srchost = parent->con[0][0]->p_ip;
                			srcmask = parent->con[0][0]->p_mask;

               				dstport = ntohl(*(((int *)(c->buffer))+5));
                			dsthost = ntohl(*(((int *)(c->buffer))+4));
					dstmask = 0xFFFFFFFF;

                			fwfilp = new Fwfil("TEST-MEDIA-1",17,srchost,srcmask,FWFIL_PORTRANGE_GREATERT,srcport,dsthost,dstmask,FWFIL_PORTRANGE_EQUAL,dstport);
                			if(fwhndl.getChannel(fwfilp)<0){
                        			ERR((__FILE__,__FUNCTION__,"could not configure firewall for incomming connection\n"))
                        			return(-1);
                			}
                			fwfilp = new Fwfil("TEST-MEDIA-1",17,dsthost,dstmask,FWFIL_PORTRANGE_EQUAL,dstport,srchost,srcmask,FWFIL_PORTRANGE_GREATERT,srcport);
                			if(fwhndl.getChannel(fwfilp)<0){
                        			ERR((__FILE__,__FUNCTION__,"could not configure firewall for incomming connection\n"))
                        			return(-1);
                			}
				}
			}
			play2=true;
        		return(1);
			break;
		case 103:
			LOG((__FILE__,__FUNCTION__,8,"ACK  received len=%d typ=%d seq=%d ack=%d\n",plen,ptyp,pseq,pack))
        		return(1);
			break;
		case 104:
			LOG((__FILE__,__FUNCTION__,8,"STOP received len=%d typ=%d seq=%d ack=%d\n",plen,ptyp,pseq,pack))
        		return(1);
			break;
		default:
			LOG((__FILE__,__FUNCTION__,8,"UNKNOWN packet received\n"))
			break;
	}
        return(-1);
}
int TestParser::parse_11(Connection *c){
	int plen,ptyp,pack,pseq;
        LOG((__FILE__,__FUNCTION__,15,"parse_11\n"))

	plen=ntohl(*(((int *)(c->buffer))+0));
        ptyp=ntohl(*(((int *)(c->buffer))+1));
        pseq=ntohl(*(((int *)(c->buffer))+2));
        pack=ntohl(*(((int *)(c->buffer))+3));
	
	switch (ptyp) {
		case 501:
			LOG((__FILE__,__FUNCTION__,8,"TEST received len=%d typ=%d seq=%d ack=%d\n",plen,ptyp,pseq,pack))
        		return(1);
			break;
		case 502:
			LOG((__FILE__,__FUNCTION__,8,"DATA received len=%d typ=%d seq=%d ack=%d\n",plen,ptyp,pseq,pack))
        		return(1);
			break;
		default:
			LOG((__FILE__,__FUNCTION__,8,"UNKNOWN packet received\n"))
			break;
	}
        return(-1);
}
int TestParser::parse_21(Connection *c){
        LOG((__FILE__,__FUNCTION__,15,"parse_21\n"))
	return(1);
}
int TestParser::parse_31(Connection *c){
        LOG((__FILE__,__FUNCTION__,15,"parse_31\n"))
	return(1);
}
int TestParser::parse_default1(Connection *c){
        LOG((__FILE__,__FUNCTION__,15,"parse_default1\n"))
	return(1);
}

